@charset "UTF-8";
.timeline-axis {
  border-right-color: #454f55 !important; }
  .timeline-axis::before {
    background-color: #454f55 !important; }

.sidebar .server-name, .dashboard-navigation-tabs .sortable li div span, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div span, .dashboard-grid-widget-header h4, .dashboard-grid-iterator-header h4, .list-table .action-container .link-action, div.dashboard-widget-map .btn-back-map-container a:focus .btn-back-map .btn-back-map-content, div.dashboard-widget-map .btn-back-map-container a:hover .btn-back-map .btn-back-map-content, form.dashboard-widget-tophosts #list_columns .text, .menu-popup .menu-popup-item, .menu-popup h3, .notif-body h4, .overlay-descr-url, .tag, .overflow-ellipsis, .overflow-ellipsis td:not(.wordbreak, .wordwrap), .overflow-ellipsis th, .overflow-ellipsis th a, .preprocessing-list .preprocessing-list-head .step-on-fail, .totals-list.totals-list-horizontal .count,
.totals-list.totals-list-horizontal .name {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap; }

html, body, div, span, applet, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, a, abbr, acronym, address, big, cite, code, del, dfn, em, img, ins, kbd, q, s, samp, small, strike, strong, sub, sup, tt, var,
b,
strong, u, i, center, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td, article, aside, canvas, details, embed, figure, figcaption, footer, header, hgroup, menu, nav, output, ruby, section, summary,
time, mark, audio, video, textarea {
  margin: 0;
  padding: 0;
  border: 0;
  font-size: 100%;
  font: inherit;
  vertical-align: baseline; }

article, aside, details, figcaption, figure, footer, header, hgroup, menu, nav, section {
  display: block; }

body {
  line-height: 1; }

ol, ul {
  list-style: none; }

blockquote, q {
  quotes: none; }

blockquote::before, blockquote::after, q::before, q::after {
  content: '';
  content: none; }

table {
  border-collapse: collapse;
  border-spacing: 0; }

svg:focus,
svg *:focus {
  outline: none; }

@font-face {
  font-family: "zabbix-icons";
  src: url("../fonts/zabbix-icons.woff2?t=1715610469019") format("woff2"), url("../fonts/zabbix-icons.woff?t=1715610469019") format("woff"), url("../fonts/zabbix-icons.ttf?t=1715610469019") format("truetype"), url("../fonts/zabbix-icons.svg?t=1715610469019#zabbix-icons") format("svg"); }
[class^="zi-"]::before, [class*=" zi-"]::before, .btn-overlay-close::before, .color-picker .use-transparent::after,
.color-picker-dialogue .use-transparent::after, .dashboard-widget-placeholder .dashboard-widget-placeholder-label::before, .menu-main .has-submenu > a::after, .menu-user .has-submenu > a::after, output::before, .z-select .focusable::after,
z-select .focusable::after, .link-external::after, .drag-icon::before, .btn-dropdown-container .btn-dropdown-toggle::after {
  font-family: 'zabbix-icons';
  font-size: 18px;
  line-height: 1;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale; }

button[class^="zi-"], [class^="zi-"].btn, [class^="zi-"].btn-alt, [class^="zi-"].btn-grey, [class^="zi-"].btn-icon, [class^="zi-"].btn-link, [class^="zi-"].btn-overlay-close, [class^="zi-"].btn-debug, button[class*=" zi-"], [class*=" zi-"].btn, [class*=" zi-"].btn-alt, [class*=" zi-"].btn-grey, [class*=" zi-"].btn-icon, [class*=" zi-"].btn-link, [class*=" zi-"].btn-overlay-close, [class*=" zi-"].btn-debug, .btn-overlay-close, .icon, .color-picker .use-transparent::after,
.color-picker-dialogue .use-transparent::after, .drag-icon {
  position: relative;
  display: inline-flex;
  align-items: center;
  vertical-align: top; }
  button[class^="zi-"]:empty, [class^="zi-"].btn:empty, [class^="zi-"].btn-alt:empty, [class^="zi-"].btn-grey:empty, [class^="zi-"].btn-icon:empty, [class^="zi-"].btn-link:empty, [class^="zi-"].btn-debug:empty, button[class*=" zi-"]:empty, [class*=" zi-"].btn:empty, [class*=" zi-"].btn-alt:empty, [class*=" zi-"].btn-grey:empty, [class*=" zi-"].btn-icon:empty, [class*=" zi-"].btn-link:empty, [class*=" zi-"].btn-debug:empty, .btn-overlay-close:empty, .icon:empty, .color-picker .use-transparent:empty::after,
  .color-picker-dialogue .use-transparent:empty::after, .drag-icon:empty {
    justify-content: center; }

.zi-administration:before {
  content: "\ea01"; }

.zi-alert-check:before {
  content: "\ea02"; }

.zi-alert-more:before {
  content: "\ea03"; }

.zi-alert-x:before {
  content: "\ea04"; }

.zi-alert:before, .zi-alert-with-content:before {
  content: "\ea05"; }

.zi-alerts:before {
  content: "\ea06"; }

.zi-arrow-back:before {
  content: "\ea07"; }

.zi-arrow-down:before, .zi-arrow-down-small:before {
  content: "\ea08"; }

.zi-arrow-forward:before {
  content: "\ea09"; }

.zi-arrow-left:before {
  content: "\ea0a"; }

.zi-arrow-right-top:before {
  content: "\ea0b"; }

.zi-arrow-right:before {
  content: "\ea0c"; }

.zi-arrow-top-right:before {
  content: "\ea0d"; }

.zi-arrow-up:before, .zi-arrow-up-small:before {
  content: "\ea0e"; }

.zi-arrows-left-right:before {
  content: "\ea0f"; }

.zi-arrows-top-bottom:before {
  content: "\ea10"; }

.zi-bell-off:before {
  content: "\ea11"; }

.zi-bell:before {
  content: "\ea12"; }

.zi-bullet-alt-down:before {
  content: "\ea13"; }

.zi-bullet-alt-left:before {
  content: "\ea14"; }

.zi-bullet-alt-right:before {
  content: "\ea15"; }

.zi-bullet-alt-up:before {
  content: "\ea16"; }

.zi-bullet-down:before {
  content: "\ea17"; }

.zi-bullet-left:before {
  content: "\ea18"; }

.zi-bullet-right:before, .zi-bullet-right-with-content:before {
  content: "\ea19"; }

.zi-bullet-up:before {
  content: "\ea1a"; }

.zi-calendar-check:before {
  content: "\ea1b"; }

.zi-calendar-warning:before {
  content: "\ea1c"; }

.zi-calendar:before {
  content: "\ea1d"; }

.zi-check:before {
  content: "\ea1e"; }

.zi-checkbox:before {
  content: "\ea1f"; }

.zi-chevron-double-down:before {
  content: "\ea20"; }

.zi-chevron-double-left:before {
  content: "\ea21"; }

.zi-chevron-double-right:before {
  content: "\ea22"; }

.zi-chevron-double-up:before {
  content: "\ea23"; }

.zi-chevron-down-thin:before {
  content: "\ea24"; }

.zi-chevron-down:before, .zi-chevron-down-small:before {
  content: "\ea25"; }

.zi-chevron-left:before {
  content: "\ea26"; }

.zi-chevron-right:before {
  content: "\ea27"; }

.zi-chevron-up:before {
  content: "\ea28"; }

.zi-circle-check-filled:before {
  content: "\ea29"; }

.zi-circle-check:before {
  content: "\ea2a"; }

.zi-circle-info-filled:before {
  content: "\ea2b"; }

.zi-circle-info:before {
  content: "\ea2c"; }

.zi-circle-question-filled:before {
  content: "\ea2d"; }

.zi-circle-question:before {
  content: "\ea2e"; }

.zi-circle-warning-filled:before {
  content: "\ea2f"; }

.zi-circle-warning:before {
  content: "\ea30"; }

.zi-clock:before {
  content: "\ea31"; }

.zi-cog-filled:before {
  content: "\ea32"; }

.zi-collapse:before {
  content: "\ea33"; }

.zi-command:before {
  content: "\ea34"; }

.zi-copy:before {
  content: "\ea35"; }

.zi-dashboards:before {
  content: "\ea36"; }

.zi-data-collection:before {
  content: "\ea37"; }

.zi-drag-handle:before, .drag-icon:before {
  content: "\ea38"; }

.zi-envelope-filled:before {
  content: "\ea39"; }

.zi-expand:before {
  content: "\ea3a"; }

.zi-eye-alt-off:before {
  content: "\ea3b"; }

.zi-eye-off:before {
  content: "\ea3c"; }

.zi-eye:before {
  content: "\ea3d"; }

.zi-filter-large:before {
  content: "\ea3e"; }

.zi-filter:before {
  content: "\ea3f"; }

.zi-fullscreen:before {
  content: "\ea40"; }

.zi-help-circled:before {
  content: "\ea41"; }

.zi-help:before, .zi-help-small:before, .zi-help-filled-small:before {
  content: "\ea42"; }

.zi-home:before {
  content: "\ea43"; }

.zi-i:before, .zi-i-negative:before, .zi-i-positive:before, .zi-i-warning:before {
  content: "\ea44"; }

.zi-integrations:before {
  content: "\ea45"; }

.zi-inventory:before {
  content: "\ea46"; }

.zi-link-external-small:before {
  content: "\ea47"; }

.zi-link-external:before {
  content: "\ea48"; }

.zi-lock:before {
  content: "\ea49"; }

.zi-maximize:before {
  content: "\ea4a"; }

.zi-menu:before {
  content: "\ea4b"; }

.zi-minimize:before {
  content: "\ea4c"; }

.zi-monitoring:before {
  content: "\ea4d"; }

.zi-more:before {
  content: "\ea4e"; }

.zi-pause:before {
  content: "\ea4f"; }

.zi-pencil:before {
  content: "\ea50"; }

.zi-picture:before {
  content: "\ea51"; }

.zi-play:before {
  content: "\ea52"; }

.zi-plus:before, .zi-plus-small:before {
  content: "\ea53"; }

.zi-reference:before {
  content: "\ea54"; }

.zi-reload:before {
  content: "\ea55"; }

.zi-remove:before, .zi-remove-small:before, .zi-remove-smaller:before {
  content: "\ea56"; }

.zi-reports:before {
  content: "\ea57"; }

.zi-search-large:before {
  content: "\ea58"; }

.zi-search:before {
  content: "\ea59"; }

.zi-services:before {
  content: "\ea5a"; }

.zi-sign-out:before {
  content: "\ea5b"; }

.zi-slash:before {
  content: "\ea5c"; }

.zi-speaker-off:before {
  content: "\ea5d"; }

.zi-speaker:before {
  content: "\ea5e"; }

.zi-star-filled:before {
  content: "\ea5f"; }

.zi-star:before {
  content: "\ea60"; }

.zi-stop-sign:before {
  content: "\ea61"; }

.zi-support:before {
  content: "\ea62"; }

.zi-text:before {
  content: "\ea63"; }

.zi-time-period:before {
  content: "\ea64"; }

.zi-trash:before {
  content: "\ea65"; }

.zi-tree-top-bottom:before, .zi-tree-top-bottom-small:before {
  content: "\ea66"; }

.zi-tree-top-right-bottom:before, .zi-tree-top-right-bottom-small:before {
  content: "\ea67"; }

.zi-tree-top-right:before, .zi-tree-top-right-small:before {
  content: "\ea68"; }

.zi-triangle-flipped-warning:before {
  content: "\ea69"; }

.zi-triangle-warning:before {
  content: "\ea6a"; }

.zi-uncheck:before {
  content: "\ea6b"; }

.zi-user-filled:before, .zi-user-filled-small:before {
  content: "\ea6c"; }

.zi-user-settings:before {
  content: "\ea6d"; }

.zi-user:before {
  content: "\ea6e"; }

.zi-users-filled:before, .zi-users-filled-small:before {
  content: "\ea6f"; }

.zi-users:before {
  content: "\ea70"; }

.zi-widget-add:before {
  content: "\ea71"; }

.zi-widget-awaiting-data-large:before {
  content: "\ea72"; }

.zi-widget-empty-references-large:before {
  content: "\ea73"; }

.zi-widget-resize:before {
  content: "\ea74"; }

.zi-wrench-alt:before, .zi-wrench-alt-small:before {
  content: "\ea75"; }

.zi-wrench:before {
  content: "\ea76"; }

div.zabbix-logo {
  width: 114px;
  height: 30px;
  background: url("data:image/svg+xml;base64,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") no-repeat;
  background-size: contain; }

div.zabbix-logo-sidebar {
  width: 91px;
  height: 24px;
  background: url("data:image/svg+xml;base64,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") no-repeat;
  background-size: contain; }

div.zabbix-logo-sidebar-compact {
  width: 24px;
  height: 24px;
  background: url("data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyNCIgaGVpZ2h0PSIyNCI+PHBhdGggZmlsbD0iI2Q0MDAwMCIgZD0iTTAgMGgyNHYyNEgweiIvPjxwYXRoIGZpbGw9IiNmZmYiIHN0cm9rZT0iI2ZmZiIgc3Ryb2tlLXdpZHRoPSIuMiIgZD0iTTIxLjA5OSAxOC44ODd2MS40NTVIMy42NDV2LS45NjNsMTMuOTIxLTE0LjMxSDQuMzcyVjMuNjE1aDE2djEuMTM2TDYuMzg4IDE4Ljg4N3oiLz48L3N2Zz4K") no-repeat; }

body {
  position: relative;
  background-color: #0e1012;
  font-size: 75%;
  font-family: Arial, Tahoma, Verdana, sans-serif;
  line-height: 1.4em;
  color: #f2f2f2; }

h1 {
  font-size: 1.75em;
  line-height: 1.2em; }

h2 {
  font-size: 1.5em;
  line-height: 1.2em; }

h3 {
  font-size: 0.917em;
  text-transform: uppercase;
  color: #737373;
  letter-spacing: .0688em;
  text-rendering: optimizeLegibility; }

h4 {
  font-size: 1.167em;
  color: #f2f2f2;
  line-height: 1.2em; }

sup {
  font-size: 0.917em;
  color: #737373; }

b, strong {
  font-weight: bold; }

em {
  font-style: italic; }

ol {
  list-style: decimal;
  list-style-position: inside; }
  ol li {
    padding: 0 0 .7em; }

p {
  margin: 0 0 .7em; }

pre {
  font-family: "Courier New", Courier, monospace;
  font-size: 1em; }

a {
  color: #4796c4;
  text-decoration: none;
  transition: color 0.2s ease-out; }
  a:visited {
    color: #4796c4;
    text-decoration: none; }
  a:hover {
    outline: 0;
    color: #4796c4;
    border-bottom: 1px solid rgba(71, 150, 196, 0.5); }
  a:focus {
    color: #4796c4;
    outline: none;
    border-bottom: 2px solid rgba(71, 150, 196, 0.5); }
  a:active {
    outline: 0;
    color: #4796c4;
    border-bottom: 1px solid rgba(71, 150, 196, 0.5); }
  a img {
    border: 0;
    vertical-align: top; }
  a.disabled:active, a.disabled:focus, a.disabled:hover {
    cursor: default !important;
    border-bottom: 0;
    outline: 0;
    opacity: .35; }

svg a {
  fill: #4796c4;
  cursor: pointer; }
  svg a:visited {
    fill: #4796c4; }
  svg a:hover, svg a:focus {
    fill: #4796c4;
    text-decoration: underline; }
  svg a:active {
    fill: #4796c4;
    text-decoration: underline; }

.monospace-font {
  font-family: "Courier New", Courier, monospace; }

.float-left {
  float: left; }

.text-placeholder {
  padding: 1px 4px;
  border-radius: 2px;
  background-color: #4f4f4f;
  color: #ffffff; }

[data-indicator="count"][data-indicator-value]:not([data-indicator-value="0"])::after {
  content: attr(data-indicator-value);
  margin-left: .4em;
  color: #768d99;
  font-size: 0.917em; }

[data-indicator="mark"][data-indicator-value]:not([data-indicator-value="0"])::after {
  content: '';
  display: inline-block;
  margin-left: .4em;
  width: 7px;
  height: 7px;
  border-radius: 50%;
  background: #59db8f; }

.ui-resizable-border-n {
  border-top: 1px solid #768d99;
  height: 5px;
  margin: 0 .25rem;
  flex: 1; }

.ui-resizable-border-e {
  border-right: 1px solid #768d99;
  width: 5px;
  margin: .25rem 0;
  flex: 1; }

.ui-resizable-border-s {
  border-bottom: 1px solid #768d99;
  height: 5px;
  margin: 0 .25rem;
  flex: 1; }

.ui-resizable-border-w {
  border-left: 1px solid #768d99;
  width: 5px;
  margin: .25rem 0;
  flex: 1; }

.ui-resizable-n {
  cursor: ns-resize;
  height: 5px;
  width: 100%;
  top: 4px;
  left: 0; }
  .ui-resizable-n .ui-resize-dot {
    left: 0;
    right: 0;
    margin: auto;
    bottom: 0; }

.ui-resizable-ne {
  cursor: nesw-resize;
  right: 0;
  top: 0; }

.ui-resizable-nw {
  cursor: nwse-resize;
  left: 0;
  top: 0; }

.ui-resizable-e {
  cursor: ew-resize;
  width: 5px;
  right: 4px;
  top: 0;
  height: 100%; }
  .ui-resizable-e .ui-resize-dot {
    left: 0;
    top: 50%;
    transform: translateY(-50%); }

.ui-resizable-s {
  cursor: ns-resize;
  height: 5px;
  width: 100%;
  bottom: 5px;
  left: 0; }
  .ui-resizable-s .ui-resize-dot {
    left: 0;
    right: 0;
    margin: auto;
    bottom: -5px; }

.ui-resizable-se {
  cursor: nwse-resize;
  right: 0;
  bottom: 0; }

.ui-resizable-sw {
  cursor: nesw-resize;
  left: 0;
  bottom: 0; }

.ui-resizable-w {
  cursor: ew-resize;
  width: 5px;
  left: 4px;
  top: 0;
  height: 100%; }
  .ui-resizable-w .ui-resize-dot {
    right: 0;
    top: 50%;
    transform: translateY(-50%); }

.ui-inner-handles .ui-resizable-n {
  top: 0; }
.ui-inner-handles .ui-resizable-ne {
  top: -4px;
  right: -4px; }
.ui-inner-handles .ui-resizable-nw {
  top: -4px;
  left: -4px; }
.ui-inner-handles .ui-resizable-e {
  right: 0; }
.ui-inner-handles .ui-resizable-s {
  bottom: 0; }
.ui-inner-handles .ui-resizable-se {
  bottom: -4px;
  right: -4px; }
.ui-inner-handles .ui-resizable-sw {
  bottom: -4px;
  left: -4px; }
.ui-inner-handles .ui-resizable-w {
  left: 0; }

.ui-resize-dot, .ui-resizable-ne, .ui-resizable-nw, .ui-resizable-se, .ui-resizable-sw {
  z-index: 90;
  width: 7px;
  height: 7px;
  position: absolute;
  background-color: #2b2b2b;
  border: 1px solid #768d99;
  border-radius: 100%; }

.ui-resizable-n, .ui-resizable-e, .ui-resizable-s, .ui-resizable-w {
  z-index: 90;
  position: absolute;
  display: flex; }

.scrollable {
  overflow-y: auto;
  overflow-x: hidden;
  position: relative;
  scrollbar-width: none;
  -ms-overflow-style: none; }
  .scrollable:focus {
    outline: 0; }
  .scrollable::-webkit-scrollbar {
    width: 0; }
  .scrollable .scrollbar-track {
    background: #1f1f1f;
    width: 4px;
    border-radius: 2px;
    height: 100%;
    position: absolute;
    right: 2px;
    top: 0;
    opacity: 0;
    z-index: 999; }
    .scrollable .scrollbar-track::before {
      content: '';
      position: absolute;
      top: 0;
      right: -2px;
      bottom: 0;
      left: -6px; }
    .scrollable .scrollbar-track.is-active {
      opacity: 1; }
    .scrollable .scrollbar-track:hover {
      width: 8px; }
      .scrollable .scrollbar-track:hover .scrollbar-thumb {
        width: 8px; }
  .scrollable .scrollbar-thumb {
    background: #383838;
    border-radius: 2px;
    width: 4px;
    position: absolute;
    height: 0;
    right: 0;
    opacity: .5; }
    .scrollable .scrollbar-thumb:hover {
      width: 8px;
      opacity: 1; }
  .scrollable .scrollbar-track {
    transition: opacity 0.3s ease-out 2s, width 0.3s ease-out 2s; }
    .scrollable .scrollbar-track.is-active {
      transition: opacity 0.3s ease-out, width 0.3s ease-out 2s; }
    .scrollable .scrollbar-track:hover {
      transition: opacity 0.3s ease-out, width 0.3s ease-out; }
      .scrollable .scrollbar-track:hover .scrollbar-thumb {
        transition: opacity 0.3s ease-out, width 0.3s ease-out; }
  .scrollable .scrollbar-thumb {
    transition: opacity 0.3s ease-out, width 0.3s ease-out 2s; }

.sortable .sortable-item {
  position: relative;
  transition: left 0.2s, top 0.2s; }
.sortable.sortable-dragging {
  isolation: isolate; }
  .sortable.sortable-dragging .sortable-item.sortable-item-dragging {
    opacity: 0.6;
    z-index: 1;
    transition: left 0s, top 0s; }
.sortable.sortable-transitions-disabled .sortable-item {
  transition: left 0s, top 0s; }

html,
body {
  height: 100vh; }

body {
  display: flex;
  overflow: hidden; }

aside,
.wrapper {
  box-sizing: border-box; }

.wrapper {
  display: flex;
  flex: 1;
  flex-direction: column;
  overflow: auto;
  position: relative; }
  .wrapper > [class^="msg-"] + [class^="msg-"] {
    margin-top: 0; }
  .wrapper > .debug-output {
    flex-shrink: 0; }
  .wrapper > output {
    min-width: 1200px; }

header,
footer {
  flex-shrink: 0;
  min-width: 1200px; }

main {
  position: relative;
  min-width: 1200px;
  padding: 10px; }
  main > .container:not(:first-of-type) {
    margin-top: 10px; }

footer {
  margin-top: auto; }

output {
  display: block; }

.container {
  background: #2b2b2b; }

header {
  display: flex;
  align-items: center;
  height: 45px;
  padding: 0 10px;
  background: #2b2b2b;
  border-bottom: 1px solid #303030; }
  header > div {
    overflow: hidden; }
  header h1 {
    position: relative;
    padding-top: 1px;
    overflow: hidden;
    line-height: 24px;
    text-overflow: ellipsis;
    white-space: nowrap; }
  header .header-doc-link {
    flex-shrink: 0;
    margin: 0 -2px 0 auto;
    padding: 2px; }
    header .header-doc-link + .header-controls {
      margin-left: 0; }
  header .header-controls {
    flex-shrink: 0;
    margin: 0 -2px 0 auto;
    padding: 2px;
    white-space: nowrap; }
    header .header-controls label {
      line-height: 24px; }
    header .header-controls .radio-list-control {
      vertical-align: top; }
      header .header-controls .radio-list-control label {
        line-height: 1.2; }
    header .header-controls .zi-star,
    header .header-controls .zi-star-filled {
      padding: 0;
      vertical-align: middle;
      font-size: 0; }
      header .header-controls .zi-star::before,
      header .header-controls .zi-star-filled::before {
        padding: 0 2px; }
    header .header-controls .zi-star-filled::before {
      color: #f3a914; }
  header .header-kioskmode-controls {
    position: fixed;
    top: 5px;
    right: 45px;
    z-index: 1010;
    display: flex;
    transition: opacity 1s ease-out; }
    header .header-kioskmode-controls.hidden {
      opacity: 0; }
    header .header-kioskmode-controls > li > ul {
      margin-right: 10px; }
    header .header-kioskmode-controls ul {
      display: flex; }
      header .header-kioskmode-controls ul li {
        margin-right: 10px; }
  header .header-z-select {
    max-width: 600px; }
  header a.link-action {
    box-sizing: border-box;
    display: block;
    height: 25px;
    margin-bottom: 0; }
    header a.link-action h1 {
      padding-right: 1em; }
      header a.link-action h1::after {
        content: '';
        position: absolute;
        top: 50%;
        right: .15em;
        width: .3em;
        height: .3em;
        margin-top: -.15em;
        border-right: 2px solid #f2f2f2;
        border-bottom: 2px solid #f2f2f2;
        transform: rotate(45deg);
        transition: border-color 0.2s ease-out; }
    header a.link-action:hover, header a.link-action:focus {
      margin-bottom: 0; }
      header a.link-action:hover h1::after, header a.link-action:focus h1::after {
        border-color: #4796c4; }
  header form {
    display: flex;
    justify-content: flex-end; }
  header ul {
    text-align: right; }
    header ul li {
      display: inline-block;
      vertical-align: top;
      position: relative;
      padding: 0 0 0 10px; }
      header ul li .btn-split {
        margin-right: 0; }
        header ul li .btn-split li {
          padding-left: 0; }
    header ul ul li:first-child {
      padding-left: 0; }
    header ul button {
      line-height: 22px; }
  header z-select > button {
    line-height: normal; }
  header z-select .list {
    text-align: left; }
    header z-select .list li:first-child {
      padding: 5px; }

footer {
  padding: 55px 10px 10px;
  color: #737373;
  line-height: 15px;
  text-align: center; }

.sidebar {
  position: relative;
  z-index: 998;
  display: flex;
  flex-direction: column;
  width: max-content;
  max-width: 480px;
  background: #454545;
  transition: max-width 0.3s, transform 0.3s ease-out;
  will-change: max-width;
  transform: translate3d(0, 0, 0); }
  .sidebar .sidebar-header {
    position: relative;
    box-sizing: border-box;
    display: flex;
    margin: 10px 10px 0;
    padding: 2px;
    overflow: hidden;
    justify-content: space-between; }
  .sidebar .sidebar-header-buttons {
    display: flex;
    width: 48px;
    justify-content: flex-end; }
    .sidebar .sidebar-header-buttons .btn-icon::before {
      color: #9c9c9c; }
    .sidebar .sidebar-header-buttons .btn-icon:hover::before, .sidebar .sidebar-header-buttons .btn-icon:focus::before {
      color: #ffffff; }
  .sidebar .server-name {
    box-sizing: border-box;
    width: 0;
    margin-top: 5px;
    padding: 0 12px;
    color: rgba(194, 194, 194, 0.5);
    transition: opacity 0.3s ease-out; }
  .sidebar .form-search {
    margin: 12px; }
  .sidebar .sidebar-nav {
    display: flex;
    flex-direction: column;
    flex: 1; }
  .sidebar .nav-user {
    margin-top: auto; }
  .sidebar .scrollable .scrollbar-track {
    z-index: 1000;
    background: rgba(135, 209, 255, 0.55); }
  .sidebar .scrollable .scrollbar-thumb {
    background: rgba(135, 209, 255, 0.85); }
  .sidebar .logo {
    position: relative;
    margin-right: 12px; }
    .sidebar .logo:active, .sidebar .logo:hover, .sidebar .logo:link, .sidebar .logo:visited {
      border: 0; }
    .sidebar .logo:focus .zabbix-logo-sidebar {
      box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }
    .sidebar .logo .zabbix-logo-sidebar-compact {
      position: absolute;
      top: 0;
      left: 0;
      opacity: 0; }
  .sidebar .button-show {
    display: none; }
  .sidebar .button-expand {
    display: none; }
  .sidebar .form-search .search {
    transition: opacity 0.3s; }
  .sidebar .form-search .search-icon {
    transition: transform 0.3s; }
  .sidebar.is-compact, .sidebar.is-hidden {
    position: fixed;
    top: 0;
    bottom: 0;
    box-shadow: 2px 0 6px rgba(0, 0, 0, 0.5); }
  .sidebar.is-compact + .wrapper {
    margin-left: 48px; }
  .sidebar.is-compact .button-compact,
  .sidebar.is-compact .button-show {
    display: none; }
  .sidebar.is-compact .button-expand,
  .sidebar.is-compact .button-hide {
    display: inline-block; }
  .sidebar.is-compact:not(.is-opened) {
    max-width: 48px !important; }
    .sidebar.is-compact:not(.is-opened) .scrollable .scrollbar-track {
      display: none; }
    .sidebar.is-compact:not(.is-opened) .logo .zabbix-logo-sidebar {
      opacity: 0;
      transition: opacity 0s 0.3s; }
    .sidebar.is-compact:not(.is-opened) .logo .zabbix-logo-sidebar-compact {
      opacity: 1;
      transition: opacity 0s 0.3s; }
    .sidebar.is-compact:not(.is-opened) .server-name {
      opacity: 0; }
    .sidebar.is-compact:not(.is-opened) .form-search {
      position: relative; }
      .sidebar.is-compact:not(.is-opened) .form-search .search {
        opacity: 0; }
      .sidebar.is-compact:not(.is-opened) .form-search .search-icon {
        opacity: 1;
        transform: scale(1); }
    .sidebar.is-compact:not(.is-opened) nav a {
      color: transparent; }
      .sidebar.is-compact:not(.is-opened) nav a::after {
        opacity: 0; }
    .sidebar.is-compact:not(.is-opened) .submenu {
      max-height: 0 !important; }
  .sidebar.is-hidden .button-show {
    display: inline-block; }
  .sidebar.is-hidden .button-compact,
  .sidebar.is-hidden .button-expand,
  .sidebar.is-hidden .button-hide {
    display: none; }
  .sidebar.is-hidden.focus-off {
    display: none; }
  .sidebar.is-hidden + .wrapper .sidebar-nav-toggle {
    display: block; }
  .sidebar.is-hidden:not(.is-opened) {
    transform: translate3d(-110%, 0, 0); }

.sidebar-nav-toggle {
  display: none;
  flex-shrink: 0;
  margin-right: 10px; }
  .sidebar-nav-toggle + div {
    flex-shrink: 1; }
  .sidebar-nav-toggle .btn-icon::before {
    font-size: 20px;
    color: #4298ca; }
  .sidebar-nav-toggle .btn-icon:hover::before, .sidebar-nav-toggle .btn-icon:focus::before {
    color: #0275b8; }

.grid-columns {
  display: grid;
  align-items: start; }
  .grid-columns.columns-2 {
    grid-template-columns: repeat(2, 1fr); }
  .grid-columns.columns-3 {
    grid-template-columns: repeat(3, 1fr); }

.form-grid {
  display: grid;
  row-gap: 10px;
  column-gap: 10px;
  grid-template-columns: minmax(15%, max-content) auto; }
  .form-grid.label-width-true {
    grid-template-columns: max-content auto; }
  .form-grid fieldset {
    display: contents; }
    .form-grid fieldset legend {
      grid-column: 2 / -1;
      margin-top: 10px; }
      .form-grid fieldset legend span {
        font-size: 1.167em;
        color: #f2f2f2; }
    .form-grid fieldset > label {
      padding-left: 24px; }
    .form-grid fieldset.collapsible legend {
      grid-column: 1 / -1; }
      .form-grid fieldset.collapsible legend .toggle {
        position: relative;
        padding: 0 5px 0 10px;
        border: none;
        background-color: transparent; }
        .form-grid fieldset.collapsible legend .toggle::before {
          color: #69808d; }
        .form-grid fieldset.collapsible legend .toggle:hover::before {
          color: #a2b1ba; }
        .form-grid fieldset.collapsible legend .toggle:active::before, .form-grid fieldset.collapsible legend .toggle:focus::before {
          color: #69808d; }
        .form-grid fieldset.collapsible legend .toggle::after {
          content: ' ';
          position: absolute;
          top: 20px;
          left: 11px;
          width: 2px;
          height: var(--fieldset-height, 0);
          background: #303030;
          pointer-events: none; }
        .form-grid fieldset.collapsible legend .toggle span {
          line-height: normal;
          border-bottom: 1px dotted #f2f2f2; }
        .form-grid fieldset.collapsible legend .toggle:hover span {
          border-bottom-style: solid; }
        .form-grid fieldset.collapsible legend .toggle:focus {
          box-shadow: none; }
          .form-grid fieldset.collapsible legend .toggle:focus span {
            margin-bottom: -1px;
            border-bottom-width: 2px;
            border-bottom-style: solid; }
    .form-grid fieldset.collapsible.collapsed .toggle::after {
      display: none; }
    .form-grid fieldset.collapsible.collapsed label,
    .form-grid fieldset.collapsible.collapsed .fields-group,
    .form-grid fieldset.collapsible.collapsed .form-field {
      display: none !important; }
  .form-grid > label,
  .form-grid .collapsible > label,
  .form-grid .fields-group > label {
    align-self: start;
    line-height: 24px;
    text-align: right;
    word-wrap: break-word; }
    .form-grid > label.fields-group-label,
    .form-grid .collapsible > label.fields-group-label,
    .form-grid .fields-group > label.fields-group-label {
      margin-top: 6px; }
    .form-grid > label .btn-icon,
    .form-grid > label .icon,
    .form-grid .collapsible > label .btn-icon,
    .form-grid .collapsible > label .icon,
    .form-grid .fields-group > label .btn-icon,
    .form-grid .fields-group > label .icon {
      margin-left: 5px; }
    .form-grid > label:not([for]),
    .form-grid .collapsible > label:not([for]),
    .form-grid .fields-group > label:not([for]) {
      pointer-events: none; }
      .form-grid > label:not([for]) .btn-icon,
      .form-grid .collapsible > label:not([for]) .btn-icon,
      .form-grid .fields-group > label:not([for]) .btn-icon {
        pointer-events: initial; }
  .form-grid .fields-group {
    grid-column: 2 / -1;
    padding: 5px;
    border: 1px solid #383838; }
  .form-grid > .form-field,
  .form-grid > .field-fluid,
  .form-grid fieldset > .form-field,
  .form-grid fieldset > .field-fluid,
  .form-grid .form-actions {
    grid-column: 2 / -1;
    text-align: left; }
  .form-grid > .form-field,
  .form-grid > .field-fluid,
  .form-grid fieldset > .form-field,
  .form-grid fieldset > .field-fluid,
  .form-grid .form-actions {
    line-height: 24px; }
  .form-grid .form-actions button:not(:first-of-type) {
    margin-left: 10px; }
  .form-grid.form-grid-actions {
    margin-top: 10px; }
  .form-grid .form-field.offset-1 {
    grid-column-start: 2 !important; }
  .form-grid .form-field.offset-2 {
    grid-column-start: 3 !important; }
  .form-grid .form-field.offset-3 {
    grid-column-start: 4 !important; }
  .form-grid.form-grid-1-1 {
    grid-template-columns: repeat(2, minmax(15%, max-content) auto); }
    .form-grid.form-grid-1-1.label-width-true {
      grid-template-columns: repeat(2, max-content auto); }
  .form-grid.form-grid-3-1 {
    grid-template-columns: minmax(15%, max-content) 3fr max-content auto; }
    .form-grid.form-grid-3-1.label-width-true {
      grid-template-columns: max-content auto 3fr max-content auto; }
  .form-grid.form-grid-1-1 .form-field:not(.field-fluid), .form-grid.form-grid-3-1 .form-field:not(.field-fluid) {
    grid-column-start: initial;
    grid-column-end: span 1; }
  .form-grid .form-subfield {
    margin-top: 5px; }
  .form-grid table {
    width: 100%; }
    .form-grid table.table-initial-width {
      width: auto; }
    .form-grid table .form_row td {
      line-height: 24px; }
    .form-grid table td.sortable-item:not(.sortable-item-frozen) {
      line-height: 24px; }
    .form-grid table td.sortable-item.td-drag-icon {
      line-height: inherit; }
    .form-grid table td.sortable-item.wordbreak, .form-grid table td.sortable-item.wordwrap {
      line-height: 1.4em; }
  .form-grid th {
    padding: 0 5px 0 0;
    color: #737373; }
    .form-grid th:last-child {
      padding: 0; }
  .form-grid td {
    padding: 0 5px 5px 0; }
  .form-grid .table-forms,
  .form-grid .table-forms-separator {
    line-height: 1.4em; }
    .form-grid .table-forms td,
    .form-grid .table-forms-separator td {
      padding: 5px 5px 0 0; }
      .form-grid .table-forms td:last-child,
      .form-grid .table-forms-separator td:last-child {
        padding-right: 0; }
  .form-grid .table-forms-separator {
    display: inline-flex; }
  .form-grid .btn-split button[aria-expanded="true"] {
    background-color: #4f4f4f;
    border-color: #4f4f4f; }
  .form-grid .link:not(:first-child),
  .form-grid .btn-icon:not(:first-child),
  .form-grid .btn-link:not(:first-child) {
    margin-left: 5px; }
  .form-grid .btn-icon.btn-small {
    top: 1px;
    vertical-align: text-bottom; }

button, .btn, .btn-alt, .btn-grey, .btn-icon, .btn-link, .btn-overlay-close, .btn-debug {
  position: relative;
  min-width: 24px;
  height: 24px;
  padding: 0 11px;
  font-family: Arial, Tahoma, Verdana, sans-serif;
  font-size: 1em;
  line-height: 22px;
  text-align: center;
  cursor: pointer;
  border: 1px solid;
  border-radius: 2px;
  transition: color 0.2s ease-out, background-color 0.2s ease-out, border-color 0.2s ease-out, box-shadow 0.2s ease-out; }
  button:empty, .btn:empty, .btn-alt:empty, .btn-grey:empty, .btn-icon:empty, .btn-link:empty, .btn-overlay-close:empty, .btn-debug:empty {
    padding-right: 0;
    padding-left: 0; }
  button:active, .btn:active, .btn-alt:active, .btn-grey:active, .btn-icon:active, .btn-link:active, .btn-overlay-close:active, .btn-debug:active, button:focus, .btn:focus, .btn-alt:focus, .btn-grey:focus, .btn-icon:focus, .btn-link:focus, .btn-overlay-close:focus, .btn-debug:focus {
    box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35);
    outline: none; }
  button.btn-small, .btn-small.btn, .btn-small.btn-alt, .btn-small.btn-grey, .btn-small.btn-icon, .btn-small.btn-link, .btn-small.btn-overlay-close, .btn-small.btn-debug {
    line-height: 14px; }
  button.btn-medium, .btn-medium.btn, .btn-medium.btn-alt, .btn-medium.btn-grey, .btn-medium.btn-icon, .btn-medium.btn-link, .btn-medium.btn-overlay-close, .btn-medium.btn-debug {
    line-height: 16px; }
  button[class^="zi-"]:not(:empty)::before, [class^="zi-"].btn:not(:empty)::before, [class^="zi-"].btn-alt:not(:empty)::before, [class^="zi-"].btn-grey:not(:empty)::before, [class^="zi-"].btn-icon:not(:empty)::before, [class^="zi-"].btn-link:not(:empty)::before, [class^="zi-"].btn-overlay-close:not(:empty)::before, [class^="zi-"].btn-debug:not(:empty)::before, button[class*=" zi-"]:not(:empty)::before, [class*=" zi-"].btn:not(:empty)::before, [class*=" zi-"].btn-alt:not(:empty)::before, [class*=" zi-"].btn-grey:not(:empty)::before, [class*=" zi-"].btn-icon:not(:empty)::before, [class*=" zi-"].btn-link:not(:empty)::before, [class*=" zi-"].btn-overlay-close:not(:empty)::before, [class*=" zi-"].btn-debug:not(:empty)::before {
    margin-right: 4px;
    margin-left: -6px; }

.form-grid table .btn-icon,
.form-grid table .icon, button.btn-medium, .btn-medium.btn, .btn-medium.btn-alt, .btn-medium.btn-grey, .btn-medium.btn-icon, .btn-medium.btn-link, .btn-medium.btn-overlay-close, .btn-medium.btn-debug, .btn-overlay-close, .list-table .btn-icon:not(.btn-small),
.list-table .icon, .multiselect .multiselect-list .subfilter-enabled .btn-icon, .service-info .service-info-value .service-info-value-sla .btn-icon, div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .tools .btn-icon, form.dashboard-widget-piechart .list-item-actions .btn-icon, form.dashboard-widget-svggraph .list-item-actions .btn-icon, .table-forms .table-forms-td-right table .btn-icon:not(.btn-small),
.table-forms .table-forms-td-right table .icon, .overlay-dialogue .overlay-dialogue-header .btn-icon {
  min-width: 18px;
  height: 18px; }

.form-grid table .btn-icon.btn-small,
.form-grid table .icon.btn-small, button.btn-small, .btn-small.btn, .btn-small.btn-alt, .btn-small.btn-grey, .btn-small.btn-icon, .btn-small.btn-link, .btn-small.btn-overlay-close, .btn-small.btn-debug, .navigation-tree .navigation-tree-node-info-arrow button, div.dashboard-widget-navtree .treeview, .form-grid .subscriptions-table .icon {
  min-width: 16px;
  height: 16px; }

button, .btn {
  color: #f2f2f2;
  background-color: #69808d;
  border-color: #69808d; }
  button:hover, .btn:hover {
    color: #f2f2f2;
    background-color: #5e737e;
    border-color: #5e737e; }
  button:active, .btn:active, button:focus, .btn:focus {
    color: #f2f2f2;
    background-color: #5e737e;
    border-color: #5e737e; }

.btn-alt {
  color: #768d99;
  background-color: transparent;
  border-color: #69808d; }
  .btn-alt:hover {
    background-color: #5e737e;
    border-color: #5e737e; }
  .btn-alt:active, .btn-alt:focus {
    background-color: #5e737e;
    border-color: #5e737e; }

.btn-grey, .btn-debug {
  color: #f2f2f2;
  background-color: #383838;
  border-color: #4f4f4f; }
  .btn-grey:hover, .btn-debug:hover {
    color: #f2f2f2;
    background-color: #414141;
    border-color: #4f4f4f; }
  .btn-grey:active, .btn-debug:active, .btn-grey:focus, .btn-debug:focus {
    color: #f2f2f2;
    background-color: #414141;
    border-color: #768d99; }

.btn-icon, .btn-overlay-close {
  color: #69808d;
  background: transparent;
  border: none; }
  .btn-icon:hover, .btn-overlay-close:hover {
    color: #a2b1ba;
    background: transparent;
    border: none; }
  .btn-icon:active, .btn-overlay-close:active, .btn-icon:focus, .btn-overlay-close:focus {
    color: #69808d;
    background: transparent;
    border: none; }
  [disabled].btn-icon, [disabled].btn-overlay-close {
    color: #3d4b53;
    background-color: transparent; }

.btn-link {
  color: #4796c4;
  background: transparent;
  border-color: #4796c4; }
  .btn-link:hover {
    color: #4796c4;
    background: transparent;
    border-color: #4796c4; }
  .btn-link:active, .btn-link:focus {
    color: #4796c4;
    background: transparent;
    border-color: rgba(94, 115, 126, 0.5); }

button[disabled], [disabled].btn, [disabled].btn-alt, [disabled].btn-grey {
  pointer-events: none;
  color: #525252;
  background-color: transparent;
  border-color: #3d3d3d;
  box-shadow: none; }

button {
  margin: 0; }

.btn {
  display: inline-block; }
  .btn:link, .btn:visited {
    color: #f2f2f2; }

.btn-icon {
  display: inline-block;
  line-height: 24px; }
  .btn-icon:not(:hover):not(:focus):link, .btn-icon:not(:hover):not(:focus):visited {
    color: #69808d; }
  .btn-icon[disabled] {
    pointer-events: none; }
  .btn-icon.btn-small {
    line-height: 16px; }

.btn-link {
  box-sizing: content-box;
  padding: 0;
  min-width: 1ch;
  height: 14px;
  line-height: 14px;
  vertical-align: baseline;
  border-width: 0 0 1px;
  border-style: dotted;
  border-radius: 0; }
  .btn-link:active, .btn-link:focus, .btn-link:hover {
    border-style: solid;
    box-shadow: none; }
  .btn-link:active, .btn-link:focus {
    margin-bottom: -1px;
    border-bottom-width: 2px; }
  .btn-link[disabled] {
    background-color: transparent;
    border-style: dotted; }

.btn-tag {
  height: auto;
  outline: none;
  border: none;
  box-sizing: content-box; }
  .btn-tag:hover {
    text-decoration: underline;
    background-color: #4f4f4f; }
  .btn-tag:focus {
    background-color: #4f4f4f; }

.btn-overlay-close {
  position: absolute;
  top: 1px;
  right: 1px;
  z-index: 1000; }
  .btn-overlay-close::before {
    content: ""; }

.btn-debug {
  position: fixed;
  bottom: 4px;
  right: 31px;
  z-index: 15000;
  opacity: 0; }
  .btn-debug.visible {
    opacity: 0.75; }

.zi-filter-large::before, .zi-search-large::before, .zi-widget-awaiting-data-large::before, .zi-widget-empty-references-large::before {
  font-size: 40px; }

.sidebar .sidebar-header-buttons .btn-icon::before, .zi-chevron-down::before, .zi-chevron-down-small::before,
.zi-chevron-left::before,
.zi-chevron-right::before,
.zi-chevron-up::before, .zi-users-filled-small::before {
  font-size: 16px; }

.zi-arrow-down-small::before, .zi-arrow-up-small::before, .zi-chevron-down-small::before, .zi-help-small::before, .zi-help-filled-small::before, .zi-i-negative::before, .zi-i-positive::before, .zi-i-warning::before, .zi-link-external-small::before, .zi-plus-small::before, .zi-remove-small::before, .zi-tree-top-bottom-small::before, .zi-tree-top-right-small::before, .zi-tree-top-right-bottom-small::before, .zi-user-filled-small::before, .zi-wrench-alt-small::before, .overlay-dialogue .overlay-dialogue-header .btn-overlay-close::before {
  font-size: 12px; }

.btn-overlay-close::before, .zi-remove-smaller::before {
  font-size: 10px; }

.zi-alert-with-content::after, .zi-bullet-right-with-content::after {
  content: attr(data-content);
  position: absolute;
  top: 50%;
  font-size: 9px;
  line-height: 1;
  letter-spacing: -.25px;
  transform: translateY(calc(-50% - 2px)); }

.icon {
  color: #69808d; }

.icon-empty-small {
  display: inline-block;
  width: 12px; }

.zi-alert-with-content::after {
  color: #ffffff; }
.zi-alert-with-content.btn-icon {
  color: #69808d; }

.zi-bullet-right-with-content::before {
  position: absolute;
  bottom: 0;
  line-height: 6px; }
.zi-bullet-right-with-content::after {
  color: #f2f2f2; }
.zi-bullet-right-with-content.btn-icon {
  color: #69808d; }

.zi-help-filled-small::before {
  padding: 1px;
  border-radius: 50%; }
.zi-help-filled-small, .zi-help-filled-small.btn-icon {
  color: #ffffff;
  background-color: #69808d;
  border-radius: 50%; }

.zi-i-negative, .zi-i-negative.btn-icon {
  color: #ffffff;
  background-color: #e45959; }

.zi-i-positive, .zi-i-positive.btn-icon {
  color: #ffffff;
  background-color: #59db8f; }

.zi-i-warning, .zi-i-warning.btn-icon {
  color: #ffffff;
  background-color: #e99003; }

.zi-widget-empty-references-large::before {
  color: #e45959; }

.checkbox-list {
  display: grid;
  grid-column-gap: 10px;
  grid-template-columns: repeat(var(--columns), auto); }
  .checkbox-list.fixed {
    grid-template-columns: repeat(var(--columns), 1fr); }
  .checkbox-list.vertical {
    grid-template-rows: repeat(var(--rows), 1fr);
    grid-auto-flow: column; }
  .checkbox-list li {
    padding: 0 2px;
    margin: 0 -2px;
    overflow: hidden;
    text-overflow: ellipsis;
    line-height: 24px; }

.color-picker .color-picker-preview,
.color-picker-dialogue .color-picker-preview {
  position: relative;
  background: #ffffff;
  border: 1px solid #4f4f4f;
  color: #f2f2f2; }
  .color-picker .color-picker-preview::after,
  .color-picker-dialogue .color-picker-preview::after {
    position: absolute;
    top: 2px;
    right: 2px;
    bottom: 2px;
    left: 2px; }
.color-picker .use-default,
.color-picker-dialogue .use-default {
  background: transparent; }
  .color-picker .use-default::after,
  .color-picker-dialogue .use-default::after {
    content: attr(data-use-default);
    text-align: center;
    line-height: 18px; }
.color-picker .use-transparent::after,
.color-picker-dialogue .use-transparent::after {
  content: "";
  font-size: 20px;
  color: #d40000; }

.color-picker {
  display: inline-block;
  line-height: 24px; }
  .color-picker .color-picker-preview {
    padding: 0;
    width: 24px;
    vertical-align: top; }
  .color-picker input:disabled + button {
    pointer-events: none;
    background: #525252 !important;
    border-color: #525252; }

.color-picker-dialogue > div:first-of-type {
  margin-bottom: 5px; }
.color-picker-dialogue > div:not(:first-of-type) button {
  min-width: 20px;
  height: 20px;
  padding: 0;
  border: 1px solid #2b2b2b;
  border-radius: 0;
  vertical-align: middle;
  transition: none; }
  .color-picker-dialogue > div:not(:first-of-type) button:hover, .color-picker-dialogue > div:not(:first-of-type) button:focus {
    border-color: #768d99;
    box-shadow: inset 0 0 0 1px #2b2b2b; }
  .color-picker-dialogue > div:not(:first-of-type) button:active {
    box-shadow: inset 0 0 0 2px #2b2b2b; }
.color-picker-dialogue .color-picker-input {
  position: relative;
  display: inline-block;
  margin-right: 3px; }
  .color-picker-dialogue .color-picker-input .color-picker-preview {
    position: absolute;
    top: 2px;
    left: 2px;
    width: 18px;
    height: 18px; }
    .color-picker-dialogue .color-picker-input .color-picker-preview::after {
      line-height: 14px; }
    .color-picker-dialogue .color-picker-input .color-picker-preview.use-transparent::after {
      font-size: 18px; }
  .color-picker-dialogue .color-picker-input input {
    padding-left: 25px; }

.columns-wrapper {
  display: flex;
  flex-wrap: wrap;
  align-items: start; }
  .columns-wrapper.columns-nowrap {
    flex-wrap: nowrap; }
  .columns-wrapper.columns-2 > div,
  .columns-wrapper.columns-2 > li {
    display: block;
    flex: 0 0 50%;
    max-width: 50%; }
  .columns-wrapper.columns-3 > div,
  .columns-wrapper.columns-3 > li {
    display: block;
    flex: 0 0 33.33333%;
    max-width: 33.33333%; }
  .columns-wrapper .column-5 {
    flex: 0 0 5%;
    max-width: 5%; }
  .columns-wrapper .column-10 {
    flex: 0 0 10%;
    max-width: 10%; }
  .columns-wrapper .column-15 {
    flex: 0 0 15%;
    max-width: 15%; }
  .columns-wrapper .column-20 {
    flex: 0 0 20%;
    max-width: 20%; }
  .columns-wrapper .column-33 {
    flex: 0 0 33.33333%;
    max-width: 33.33333%; }
  .columns-wrapper .column-35 {
    flex: 0 0 35%;
    max-width: 35%; }
  .columns-wrapper .column-40 {
    flex: 0 0 40%;
    max-width: 40%; }
  .columns-wrapper .column-50 {
    flex: 0 0 50%;
    max-width: 50%; }
  .columns-wrapper .column-75 {
    flex: 0 0 75%;
    max-width: 75%; }
  .columns-wrapper .column-90 {
    flex: 0 0 90%;
    max-width: 90%; }
  .columns-wrapper .column-95 {
    flex: 0 0 95%;
    max-width: 95%; }
  .columns-wrapper .column-center {
    display: flex;
    justify-content: center;
    text-align: center; }
  .columns-wrapper .column-middle {
    display: flex;
    align-items: center; }
  .columns-wrapper > div:not(:last-child) section,
  .columns-wrapper > ul:not(:last-child) section {
    margin-right: 10px; }

.header-kioskmode-controls .dashboard-kioskmode-controls li {
  margin-right: 6px; }

.btn-dashboard-toggle-slideshow.slideshow-state-started .slideshow-state-stopped, .btn-dashboard-toggle-slideshow.slideshow-state-stopped .slideshow-state-started,
.btn-dashboard-kioskmode-toggle-slideshow.slideshow-state-started .slideshow-state-stopped,
.btn-dashboard-kioskmode-toggle-slideshow.slideshow-state-stopped .slideshow-state-started {
  display: none; }

.dashboard-edit {
  display: inline-block;
  margin-right: 5px;
  position: relative; }
  .dashboard-edit ul li {
    vertical-align: middle; }
    .dashboard-edit ul li:first-child {
      padding: 0; }

.dashboard-navigation {
  display: flex;
  margin-top: 2px;
  margin-bottom: 6px; }
  .dashboard-navigation.is-scrollable .dashboard-navigation-controls button.btn-dashboard-previous-page, .dashboard-navigation.is-scrollable .dashboard-navigation-controls button.btn-dashboard-next-page {
    display: inline-block; }

.dashboard-navigation-tabs, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs {
  margin: -2px 10px -2px 0;
  overflow: hidden; }
  .dashboard-navigation-tabs .sortable, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable {
    padding: 2px 0;
    display: flex;
    align-items: center; }
    .dashboard-navigation-tabs .sortable li, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li {
      box-sizing: border-box;
      display: block;
      padding: 0 1.5px;
      margin: 0;
      list-style-type: none;
      outline: none; }
      .dashboard-navigation-tabs .sortable li div, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div {
        display: flex;
        align-items: center;
        min-height: 22px;
        line-height: 22px;
        padding: 0 10px;
        flex-shrink: 0;
        font-size: 12px;
        white-space: nowrap;
        cursor: pointer;
        user-select: none;
        background-color: #2b2b2b;
        border: 1px solid #303030;
        border-radius: 2px;
        transition: background-color 0.3s, border-color 0.3s; }
        .dashboard-navigation-tabs .sortable li div span, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div span {
          max-width: 300px;
          color: #f2f2f2; }
        .dashboard-navigation-tabs .sortable li div button, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div button {
          margin: -1px -9px -1px 4px; }
        .dashboard-navigation-tabs .sortable li div:not(.selected-tab) button, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div:not(.selected-tab) button {
          display: none; }
    .dashboard-navigation-tabs .sortable li:not(.sortable-item-dragging):focus:not(:active), .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li:not(.sortable-item-dragging):focus:not(:active) {
      padding: 0 3.5px; }
      .dashboard-navigation-tabs .sortable li:not(.sortable-item-dragging):focus:not(:active) div, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li:not(.sortable-item-dragging):focus:not(:active) div {
        padding: 0 8px;
        border: 1px solid transparent;
        box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }
    .dashboard-navigation-tabs .sortable li div:hover, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div:hover,
    .dashboard-navigation-tabs .sortable li.sortable-item-dragging div,
    .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li.sortable-item-dragging div {
      background: #454545;
      border: 1px solid #121212; }
    .dashboard-navigation-tabs .sortable li div.selected-tab, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div.selected-tab {
      background-color: #4f4f4f;
      border: 1px solid transparent; }
      .dashboard-navigation-tabs .sortable li div.selected-tab span, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div.selected-tab span {
        color: #f2f2f2; }
  .dashboard-navigation-tabs .btn-dashboard-page-properties:hover, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .btn-dashboard-page-properties:hover, .dashboard-navigation-tabs .btn-dashboard-page-properties:focus, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .btn-dashboard-page-properties:focus, .dashboard-navigation-tabs .btn-dashboard-page-properties:active, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .btn-dashboard-page-properties:active {
    color: #f2f2f2; }

.dashboard-navigation-controls {
  display: flex;
  flex-shrink: 0; }
  .dashboard-navigation-controls button {
    margin-left: 10px; }
    .dashboard-navigation-controls button.btn-dashboard-previous-page, .dashboard-navigation-controls button.btn-dashboard-next-page {
      display: none;
      margin: 0; }

.dashboard-grid {
  position: relative;
  margin: -4px;
  z-index: 3;
  transition: margin-top 0.3s; }
  .dashboard-grid.reserve-header-lines-1 {
    margin-top: 2rem; }
  .dashboard-grid.reserve-header-lines-2 {
    margin-top: calc(4rem + 8px); }

.dashboard:not(.dashboard-is-multipage):not(.dashboard-is-edit-mode) .dashboard-navigation {
  display: none; }
.dashboard.dashboard-is-edit-mode .btn-dashboard-toggle-slideshow, .dashboard:not(.dashboard-is-multipage) .btn-dashboard-toggle-slideshow {
  display: none; }

.dashboard-grid-widget-blocker {
  position: absolute;
  width: 100%;
  height: 100%;
  z-index: 998; }

.dashboard-widget-placeholder {
  box-sizing: border-box;
  display: flex;
  height: 200px;
  padding: .25rem;
  width: calc(100% / 6 - .65rem);
  z-index: 999;
  color: #737373;
  cursor: pointer;
  user-select: none; }
  .dashboard-widget-placeholder .dashboard-widget-placeholder-box {
    position: relative;
    overflow: hidden;
    background-color: rgba(43, 43, 43, 0.7);
    border: 1px dashed #4f4f4f;
    box-shadow: 2px 2px 6px rgba(0, 0, 0, 0.5);
    flex: 1; }
  .dashboard-widget-placeholder .dashboard-widget-placeholder-label {
    position: absolute;
    left: 0;
    top: calc(50% - 23px);
    right: 0;
    bottom: 0;
    text-align: center; }
    .dashboard-widget-placeholder .dashboard-widget-placeholder-label::before {
      content: "";
      margin: 0 -4px;
      font-size: 46px; }
    .dashboard-widget-placeholder .dashboard-widget-placeholder-label div {
      padding: 0 5px 5px; }
  .dashboard-widget-placeholder .dashboard-widget-placeholder-resizing {
    background: #010101;
    box-shadow: inset 2px 2px 6px rgba(0, 0, 0, 0.5); }
    .dashboard-widget-placeholder .dashboard-widget-placeholder-resizing .dashboard-widget-placeholder-label::before {
      content: ""; }
  .dashboard-widget-placeholder.disabled {
    opacity: 1; }
    .dashboard-widget-placeholder.disabled .dashboard-widget-placeholder-box {
      background-color: rgba(43, 43, 43, 0.7); }

.dashboard-grid-widget-container {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  padding: .25rem;
  box-sizing: border-box;
  display: flex;
  flex-direction: column; }

.dashboard-grid-widget-header {
  position: absolute;
  top: .25rem;
  left: .25rem;
  right: .25rem;
  height: 2rem;
  padding-left: 10px;
  display: flex;
  flex-wrap: nowrap;
  flex-direction: row;
  overflow: hidden;
  align-items: flex-start;
  border: 1px solid #303030;
  border-bottom: 0;
  background-color: #2b2b2b;
  z-index: 3; }
  .dashboard-grid-widget-header h4 {
    flex-grow: 1;
    margin-top: .5rem;
    color: #dedede;
    font-weight: bold; }
  .dashboard-grid-widget-header ul {
    flex-shrink: 0; }
    .dashboard-grid-widget-header ul li {
      float: left;
      padding: 0; }
      .dashboard-grid-widget-header ul li select {
        margin: 4px 0; }

.dashboard-grid-widget-actions {
  opacity: 0;
  transition: opacity 0.2s ease-out; }
  .dashboard-grid-widget-actions li {
    margin-top: 2px;
    margin-right: 2px; }

.dashboard-grid-widget-messages .msg-bad,
.dashboard-grid-widget-messages .msg-good,
.dashboard-grid-widget-messages .msg-warning {
  margin: 0 10px 8px; }

.dashboard-grid-widget-body {
  display: contents; }

.dashboard-grid-widget-debug .debug-output {
  margin: 8px 10px 0;
  max-height: none;
  border-color: #383838; }

.dashboard-grid-widget-contents {
  position: relative;
  flex-grow: 1;
  margin-top: 2rem;
  box-sizing: border-box;
  height: calc(100% - 2rem);
  padding-bottom: 8px;
  overflow: auto;
  border: 1px solid #303030;
  background-color: #2b2b2b; }
  .dashboard-grid-widget-contents .list-table {
    border: 0; }
    .dashboard-grid-widget-contents .list-table tbody tr:last-child th, .dashboard-grid-widget-contents .list-table tbody tr:last-child td {
      border-bottom: 1px solid #383838; }
      .dashboard-grid-widget-contents .list-table tbody tr:last-child th.list-table-footer, .dashboard-grid-widget-contents .list-table tbody tr:last-child td.list-table-footer {
        border-bottom: 0; }
    .dashboard-grid-widget-contents .list-table.sticky-footer {
      margin-bottom: -8px; }
      .dashboard-grid-widget-contents .list-table.sticky-footer tfoot {
        bottom: -8px; }
    .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) {
      height: 100%; }
      .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) thead th, .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) thead td {
        height: 1%; }
      .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) tbody tr td {
        vertical-align: middle; }
      .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) tbody tr:last-child td {
        border-bottom: 0; }
      .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) tbody tr .no-data-message[class^="zi-"], .dashboard-grid-widget-contents .list-table.no-data:not(.no-data-without-icon) tbody tr .no-data-message[class*=" zi-"] {
        margin-top: 0;
        margin-bottom: 0; }
  .dashboard-grid-widget-contents.no-padding .dashboard-grid-widget-debug .debug-output {
    margin: 8px 10px; }

.dashboard-grid-widget-mask {
  position: absolute;
  top: .25rem;
  left: .25rem;
  right: .25rem;
  bottom: .25rem;
  display: none; }

.dashboard-grid-widget {
  position: absolute; }
  .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-header {
    height: 0;
    transition: height 0.3s, margin-top 0.3s, box-shadow 0.3s; }
  .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-contents {
    margin-top: 0;
    height: 100%;
    padding-top: 8px; }
    .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-contents.no-padding .dashboard-grid-widget-messages .msg-bad,
    .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-contents.no-padding .dashboard-grid-widget-messages .msg-good,
    .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-contents.no-padding .dashboard-grid-widget-messages .msg-warning {
      margin: 8px 10px; }
    .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-contents .list-table.sticky-footer {
      margin-bottom: 0; }
      .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-contents .list-table.sticky-footer tfoot {
        bottom: 0; }
  .dashboard-grid-widget.dashboard-grid-widget-hidden-header.dashboard-grid-widget-focus .dashboard-grid-widget-header {
    height: 2rem;
    margin-top: -2rem;
    box-shadow: 0 -6px 8px -2px rgba(128, 128, 128, 0.15);
    z-index: 5; }
  .dashboard-grid-widget.dashboard-grid-widget-focus .dashboard-grid-widget-actions {
    opacity: 1; }

.dashboard-grid-iterator-container {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%; }

.dashboard-grid-iterator-header {
  position: absolute;
  top: .25rem;
  left: .25rem;
  right: .25rem;
  height: 0;
  display: flex;
  flex-wrap: nowrap;
  flex-direction: row;
  overflow: hidden;
  padding-left: 10px;
  box-sizing: border-box;
  border: 1px solid #303030;
  border-bottom: 0;
  background-color: #2b2b2b;
  opacity: 0;
  transition: margin-top 0.3s ease-out, height 0.3s ease-out, opacity 0.15s 0.15s ease-out, box-shadow 0.3s ease-out;
  z-index: 3; }
  .dashboard-grid-iterator-header h4 {
    flex-grow: 1;
    margin-top: .5rem;
    color: #dedede;
    font-weight: bold; }
  .dashboard-grid-iterator-header ul {
    flex-shrink: 0; }
    .dashboard-grid-iterator-header ul li {
      float: left;
      padding: 0; }
  .dashboard-grid-iterator-header .dashboard-grid-iterator-pager {
    margin: 0 10px;
    height: 2rem;
    display: none;
    flex-direction: row;
    flex-shrink: 0;
    align-items: center; }
    .dashboard-grid-iterator-header .dashboard-grid-iterator-pager .dashboard-grid-iterator-pager-info {
      margin: 2px .5em 0;
      white-space: nowrap;
      color: #dedede; }
  .dashboard-grid-iterator-header.pager-visible .dashboard-grid-iterator-pager {
    display: flex; }

.dashboard-grid-iterator-actions {
  opacity: 0;
  transition: opacity 0.2s ease-out; }
  .dashboard-grid-iterator-actions li {
    margin-top: 2px;
    margin-right: 2px; }

.dashboard-grid-iterator-contents {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  overflow: visible;
  transition: margin-top 0.3s ease-out; }

.dashboard-grid-iterator-placeholder {
  position: absolute;
  display: flex; }
  .dashboard-grid-iterator-placeholder > div {
    margin: .25rem;
    display: flex;
    flex-grow: 1;
    flex-direction: column;
    border: 1px solid #303030;
    background-color: rgba(43, 43, 43, 0.3); }

.dashboard-grid-iterator-too-small {
  position: absolute;
  top: .25rem;
  left: .25rem;
  right: .25rem;
  bottom: .25rem;
  overflow: hidden;
  display: none;
  align-items: center;
  justify-content: center;
  border: 1px solid #303030;
  background-color: #2b2b2b; }
  .dashboard-grid-iterator-too-small > div {
    padding: 0 10px;
    text-align: center;
    color: #737373; }

.dashboard-grid-iterator-mask {
  position: absolute;
  top: .25rem;
  left: .25rem;
  right: .25rem;
  bottom: .25rem;
  display: none; }

.dashboard-grid-iterator {
  position: absolute; }
  .dashboard-grid-iterator.iterator-too-small .dashboard-grid-iterator-too-small {
    display: flex; }
  .dashboard-grid-iterator.iterator-too-small .dashboard-grid-iterator-contents {
    display: none; }
  .dashboard-grid-iterator.dashboard-grid-iterator-focus {
    z-index: 5; }
    .dashboard-grid-iterator.dashboard-grid-iterator-focus .dashboard-grid-iterator-header {
      margin-top: -2rem;
      height: 2rem;
      box-shadow: 0 -6px 8px -2px rgba(128, 128, 128, 0.15);
      opacity: 1;
      transition: margin-top 0.3s ease-out, height 0.3s ease-out, box-shadow 0.3s ease-out; }
    .dashboard-grid-iterator.dashboard-grid-iterator-focus .dashboard-grid-iterator-actions {
      opacity: 1; }
    .dashboard-grid-iterator.dashboard-grid-iterator-focus.dashboard-grid-iterator-hidden-header.iterator-double-header .dashboard-grid-iterator-header {
      margin-top: calc(-4rem - 8px);
      height: calc(4rem + 8px); }
  .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents {
    position: absolute;
    top: .25rem;
    left: .25rem;
    right: .25rem;
    bottom: .25rem;
    border: 1px solid #303030;
    background-color: #2b2b2b; }
    .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div {
      padding-top: 8px; }
      .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table {
        border: 0; }
        .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table tbody tr:last-child th, .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table tbody tr:last-child td {
          border-bottom: 1px solid #383838; }
          .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table tbody tr:last-child th.list-table-footer, .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table tbody tr:last-child td.list-table-footer {
            border-bottom: 0; }
      .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .msg-good,
      .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .msg-bad {
        margin: 0 10px; }

.dashboard-grid-widget.ui-draggable .dashboard-grid-widget-header,
.dashboard-grid-widget.ui-draggable .dashboard-grid-iterator-header,
.dashboard-grid-iterator.ui-draggable .dashboard-grid-widget-header,
.dashboard-grid-iterator.ui-draggable .dashboard-grid-iterator-header {
  cursor: grab; }
.dashboard-grid-widget.ui-draggable.ui-draggable-dragging .dashboard-grid-widget-header,
.dashboard-grid-widget.ui-draggable.ui-draggable-dragging .dashboard-grid-iterator-header,
.dashboard-grid-iterator.ui-draggable.ui-draggable-dragging .dashboard-grid-widget-header,
.dashboard-grid-iterator.ui-draggable.ui-draggable-dragging .dashboard-grid-iterator-header {
  cursor: grabbing; }
.dashboard-grid-widget.ui-draggable .dashboard-grid-widget .dashboard-grid-widget-header,
.dashboard-grid-widget.ui-draggable .dashboard-grid-iterator .dashboard-grid-iterator-header,
.dashboard-grid-iterator.ui-draggable .dashboard-grid-widget .dashboard-grid-widget-header,
.dashboard-grid-iterator.ui-draggable .dashboard-grid-iterator .dashboard-grid-iterator-header {
  cursor: auto; }

.dashboard.dashboard-is-edit-mode .dashboard-grid-widget-actions {
  opacity: 1; }
.dashboard.dashboard-is-edit-mode.dashboard-positioning .dashboard-grid-widget.dashboard-grid-widget-hidden-header .dashboard-grid-widget-header {
  transition: none; }
.dashboard.dashboard-is-edit-mode.dashboard-positioning .dashboard-grid-iterator.dashboard-grid-widget-hidden-header .dashboard-grid-widget-header {
  transition: none; }
.dashboard.dashboard-is-edit-mode.dashboard-positioning .dashboard-grid-widget-mask {
  display: block; }
.dashboard.dashboard-is-edit-mode.dashboard-positioning .dashboard-grid-iterator-mask {
  display: block; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget .ui-resizable-handle {
  visibility: hidden; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.dashboard-grid-widget-focus .ui-resizable-handle {
  visibility: visible; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing.dashboard-grid-widget-hidden-header .dashboard-grid-widget-header {
  margin-top: 0;
  height: 0;
  box-shadow: none; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging .ui-resizable-handle {
  visibility: hidden; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging.dashboard-grid-widget-hidden-header .dashboard-grid-widget-header {
  margin-top: -2rem;
  height: 2rem;
  box-shadow: 0 -6px 8px -2px rgba(128, 128, 128, 0.15);
  opacity: .5; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging {
  z-index: 1000; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing::before, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging::before {
    content: '';
    display: block;
    position: absolute;
    top: .25rem;
    left: .25rem;
    right: .25rem;
    bottom: .25rem;
    background-color: rgba(1, 1, 1, 0.75);
    box-shadow: 0 4px 20px 2px rgba(0, 0, 0, 0.15); }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing .dashboard-grid-widget-header, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging .dashboard-grid-widget-header {
    border-color: #212121; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing .dashboard-grid-widget-contents, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging .dashboard-grid-widget-contents {
    border-color: #212121; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget:not(.ui-draggable-dragging):not(.dashboard-grid-widget-focus) {
  transition-property: height, top;
  transition-duration: 0.15s; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator .ui-resizable-handle {
  visibility: hidden; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.dashboard-grid-iterator-focus .ui-resizable-handle {
  visibility: visible; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-header {
  margin-top: 0;
  height: 0;
  box-shadow: none;
  opacity: 0; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .ui-resizable-handle {
  visibility: hidden; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-header {
  margin-top: -2rem;
  height: 2rem;
  box-shadow: 0 -6px 8px -2px rgba(128, 128, 128, 0.15);
  opacity: .5; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging.dashboard-grid-iterator-hidden-header.iterator-double-header .dashboard-grid-iterator-header {
  margin-top: calc(-4rem - 8px);
  height: calc(4rem + 8px); }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging {
  z-index: 1000; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing::before, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging::before {
    content: '';
    display: block;
    position: absolute;
    top: .25rem;
    left: .25rem;
    right: .25rem;
    bottom: .25rem;
    background-color: rgba(1, 1, 1, 0.75);
    box-shadow: 0 4px 20px 2px rgba(0, 0, 0, 0.15); }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-header, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-header {
    border-color: #212121; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-header, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-header {
    border-color: #212121; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-contents, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-contents {
    border-color: #212121; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-contents .dashboard-grid-iterator-placeholder > div, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-contents .dashboard-grid-iterator-placeholder > div {
    border-color: #212121; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-mask, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-mask {
    border: 1px solid #1c1c1c;
    border-top: 0; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator:not(.ui-draggable-dragging):not(.dashboard-grid-iterator-focus) {
  transition-property: height, top;
  transition-duration: 0.15s; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator .dashboard-grid-widget:not(.dashboard-grid-widget-focus) {
  transition-property: height, top;
  transition-duration: 0s; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget-placeholder {
  position: absolute;
  z-index: 999; }
  .dashboard.dashboard-is-edit-mode .dashboard-grid-widget-placeholder > div {
    position: absolute;
    top: .25rem;
    left: .25rem;
    right: .25rem;
    bottom: .25rem;
    background-color: #010101; }

@keyframes zoom-in-out {
  0% {
    transform: scale(1); }
  50% {
    transform: scale(1.05); }
  100% {
    transform: scale(1); } }
.new-widget {
  animation: zoom-in-out 0.2s linear; }

.dashboard-grid-widget-contents .list-table th:first-child, .dashboard-grid-widget-contents .list-table td:first-child, .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table th:first-child, .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table td:first-child {
  padding-left: 10px; }
.dashboard-grid-widget-contents .list-table th:last-child, .dashboard-grid-widget-contents .list-table td:last-child, .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table th:last-child, .dashboard-grid-iterator.iterator-alt-content .dashboard-grid-iterator-contents > div .list-table td:last-child {
  padding-right: 10px; }

.dashboard-widget-graph-link {
  display: block; }
  .dashboard-widget-graph-link:link {
    border: 0; }

.dashboard-grid-iterator-contents.is-loading::before {
  margin: calc(.25rem + 1px); }

.overlay-dialogue.modal-popup.modal-widget-configuration {
  width: auto;
  max-width: calc(100vw - 20px);
  min-width: 650px;
  box-sizing: border-box; }
  .overlay-dialogue.modal-popup.modal-widget-configuration .overlay-dialogue-body {
    overflow-x: auto; }

.dashboard-widget-form {
  position: relative; }
  .dashboard-widget-form .form-field-show-header {
    position: absolute;
    right: 5px; }
  .dashboard-widget-form .table-tags tr:last-of-type td {
    padding-bottom: 0;
    line-height: 1.4; }

.wrapper.layout-kioskmode .dashboard-navigation {
  display: none; }

@page {
  margin: 10px; }
@media print {
  .dashboard-page {
    margin: 10px;
    break-after: page; } }
.diff {
  padding: 4px;
  background: #0e1012; }
  .diff .diff-added {
    background-color: #32453a; }
  .diff .diff-removed {
    background-color: #433131; }
  .diff pre {
    width: max-content;
    min-width: 100%; }
    .diff pre > div {
      white-space: pre; }

.entity-count {
  display: inline-block;
  padding: 0 3px;
  min-width: 10px;
  line-height: 16px;
  text-align: center;
  background-color: #4f4f4f;
  border-radius: 2px;
  overflow: hidden;
  text-overflow: ellipsis; }
  .entity-count, .entity-count:hover, .entity-count:focus, .entity-count:link, .entity-count:visited {
    color: #f2f2f2;
    border-bottom: none; }
  .entity-count:focus {
    box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }

.expandable-subfilter {
  position: relative;
  word-break: break-word;
  max-height: 18px;
  line-height: 18px;
  overflow: hidden; }
  .expandable-subfilter.ten-lines {
    max-height: calc(20px * 10); }
  .expandable-subfilter > div {
    display: inline; }
  .expandable-subfilter .hidden {
    display: none; }
  .expandable-subfilter.expanded {
    max-height: none; }
    .expandable-subfilter.expanded .hidden {
      display: unset; }

.form-search {
  position: relative;
  height: 24px; }
  .form-search .search {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 24px;
    padding-right: 22px;
    color: #ffffff;
    background: #191919;
    border: 1px solid #010a0f;
    border-radius: 3px; }
    .form-search .search:focus {
      color: #ffffff;
      background: #191919;
      border-color: #787878; }
      .form-search .search:focus ~ .btn-icon:not([disabled]) {
        color: #ffffff !important; }
  .form-search .btn-icon {
    position: absolute;
    top: 2px;
    right: 2px;
    min-width: 20px;
    height: 20px;
    color: #b1b1b1; }
    .form-search .btn-icon:hover {
      color: #ffffff; }
    .form-search .btn-icon:focus {
      color: #ffffff;
      box-shadow: 0 0 0 1.5px #787878; }

.search-suggest {
  position: absolute;
  left: 0;
  right: 0;
  z-index: 1000;
  min-width: 16px;
  color: #737373;
  background-color: #383838;
  border: 1px solid #383838;
  border-top: 0;
  box-shadow: 0 6px 10px 0 rgba(0, 0, 0, 0.5); }
  .search-suggest li {
    display: block;
    height: inherit;
    padding: .4em 5px;
    margin: 0;
    overflow: hidden;
    color: #f2f2f2;
    line-height: normal;
    word-break: break-word;
    cursor: pointer;
    transition: background-color 0.2s ease-out; }
  .search-suggest .suggest-hover {
    background-color: #454545; }

.inline-filter {
  line-height: 24px;
  text-align: right; }
  .inline-filter button,
  .inline-filter input,
  .inline-filter label {
    vertical-align: top; }
  .inline-filter > li {
    display: inline-block; }
    .inline-filter > li:not(:last-child) {
      margin-right: 3px; }
    .inline-filter > li.inline-filter-label {
      margin-right: 10px; }

.inline-filter-footer {
  display: flex;
  align-items: baseline;
  justify-content: space-between; }
  .inline-filter-footer > li.inline-filter-stats {
    color: #737373; }

.list-table {
  width: 100%;
  background-color: #2b2b2b;
  border-collapse: separate;
  border: 1px solid #303030; }
  .list-table.fixed {
    table-layout: fixed; }
  .list-table thead th {
    height: 100%;
    overflow: hidden;
    border-bottom-width: 2px;
    border-bottom-color: #2b2b2b;
    color: #737373;
    white-space: nowrap;
    vertical-align: bottom; }
    .list-table thead th z-vertical {
      display: inline-block;
      position: relative;
      max-height: 150px;
      min-width: 22px;
      overflow: hidden;
      text-overflow: ellipsis;
      line-height: 22px; }
    .list-table thead th .arrow-up {
      margin-left: 3px;
      border-bottom-color: #737373; }
    .list-table thead th .arrow-right {
      margin-left: 3px;
      border-left-color: #737373; }
    .list-table thead th .arrow-down {
      margin-left: 3px;
      border-top-color: #737373; }
    .list-table thead th a {
      display: block;
      position: relative;
      margin: -1em;
      padding: 1em;
      border: 0;
      transition: background-color 0.2s ease-out; }
      .list-table thead th a:hover, .list-table thead th a:focus, .list-table thead th a:active {
        text-decoration: none;
        background-color: #383838; }
  .list-table tbody tr.hidden {
    display: none; }
  .list-table tbody tr:last-child th,
  .list-table tbody tr:last-child td {
    border-bottom-style: none; }
  .list-table tbody tr:not(.hover-nobg):not(.row-selected):hover {
    background-color: #383838; }
  .list-table tbody tr.row-selected {
    background-color: #2f280a; }
  .list-table tbody th.table-info,
  .list-table tbody td.table-info {
    color: #737373; }
  .list-table tbody th.list-table-actions,
  .list-table tbody td.list-table-actions {
    text-align: right;
    white-space: nowrap; }
  .list-table th,
  .list-table td {
    position: relative;
    padding: 6px 5px;
    line-height: 18px;
    text-align: left;
    vertical-align: top;
    border-bottom: 1px solid #383838; }
    .list-table th.second-col,
    .list-table td.second-col {
      width: 20px;
      padding-left: 0;
      padding-right: 0;
      text-align: center; }
  .list-table.sticky-header thead {
    position: sticky;
    top: 0;
    z-index: 1;
    background-color: #2b2b2b; }
  .list-table.sticky-footer tfoot {
    position: sticky;
    bottom: 0;
    z-index: 1;
    background: #2b2b2b; }
    .list-table.sticky-footer tfoot tr:first-of-type td, .list-table.sticky-footer tfoot tr:first-of-type th {
      border-top: 2px solid #2b2b2b;
      border-bottom-style: none; }
  .list-table .list-table-footer {
    border-bottom: 0;
    color: #737373;
    text-align: right; }
    .list-table .list-table-footer:hover {
      background-color: #2b2b2b; }
  .list-table .action-container {
    display: flex;
    width: 100%; }
    .list-table .action-container .link-action {
      height: 16px;
      min-width: 1ch; }
    .list-table .action-container .rel-container {
      display: flex; }
    .list-table .action-container .separator {
      flex-shrink: 0;
      margin-right: 4px; }
  .list-table .latest-values,
  .list-table .opdata {
    max-width: 300px; }
  .list-table .checkbox-radio {
    margin: 1px 0; }
    .list-table .checkbox-radio + label {
      display: inline-block;
      margin: 1px 0;
      line-height: 1;
      vertical-align: top; }
      .list-table .checkbox-radio + label span {
        margin: 0; }
  .list-table .wordbreak {
    white-space: normal;
    overflow: hidden;
    text-overflow: ellipsis; }
  .list-table.compact-view td {
    padding-top: 2px;
    padding-bottom: 2px;
    white-space: nowrap; }
    .list-table.compact-view td .rel-container span {
      margin: 0; }
    .list-table.compact-view td.nowrap {
      text-overflow: clip; }
  .list-table.compact-view td, .list-table.has-highlighted-rows td {
    border-bottom: 0;
    box-shadow: inset 0 -1px 0 0 rgba(242, 242, 242, 0.1); }
  .list-table.no-data tbody tr td {
    height: auto;
    color: #737373;
    text-align: center; }
    .list-table.no-data tbody tr td:hover {
      background-color: #2b2b2b; }
  .list-table.no-data tbody tr .no-data-message[class^="zi-"], .list-table.no-data tbody tr .no-data-message[class*=" zi-"] {
    margin-top: 100px;
    margin-bottom: 100px; }

.list-table .btn-icon,
.list-table .icon,
.list-table .link-action {
  vertical-align: top; }
  .list-table .btn-icon:not(.no-indent):not(:first-child),
  .list-table .icon:not(.no-indent):not(:first-child),
  .list-table .link-action:not(.no-indent):not(:first-child) {
    margin-left: 5px; }
.list-table .icon {
  border-radius: 2px; }
.list-table .btn-small,
.list-table .problem-icon-link,
.list-table .status-container,
.list-table .entity-count {
  vertical-align: top; }

.list-table thead th.column-tags-1 {
  width: 75px; }
  @media screen and (min-width: 1200px) {
    .list-table thead th.column-tags-1 {
      width: 106px; } }
  @media screen and (min-width: 1400px) {
    .list-table thead th.column-tags-1 {
      width: 137px; } }
  @media screen and (min-width: 1600px) {
    .list-table thead th.column-tags-1 {
      width: 168px; } }
.list-table thead th.column-tags-2 {
  width: 124px; }
  @media screen and (min-width: 1200px) {
    .list-table thead th.column-tags-2 {
      width: 186px; } }
  @media screen and (min-width: 1400px) {
    .list-table thead th.column-tags-2 {
      width: 248px; } }
  @media screen and (min-width: 1600px) {
    .list-table thead th.column-tags-2 {
      width: 310px; } }
.list-table thead th.column-tags-3 {
  width: 173px; }
  @media screen and (min-width: 1200px) {
    .list-table thead th.column-tags-3 {
      width: 266px; } }
  @media screen and (min-width: 1400px) {
    .list-table thead th.column-tags-3 {
      width: 359px; } }
  @media screen and (min-width: 1600px) {
    .list-table thead th.column-tags-3 {
      width: 452px; } }
@media screen and (min-width: 1200px) {
  .list-table .tags-wrapper {
    max-width: 266px; } }
@media screen and (min-width: 1400px) {
  .list-table .tags-wrapper {
    max-width: 359px; } }
@media screen and (min-width: 1600px) {
  .list-table .tags-wrapper {
    max-width: 452px; } }
.list-table.compact-view .tag {
  max-width: 40px; }
  @media screen and (min-width: 1200px) {
    .list-table.compact-view .tag {
      max-width: 71px; } }
  @media screen and (min-width: 1400px) {
    .list-table.compact-view .tag {
      max-width: 102px; } }
  @media screen and (min-width: 1600px) {
    .list-table.compact-view .tag {
      max-width: 133px; } }

.hintbox-wrap .tag {
  cursor: pointer; }
  .hintbox-wrap .tag:last-child {
    margin-right: 0; }

.list-table .timeline-date,
.list-table .timeline-axis,
.list-table .timeline-td {
  border-bottom-color: transparent; }
.list-table .timeline-date {
  text-align: right;
  white-space: nowrap; }
.list-table .timeline-axis,
.list-table .timeline-th,
.list-table .timeline-td {
  position: relative;
  padding: 0;
  width: 6px; }
.list-table .timeline-axis::before {
  content: '';
  position: absolute;
  top: 0;
  right: 0;
  bottom: -1px;
  width: 2px;
  background-color: #aad7f0; }
.list-table .timeline-dot::after,
.list-table .timeline-dot-big::after {
  content: '';
  position: absolute;
  right: -3px;
  width: 4px;
  height: 4px;
  border-radius: 50%; }
.list-table .timeline-dot::after {
  top: 10px;
  background-color: #69808d;
  border: 2px solid #2b2b2b; }
.list-table .timeline-dot-big::after {
  top: 11px;
  background-color: #2b2b2b;
  border: 2px solid #69808d; }

.list-table .problem-expand-td {
  border-bottom-color: transparent; }
.list-table .problem-nested:not([class*='flh-']):not(.row-selected) {
  background-color: #1e1e1e; }
  .list-table .problem-nested:not([class*='flh-']):not(.row-selected) td,
  .list-table .problem-nested:not([class*='flh-']):not(.row-selected) .timeline-axis,
  .list-table .problem-nested:not([class*='flh-']):not(.row-selected) .timeline-td {
    border-bottom-color: #1e1e1e; }
.list-table .problem-nested-small {
  font-size: 11px; }
  .list-table .problem-nested-small .tag {
    font-size: 12px; }
.list-table.compact-view .problem-nested:not([class*='flh-']):not(.row-selected) td,
.list-table.compact-view .problem-nested:not([class*='flh-']):not(.row-selected) .timeline-axis,
.list-table.compact-view .problem-nested:not([class*='flh-']):not(.row-selected) .timeline-td, .list-table.has-highlighted-rows .problem-nested:not([class*='flh-']):not(.row-selected) td,
.list-table.has-highlighted-rows .problem-nested:not([class*='flh-']):not(.row-selected) .timeline-axis,
.list-table.has-highlighted-rows .problem-nested:not([class*='flh-']):not(.row-selected) .timeline-td {
  border-bottom-style: none; }
.list-table.compact-view [class*='flh-']:not(.row-selected):not(:hover) a:not(.green):not(.green.btn-icon:not([disabled])):not(.green.btn-link:not([disabled])):not(.green.link-action:not(:hover):not(:focus)):not(.green.link-alt:not(:hover):not(:focus)),
.list-table.compact-view [class*='flh-']:not(.row-selected):not(:hover) td,
.list-table.compact-view [class*='flh-']:not(.row-selected):not(:hover) sup, .list-table.has-highlighted-rows [class*='flh-']:not(.row-selected):not(:hover) a:not(.green):not(.green.btn-icon:not([disabled])):not(.green.btn-link:not([disabled])):not(.green.link-action:not(:hover):not(:focus)):not(.green.link-alt:not(:hover):not(:focus)),
.list-table.has-highlighted-rows [class*='flh-']:not(.row-selected):not(:hover) td,
.list-table.has-highlighted-rows [class*='flh-']:not(.row-selected):not(:hover) sup {
  color: #000000; }
.list-table.compact-view [class*='flh-']:not(.row-selected):not(:hover) .tag, .list-table.has-highlighted-rows [class*='flh-']:not(.row-selected):not(:hover) .tag {
  color: #ffffff;
  background-color: #4f4f4f; }
.list-table.compact-view [class*='flh-']:not(.row-selected):not(:hover) .btn-icon,
.list-table.compact-view [class*='flh-']:not(.row-selected):not(:hover) .icon, .list-table.has-highlighted-rows [class*='flh-']:not(.row-selected):not(:hover) .btn-icon,
.list-table.has-highlighted-rows [class*='flh-']:not(.row-selected):not(:hover) .icon {
  background-color: #2b2b2b; }
.list-table.has-highlighted-rows .problem-unack-fg,
.list-table.has-highlighted-rows .problem-ack-fg {
  color: #cc0000; }
.list-table.has-highlighted-rows .ok-unack-fg,
.list-table.has-highlighted-rows .ok-ack-fg {
  color: #009900; }
.list-table.has-highlighted-rows .link-alt:active, .list-table.has-highlighted-rows .link-alt:not(:hover):not(:focus) {
  color: #000000;
  border-bottom-color: #000000; }
.list-table.has-highlighted-rows .entity-count {
  border: 1px solid #2b2b2b; }

.no-data-message {
  display: grid;
  align-content: center;
  height: 100%;
  color: #737373;
  text-align: center; }
  .no-data-message[class^="zi-"], .no-data-message[class*=" zi-"] {
    font-size: 16px; }
    .no-data-message[class^="zi-"]::before, .no-data-message[class*=" zi-"]::before {
      display: block;
      margin-bottom: 8px; }
  .no-data-message .no-data-description {
    font-size: 14px;
    margin-top: 4px; }

.menu-main > li {
  line-height: 16px; }
  .menu-main > li.is-selected > a {
    background: #202020;
    border-left-color: #c5c5c5;
    color: #ffffff; }
  .menu-main > li.is-expanded > a, .menu-main > li.is-expanded > a:focus {
    background: #0d0d0d;
    border-left-color: transparent;
    color: #ffffff; }
  .menu-main > li:not(.is-expanded) .submenu {
    max-height: 0 !important; }
  .menu-main > li > a {
    color: #f2f2f2; }
    .menu-main > li > a:hover, .menu-main > li > a:focus {
      background: #0e0e0e;
      border-left-color: #ffffff;
      color: #ffffff; }
.menu-main .has-submenu > a {
  padding-right: 34px; }
  .menu-main .has-submenu > a::after {
    position: absolute;
    top: 17px;
    right: 13px;
    font-size: 8px;
    content: "";
    transform: rotate(0deg);
    transition: transform 0.3s; }
.menu-main .has-submenu.is-expanded > a[class^="zi-"]::before, .menu-main .has-submenu.is-expanded > a[class*=" zi-"]::before {
  color: #9c9c9c; }
.menu-main .has-submenu.is-expanded > a[class^="zi-"]:hover::before, .menu-main .has-submenu.is-expanded > a[class^="zi-"]:focus::before, .menu-main .has-submenu.is-expanded > a[class*=" zi-"]:hover::before, .menu-main .has-submenu.is-expanded > a[class*=" zi-"]:focus::before {
  color: #ffffff; }
.menu-main .has-submenu.is-expanded > a::after {
  transform: rotate(-180deg); }
.menu-main [class^="zi-"]::before, .menu-main [class*=" zi-"]::before {
  margin-right: 10px;
  color: #9c9c9c;
  font-size: 24px;
  line-height: 0;
  transition: color 0.2s ease-out; }
.menu-main a {
  position: relative;
  display: block;
  padding: 12px 14px 12px 44px;
  font-size: 14px;
  line-height: inherit;
  white-space: nowrap;
  border-left: 3px solid transparent;
  transition: color 0.3s, border-color 0.3s, background-color 0.3s; }
  .menu-main a:active, .menu-main a:hover, .menu-main a:link, .menu-main a:visited {
    border-bottom: 0; }
  .menu-main a[class^="zi-"], .menu-main a[class*=" zi-"] {
    padding-left: 10px; }
  .menu-main a:hover[class^="zi-"]::before, .menu-main a:hover[class*=" zi-"]::before, .menu-main a:focus[class^="zi-"]::before, .menu-main a:focus[class*=" zi-"]::before {
    color: inherit; }
.menu-main .submenu {
  position: relative;
  padding-left: 24px;
  overflow: hidden;
  background: #0d0f11;
  transition: max-height 0.3s ease;
  will-change: max-height; }
  .menu-main .submenu > li {
    line-height: 14px; }
    .menu-main .submenu > li.is-selected > a {
      background: #1c1c1c;
      border-left-color: #c5c5c5;
      color: #ffffff; }
      .menu-main .submenu > li.is-selected > a:hover, .menu-main .submenu > li.is-selected > a:focus {
        background: #0e0e0e;
        color: #ffffff; }
    .menu-main .submenu > li:not(:last-child) {
      border-bottom: 1px solid #0d0f11; }
  .menu-main .submenu [class^="zi-"]::before, .menu-main .submenu [class*=" zi-"]::before {
    margin-right: 8px;
    font-size: 16px;
    color: inherit; }
  .menu-main .submenu a {
    padding: 8px 20px;
    font-size: 12px;
    background: #2b2b2b;
    color: #f2f2f2; }
    .menu-main .submenu a:hover, .menu-main .submenu a:focus {
      background: #0e0e0e;
      border-left-color: #ffffff;
      color: #ffffff; }
  .menu-main .submenu::before, .menu-main .submenu::after {
    content: '';
    position: absolute;
    right: 0;
    left: 0;
    z-index: 999;
    box-shadow: 0 0 6px 2px rgba(0, 0, 0, 0.35); }
  .menu-main .submenu::before {
    top: 0; }
  .menu-main .submenu::after {
    bottom: 0; }
  .menu-main .submenu .submenu {
    position: fixed;
    top: 0;
    left: 100%;
    padding: 1px;
    box-shadow: 4px 4px 8px 0 rgba(43, 43, 43, 0.35);
    transition: none; }
    .menu-main .submenu .submenu li {
      border: 0; }
      .menu-main .submenu .submenu li a {
        padding: 6px 20px;
        border-left: 0;
        background-color: #454545; }
      .menu-main .submenu .submenu li.is-selected a {
        background-color: #1c1c1c; }
      .menu-main .submenu .submenu li a:hover,
      .menu-main .submenu .submenu li a:focus {
        background-color: #0e0e0e; }
    .menu-main .submenu .submenu::before, .menu-main .submenu .submenu::after {
      display: none; }
  .menu-main .submenu .has-submenu > a::after {
    transform: rotate(90deg);
    top: 11px; }
  .menu-main .submenu .has-submenu:not(.is-expanded) a::after {
    transform: rotate(-90deg); }
  .menu-main .submenu .has-submenu:not(.is-expanded) .submenu {
    left: -9999px !important; }

.menu-user {
  padding: 12px 0; }
  .menu-user li {
    line-height: 16px; }
  .menu-user [class^="zi-"]::before, .menu-user [class*=" zi-"]::before {
    margin-right: 16px;
    font-size: 16px;
    line-height: 0;
    transition: color 0.2s ease-out; }
  .menu-user a,
  .menu-user span {
    position: relative;
    display: block;
    padding: 12px 14px 12px 44px;
    font-size: 12px;
    white-space: nowrap;
    border-left: 3px solid transparent;
    transition: color 0.3s ease-out; }
  .menu-user [class^="zi-"], .menu-user [class*=" zi-"] {
    padding-left: 13px; }
    .menu-user [class^="zi-"]::before, .menu-user [class*=" zi-"]::before {
      color: #9c9c9c; }
  .menu-user a {
    color: #c2c2c2; }
    .menu-user a:hover, .menu-user a:focus {
      border-bottom: 0;
      border-color: transparent;
      color: #ffffff; }
      .menu-user a:hover[class^="zi-"]::before, .menu-user a:hover[class*=" zi-"]::before, .menu-user a:focus[class^="zi-"]::before, .menu-user a:focus[class*=" zi-"]::before {
        color: inherit; }
  .menu-user span {
    color: #808080; }
  .menu-user > li.is-selected > a {
    background: #202020;
    border-left-color: #c5c5c5;
    color: #ffffff; }
  .menu-user > li.is-expanded > a, .menu-user > li.is-expanded > a:focus {
    background: #0d0d0d;
    border-left-color: transparent;
    color: #ffffff; }
  .menu-user > li.is-expanded > a[class^="zi-"]::before, .menu-user > li.is-expanded > a[class*=" zi-"]::before {
    color: #9c9c9c; }
  .menu-user > li:not(.is-expanded) .submenu {
    max-height: 0 !important; }
  .menu-user .has-submenu > a {
    padding-right: 34px; }
    .menu-user .has-submenu > a::after {
      position: absolute;
      top: 17px;
      right: 13px;
      font-size: 8px;
      content: "";
      transform: rotate(0deg);
      transition: transform 0.3s; }
  .menu-user .has-submenu.is-expanded > a::after {
    transform: rotate(-180deg); }
  .menu-user .submenu {
    position: relative;
    padding-left: 24px;
    overflow: hidden;
    background: #0d0f11;
    transition: max-height 0.3s ease;
    will-change: max-height; }
    .menu-user .submenu > li {
      line-height: 14px; }
      .menu-user .submenu > li.is-selected > a {
        background: #1c1c1c;
        border-left-color: #c5c5c5;
        color: #ffffff; }
        .menu-user .submenu > li.is-selected > a:hover, .menu-user .submenu > li.is-selected > a:focus {
          background: #0e0e0e;
          color: #ffffff; }
      .menu-user .submenu > li:not(:last-child) {
        border-bottom: 1px solid #0d0f11; }
    .menu-user .submenu [class^="zi-"]::before, .menu-user .submenu [class*=" zi-"]::before {
      margin-right: 8px;
      color: inherit; }
    .menu-user .submenu a {
      padding: 8px 20px;
      background: #2b2b2b;
      color: #f2f2f2; }
      .menu-user .submenu a:hover, .menu-user .submenu a:focus {
        background: #0e0e0e;
        border-left-color: #ffffff;
        color: #ffffff; }
    .menu-user .submenu::before, .menu-user .submenu::after {
      content: '';
      position: absolute;
      right: 0;
      left: 0;
      z-index: 999;
      box-shadow: 0 0 6px 2px rgba(0, 0, 0, 0.35); }
    .menu-user .submenu::before {
      top: 0; }
    .menu-user .submenu::after {
      bottom: 0; }
    .menu-user .submenu .submenu {
      position: fixed;
      top: 0;
      left: 100%;
      padding: 1px;
      box-shadow: 4px 4px 8px 0 rgba(43, 43, 43, 0.35);
      transition: none; }
      .menu-user .submenu .submenu li {
        border: 0; }
        .menu-user .submenu .submenu li a {
          padding: 6px 20px;
          border-left: 0;
          background-color: #454545; }
        .menu-user .submenu .submenu li.is-selected a {
          background-color: #1c1c1c; }
        .menu-user .submenu .submenu li a:hover,
        .menu-user .submenu .submenu li a:focus {
          background-color: #0e0e0e; }
      .menu-user .submenu .submenu::before, .menu-user .submenu .submenu::after {
        display: none; }
    .menu-user .submenu .has-submenu > a::after {
      transform: rotate(90deg);
      top: 11px; }
    .menu-user .submenu .has-submenu:not(.is-expanded) a::after {
      transform: rotate(-90deg); }
    .menu-user .submenu .has-submenu:not(.is-expanded) .submenu {
      left: -9999px !important; }

output {
  position: relative;
  box-sizing: border-box;
  display: grid;
  align-items: center;
  flex-shrink: 0;
  min-height: 48px;
  padding: 6px 10px;
  margin: 10px;
  background: #2b2b2b;
  border-style: solid;
  border-width: 2px 2px 2px 60px;
  border-radius: 4px; }
  output::before {
    position: absolute;
    top: 6px;
    left: -45px;
    font-size: 32px;
    color: rgba(255, 255, 255, 0.9);
    text-shadow: 0 0 5px rgba(0, 0, 0, 0.9); }
  output > span {
    font-size: 1.167em; }
  output .msg-buttons {
    margin-top: 4px;
    text-align: right; }
    output .msg-buttons button {
      font-size: 1.167em; }
      output .msg-buttons button:not(:first-child) {
        margin-left: 10px; }
  output .msg-details {
    overflow: hidden; }
    output .msg-details ul {
      max-height: 300px;
      overflow-y: auto; }
    output .msg-details li {
      white-space: pre-line; }
      output .msg-details li:only-child {
        margin-left: 0; }
        output .msg-details li:only-child::before {
          display: none; }
  output.collapsible {
    grid-template: 16.8px 1fr / max-content 1fr;
    grid-gap: 6px 10px;
    transition: grid-template-rows 0.3s, grid-row-gap 0.3s; }
    output.collapsible .link-action {
      line-height: 14px; }
      output.collapsible .link-action .arrow-down,
      output.collapsible .link-action .arrow-up {
        margin-left: 3px;
        pointer-events: none; }
    output.collapsible .msg-buttons,
    output.collapsible .msg-details {
      grid-column: 1 / -1; }
    output.collapsible .msg-details {
      padding-top: 6px;
      border-top: 1px dashed;
      border-color: inherit;
      opacity: 1;
      transform: scaleY(1) translateY(0);
      transition: padding-top 0.3s, opacity 0.3s, transform 0.3s; }
    output.collapsible.collapsed {
      grid-template-rows: 32px 0fr;
      grid-row-gap: 0; }
      output.collapsible.collapsed .msg-details {
        padding-top: 0;
        border-top: none;
        opacity: 0;
        transform: scaleY(0) translateY(50%); }
  output.msg-bad {
    border-color: #dc3838; }
    output.msg-bad::before {
      content: ""; }
    output.msg-bad .msg-buttons button {
      color: #f2f2f2;
      background-color: transparent;
      border-color: #dc3838; }
      output.msg-bad .msg-buttons button:hover {
        color: #f2f2f2;
        background-color: #5e737e;
        border-color: #5e737e; }
      output.msg-bad .msg-buttons button:active, output.msg-bad .msg-buttons button:focus {
        color: #f2f2f2;
        background-color: #5e737e;
        border-color: #5e737e; }
  output.msg-good {
    border-color: #29a847; }
    output.msg-good::before {
      content: ""; }
  output.msg-warning {
    border-color: #f4af25; }
    output.msg-warning::before {
      content: ""; }
  output.msg-global {
    max-width: 475px;
    margin: 5% auto 0; }
  output.msg-global-footer {
    position: fixed;
    bottom: 0;
    z-index: 1000;
    display: none;
    padding-top: 4px;
    padding-bottom: 4px;
    margin: 0;
    min-height: 37px; }
    output.msg-global-footer::before {
      top: 1px; }

.multiselect-control {
  display: flex;
  position: relative;
  vertical-align: top;
  white-space: nowrap; }
  .multiselect-control .btn-split {
    margin-right: 0; }
    .multiselect-control .btn-split li:first-child {
      margin-left: 0; }

.multiselect {
  display: flex;
  flex-direction: column;
  position: relative;
  margin-right: 3px;
  min-height: 24px;
  white-space: normal; }
  .multiselect.active {
    border-color: #768d99; }
  .multiselect input[type="text"] {
    border: 0;
    background: none;
    box-sizing: border-box;
    width: 100%;
    padding-bottom: 2px;
    min-height: 20px; }
  .multiselect.search-disabled input[type="text"] {
    position: absolute;
    top: 0;
    left: 0;
    width: 0;
    padding: 0;
    margin: 0; }
  .multiselect .multiselect-list {
    display: flex;
    flex-direction: row;
    flex-wrap: wrap;
    padding-left: 2px;
    padding-bottom: 2px;
    line-height: 18px; }
    .multiselect .multiselect-list li {
      display: block;
      padding-left: 0 !important;
      margin: 2px 2px 0 0;
      overflow: hidden;
      white-space: nowrap;
      cursor: default; }
      .multiselect .multiselect-list li.selected .subfilter-enabled {
        background-color: #696969; }
    .multiselect .multiselect-list .subfilter-enabled {
      padding-right: 0;
      display: flex;
      flex-direction: row;
      line-height: 18px; }
      .multiselect .multiselect-list .subfilter-enabled > span:first-child {
        overflow: hidden;
        text-overflow: ellipsis; }
      .multiselect .multiselect-list .subfilter-enabled .btn-icon {
        flex-shrink: 0;
        margin-left: 0;
        color: #fff; }
        .multiselect .multiselect-list .subfilter-enabled .btn-icon:hover, .multiselect .multiselect-list .subfilter-enabled .btn-icon:focus {
          color: #dfe4e7; }
        .multiselect .multiselect-list .subfilter-enabled .btn-icon:active, .multiselect .multiselect-list .subfilter-enabled .btn-icon:focus {
          box-shadow: none; }
    .multiselect .multiselect-list .reference span::before {
      font-size: 16px;
      color: #4796c4; }
  .multiselect .disabled {
    background-color: #2b2b2b;
    opacity: 1; }
    .multiselect .disabled .subfilter-enabled {
      background-color: #383838;
      color: #808080;
      -webkit-text-fill-color: #808080; }
      .multiselect .disabled .subfilter-enabled .btn-icon {
        pointer-events: none;
        color: #808080;
        -webkit-text-fill-color: #808080; }
  .multiselect [aria-readonly] .subfilter-enabled,
  .multiselect [aria-readonly] .btn-icon {
    cursor: default;
    background-color: #383838; }
  .multiselect div[aria-live] {
    overflow: hidden;
    position: absolute;
    left: 0;
    right: 0;
    top: 0;
    bottom: 0;
    z-index: -1; }

.multiselect-suggest, .multiselect-matches {
  padding: 0 5px;
  white-space: normal;
  color: #737373;
  background-color: #383838; }
  .multiselect-suggest li, .multiselect-matches li {
    display: block;
    height: inherit;
    line-height: normal;
    color: #f2f2f2;
    padding: .4em 5px;
    margin: 0 -5px;
    cursor: pointer;
    overflow: hidden;
    text-overflow: ellipsis;
    transition: background-color 0.2s ease-out; }
  .multiselect-suggest.multiselect-suggest-grouped li, .multiselect-suggest-grouped.multiselect-matches li {
    padding-left: 25px; }
    .multiselect-suggest.multiselect-suggest-grouped li.suggest-group, .multiselect-suggest-grouped.multiselect-matches li.suggest-group {
      padding-left: 5px;
      font-weight: bold; }
  .multiselect-suggest .suggest-hover, .multiselect-matches .suggest-hover {
    background-color: #454545; }
  .multiselect-suggest .suggest-new, .multiselect-matches .suggest-new {
    border-top: 1px solid #4f4f4f; }
    .multiselect-suggest .suggest-new span:first-child, .multiselect-matches .suggest-new span:first-child {
      font-weight: bold; }

.multiselect-matches {
  padding: .4em 5px;
  color: #f2f2f2;
  font-weight: bold; }

.multiselect-available {
  position: absolute;
  z-index: 20000;
  border: 1px solid #383838;
  overflow-y: auto;
  box-shadow: 0 6px 10px 0 rgba(0, 0, 0, 0.5); }

.suggest-found {
  font-weight: bold;
  color: #e99003; }

.navigation-tree .navigation-tree-node-info {
  display: flex;
  align-items: center;
  gap: 4px;
  height: 18px;
  padding: 9px 8px;
  border-bottom: 1px solid #383838; }
  .navigation-tree .navigation-tree-node-info:hover {
    background-color: #383838; }
.navigation-tree .navigation-tree-node-info-helpers {
  display: flex; }
.navigation-tree .navigation-tree-node-info-primary {
  display: flex;
  flex-grow: 1;
  gap: 4px;
  min-width: 0;
  height: 18px;
  line-height: 18px; }
.navigation-tree [data-level="0"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 0px; }
.navigation-tree [data-level="1"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 15px; }
.navigation-tree [data-level="2"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 30px; }
.navigation-tree [data-level="3"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 45px; }
.navigation-tree [data-level="4"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 60px; }
.navigation-tree [data-level="5"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 75px; }
.navigation-tree [data-level="6"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 90px; }
.navigation-tree [data-level="7"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 105px; }
.navigation-tree [data-level="8"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 120px; }
.navigation-tree [data-level="9"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 135px; }
.navigation-tree [data-level="10"] > .navigation-tree-node-info .navigation-tree-node-info-level {
  width: 150px; }
.navigation-tree .navigation-tree-node-info-arrow {
  width: 16px; }
  .navigation-tree .navigation-tree-node-info-arrow button {
    display: block;
    padding: 0;
    cursor: pointer;
    line-height: 16px;
    text-align: center;
    border: 0;
    background-color: transparent; }
    .navigation-tree .navigation-tree-node-info-arrow button .arrow-right {
      border-left-color: #737373; }
    .navigation-tree .navigation-tree-node-info-arrow button .arrow-down {
      position: relative;
      top: -1px;
      border-top-color: #737373; }
    .navigation-tree .navigation-tree-node-info-arrow button:hover .arrow-right, .navigation-tree .navigation-tree-node-info-arrow button:focus .arrow-right {
      border-left-color: #4796c4; }
    .navigation-tree .navigation-tree-node-info-arrow button:hover .arrow-down, .navigation-tree .navigation-tree-node-info-arrow button:focus .arrow-down {
      border-top-color: #4796c4; }
.navigation-tree .navigation-tree-node-children {
  display: none; }
.navigation-tree .navigation-tree-node-is-open > .navigation-tree-node-children {
  display: block; }
.navigation-tree .navigation-tree-node-info-name {
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis; }
.navigation-tree .navigation-tree-group-uncategorized > .navigation-tree-node-info .navigation-tree-node-info-name {
  font-style: italic;
  padding-left: 0.2em;
  padding-right: 0.2em; }
.navigation-tree .navigation-tree-node-is-item > .navigation-tree-node-info {
  cursor: pointer; }
  .navigation-tree .navigation-tree-node-is-item > .navigation-tree-node-info .navigation-tree-node-info-name {
    color: #4796c4; }
.navigation-tree .navigation-tree-node-info-problems {
  display: flex; }

.navigation-tree-node-info-group-hint .navigation-tree-group-uncategorized {
  font-style: italic; }

.navigation-tree-node-info-problems-hint .navigation-tree-node-info-problems-hint-severity {
  display: flex;
  align-items: center;
  white-space: pre;
  line-height: 18px; }
  .navigation-tree-node-info-problems-hint .navigation-tree-node-info-problems-hint-severity .navigation-tree-node-info-problems-hint-severity-color {
    margin-right: 10px; }

.navigation-tree-node-info-problems-hint .navigation-tree-node-info-problems-hint-severity-color {
  width: 10px;
  height: 10px; }

.radio-list-control {
  display: inline-block;
  height: 24px;
  line-height: 0;
  white-space: nowrap; }
  .radio-list-control li {
    position: relative;
    display: inline-block;
    padding: 0; }
    .radio-list-control li:not(:last-child) {
      margin-right: -1px; }
    .radio-list-control li:first-child {
      border-top-left-radius: 3px;
      border-bottom-left-radius: 3px; }
      .radio-list-control li:first-child label {
        border-top-left-radius: 2px;
        border-bottom-left-radius: 2px; }
    .radio-list-control li:last-child {
      border-top-right-radius: 3px;
      border-bottom-right-radius: 3px; }
      .radio-list-control li:last-child label {
        border-top-right-radius: 2px;
        border-bottom-right-radius: 2px; }
  .radio-list-control label {
    display: inline-block;
    border: 1px solid #4f4f4f;
    line-height: 14px;
    padding: 4px 11px;
    cursor: pointer;
    transition: background-color 0.2s ease-out; }
  .radio-list-control input[type="radio"] {
    position: absolute !important;
    top: auto !important;
    width: 1px !important;
    height: 1px !important;
    opacity: 0; }
    .radio-list-control input[type="radio"] + label {
      color: #f2f2f2;
      border-color: #4f4f4f;
      background-color: #383838; }
      .radio-list-control input[type="radio"] + label:hover {
        background-color: #414141; }
      .radio-list-control input[type="radio"] + label:active {
        border-color: #4f4f4f;
        background-color: #414141; }
    .radio-list-control input[type="radio"]:checked + label {
      color: #ffffff;
      -webkit-text-fill-color: #ffffff;
      border-color: #4f4f4f;
      background-color: #4f4f4f;
      position: relative;
      z-index: 1; }
    .radio-list-control input[type="radio"]:focus + label {
      box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }
    .radio-list-control input[type="radio"][readonly] + label, .radio-list-control input[type="radio"][disabled] + label {
      transition: none;
      cursor: default;
      box-shadow: none; }
    .radio-list-control input[type="radio"][readonly]:checked + label, .radio-list-control input[type="radio"][disabled]:checked + label {
      background-color: #a3a3a3; }
    .radio-list-control input[type="radio"][readonly]:checked + label {
      color: #f2f2f2;
      -webkit-text-fill-color: #f2f2f2; }

.range-control {
  position: relative;
  display: inline-flex; }
  .range-control input[type=range] {
    width: calc(100% + 10px);
    height: 20px;
    margin: 2px -5px;
    padding: 0;
    vertical-align: top;
    opacity: 0;
    cursor: pointer;
    -webkit-appearance: none;
    -moz-appearance: none; }
    .range-control input[type=range]:focus {
      outline: none; }
    .range-control input[type=range]::-webkit-slider-thumb, .range-control input[type=range]::-moz-range-thumb {
      height: 10px;
      width: 10px;
      margin-top: 1px;
      opacity: 0; }
    .range-control input[type=range]::-webkit-slider-runnable-track {
      height: 10px;
      opacity: 0; }
  .range-control input[type=text] {
    width: 31px; }
  .range-control > div {
    position: relative;
    width: calc(100% - 10px - 31px);
    margin-right: 10px; }
  .range-control .range-control-track,
  .range-control .range-control-progress {
    position: absolute;
    top: 50%;
    left: 0;
    height: 2px;
    margin-top: -1px;
    cursor: pointer; }
  .range-control .range-control-track {
    width: 100%;
    background: #c0c0c0; }
  .range-control .range-control-progress {
    background: #4796c4; }
  .range-control .range-control-thumb {
    position: absolute;
    top: 50%;
    width: 10px;
    height: 10px;
    margin-top: -5px;
    margin-left: -5px;
    background: #4796c4;
    border-radius: 50%; }
  .range-control.range-control-focus .range-control-thumb {
    margin-top: -6.66667px;
    margin-left: -6.66667px;
    background: #4796c4;
    border: 2px solid #8599a4; }
  .range-control.range-control-focus .range-control-progress {
    background: #4796c4; }
  .range-control.disabled {
    opacity: 1; }
    .range-control.disabled input[type=range] {
      cursor: default; }
    .range-control.disabled .range-control-progress,
    .range-control.disabled .range-control-thumb {
      background: #c0c0c0; }
  .range-control datalist {
    position: absolute;
    top: 50%;
    display: flex;
    width: 100%;
    margin-top: -1px; }
    .range-control datalist option {
      flex: 1 0 0;
      height: 2px;
      padding: 0;
      font-size: 0;
      border-left: 1px solid #0e1012; }

section {
  background-color: #2b2b2b;
  border: 1px solid #303030; }
  section .section-head {
    display: flex;
    height: 32px;
    line-height: 32px; }
    section .section-head h4 {
      padding: 0 10px;
      margin-right: auto;
      overflow: hidden;
      text-overflow: ellipsis;
      font-weight: bold;
      line-height: inherit;
      color: white; }
    section .section-head .section-toggle {
      max-height: 24px; }
  section .section-foot {
    padding: 0 10px;
    text-align: right;
    line-height: 32px;
    color: #737373; }
  section.collapsed .section-body,
  section.collapsed .section-foot {
    display: none; }
  section:not(:last-child) {
    margin-bottom: 10px; }
  section .list-table {
    border: 0; }
    section .list-table tbody tr:last-child td {
      border-bottom: 1px solid #383838; }
    section .list-table td:first-child,
    section .list-table th:first-child {
      padding-left: 10px; }
    section .list-table td:last-child,
    section .list-table th:last-child {
      padding-right: 10px; }
    section .list-table.no-data tbody tr:last-child td {
      border-bottom: none; }

.service-info {
  margin: -10px 0;
  border-left: 4px solid #59db8f; }
  .service-info .service-info-grid {
    display: grid;
    grid-template-columns: max-content auto;
    row-gap: 10px;
    column-gap: 10px;
    padding: 10px 10px 0; }
    .service-info .service-info-grid:last-of-type {
      padding-bottom: 10px; }
    .service-info .service-info-grid .tags-wrapper {
      min-width: 1%; }
      .service-info .service-info-grid .tags-wrapper .btn-icon {
        height: 18px;
        width: 18px; }
  .service-info .service-info-label::after {
    content: ':'; }
  .service-info .service-info-value .service-info-value-sla {
    display: inline-block;
    word-break: break-word;
    line-height: 18px; }
    .service-info .service-info-value .service-info-value-sla:not(:last-child) {
      margin-right: 10px; }
    .service-info .service-info-value .service-info-value-sla .btn-icon {
      margin-left: 5px; }
  .service-info .service-name {
    padding-bottom: 5px;
    font-size: 1.5em; }
  .service-info .service-actions {
    justify-self: end; }
  .service-info .service-status {
    display: inline-block;
    padding: 0 4px;
    border-radius: 3px;
    background-color: #59db8f;
    color: #ffffff; }
  .service-info.service-status-na-bg {
    border-color: var(--severity-color-na-bg); }
    .service-info.service-status-na-bg .service-status {
      background-color: var(--severity-color-na-bg);
      color: #2a353a; }
  .service-info.service-status-info-bg {
    border-color: var(--severity-color-info-bg); }
    .service-info.service-status-info-bg .service-status {
      background-color: var(--severity-color-info-bg);
      color: #00268e; }
  .service-info.service-status-average-bg {
    border-color: var(--severity-color-average-bg); }
    .service-info.service-status-average-bg .service-status {
      background-color: var(--severity-color-average-bg);
      color: #733100; }
  .service-info.service-status-warning-bg {
    border-color: var(--severity-color-warning-bg); }
    .service-info.service-status-warning-bg .service-status {
      background-color: var(--severity-color-warning-bg);
      color: #734d00; }
  .service-info.service-status-high-bg {
    border-color: var(--severity-color-high-bg); }
    .service-info.service-status-high-bg .service-status {
      background-color: var(--severity-color-high-bg);
      color: #52190b; }
  .service-info.service-status-disaster-bg {
    border-color: var(--severity-color-disaster-bg); }
    .service-info.service-status-disaster-bg .service-status {
      background-color: var(--severity-color-disaster-bg);
      color: #4b0c0c; }

.status-na-bg,
.status-info-bg,
.status-warning-bg,
.status-average-bg,
.status-high-bg,
.status-disaster-bg, .status-green, .status-disabled, .status-grey, .status-yellow, .status-red {
  box-sizing: border-box;
  display: inline-block;
  min-width: 16px;
  height: 16px;
  padding-right: 3px;
  padding-left: 3px;
  font-size: 0.917em;
  line-height: 15px;
  text-align: center;
  vertical-align: top;
  border-width: 1px;
  border-style: solid;
  border-radius: 2px; }

.status-container {
  display: inline-block;
  height: 16px;
  white-space: nowrap; }
  .status-container:not(:first-of-type) {
    margin-left: 3px; }
  .status-container [class^="status-"]:not(:last-of-type) {
    padding-right: 4px;
    border-right-style: none; }
  .status-container [class^="status-"]:not(:first-of-type) {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0; }
  .status-container [class^="status-"]:not(:last-of-type) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0; }
  .status-container [class^="status-"][data-hintbox='1'] {
    cursor: pointer; }

.status-na-bg,
.status-info-bg,
.status-warning-bg,
.status-average-bg,
.status-high-bg,
.status-disaster-bg {
  border-color: rgba(43, 43, 43, 0.2); }

.status-green {
  color: #ffffff;
  background-color: #34af67;
  border-color: #2f9f5e; }

.status-disabled {
  color: #525252;
  background-color: #2b2b2b;
  border-color: rgba(242, 242, 242, 0.2); }

.status-grey {
  color: #525252;
  background-color: #2b2b2b;
  border-color: #3d3d3d; }

.status-yellow {
  color: #ffffff;
  background-color: #f1a50b;
  border-color: #e79e0b; }

.status-red {
  color: #ffffff;
  background-color: #d64e4e;
  border-color: #d23d3d; }

.subfilter-option-grid {
  grid-template-columns: minmax(75px, max-content) auto;
  display: grid;
  margin: 5px 0; }
  .subfilter-option-grid .subfilter-options {
    grid-column: 2 / -1; }

.subfilter,
.subfilter-enabled {
  display: inline-block;
  height: 18px;
  position: relative; }

.subfilter {
  margin-right: 10px;
  white-space: nowrap;
  line-height: 14px;
  height: 16px;
  margin-bottom: 2px; }

.subfilter-enabled {
  background-color: #4f4f4f;
  border-radius: 2px;
  color: #f2f2f2;
  padding: 0 4px; }
  .subfilter-enabled .link-action {
    color: #f2f2f2;
    text-decoration: none; }
    .subfilter-enabled .link-action:focus {
      color: #f2f2f2;
      border-bottom: 2px solid rgba(242, 242, 242, 0.7); }
    .subfilter-enabled .link-action:hover {
      color: #f2f2f2;
      border-bottom: 1px solid rgba(242, 242, 242, 0.7); }
  .subfilter-enabled sup {
    color: #8f8f8f; }

.svg-gauge {
  fill: #f2f2f2; }
  .svg-gauge .svg-gauge-description {
    text-align: center;
    white-space: nowrap;
    color: #f2f2f2; }
    .svg-gauge .svg-gauge-description div {
      overflow: hidden;
      text-overflow: ellipsis; }
  .svg-gauge .svg-gauge-thresholds-arc-sector,
  .svg-gauge .svg-gauge-empty-arc-sector {
    fill: #383838; }
  .svg-gauge .svg-gauge-value-arc-sector {
    fill: #69808d; }
  .svg-gauge .svg-gauge-value-arc-sector,
  .svg-gauge .svg-gauge-empty-arc-sector {
    transition: fill 0.5s ease-in-out; }
  .svg-gauge .svg-gauge-needle {
    fill: #f2f2f2;
    stroke: #f2f2f2;
    vector-effect: non-scaling-stroke;
    transition: fill 0.5s ease-in-out, stroke 0.5s ease-in-out; }
    .svg-gauge .svg-gauge-needle.svg-gauge-needle-light {
      stroke: rgba(0, 0, 0, 0.2); }
    .svg-gauge .svg-gauge-needle.svg-gauge-needle-dark {
      stroke: rgba(255, 255, 255, 0.3); }
  .svg-gauge .svg-gauge-label-left {
    text-anchor: end; }
  .svg-gauge .svg-gauge-label-center {
    text-anchor: middle; }
  .svg-gauge .svg-gauge-value-and-units {
    text-align: center;
    white-space: nowrap;
    color: #f2f2f2; }
    .svg-gauge .svg-gauge-value-and-units > div {
      margin: auto;
      overflow: hidden;
      text-overflow: ellipsis; }
      .svg-gauge .svg-gauge-value-and-units > div .svg-gauge-value {
        overflow: hidden;
        text-overflow: ellipsis; }
  .svg-gauge .svg-gauge-value-and-units-horizontal .svg-gauge-value,
  .svg-gauge .svg-gauge-value-and-units-horizontal .svg-gauge-units {
    display: inline; }
  .svg-gauge .svg-gauge-value-and-units-horizontal .svg-gauge-space {
    display: inline-block; }
  .svg-gauge .svg-gauge-value-and-units-vertical .svg-gauge-value,
  .svg-gauge .svg-gauge-value-and-units-vertical .svg-gauge-units {
    overflow: hidden;
    text-overflow: ellipsis; }
  .svg-gauge .svg-gauge-no-data {
    text-anchor: middle;
    fill: #737373; }

.svg-gauge .svg-gauge-needle.svg-gauge-needle-dark {
  stroke: rgba(255, 255, 255, 0.2); }

.svg-graph {
  display: block; }

.svg-point-highlight {
  fill: #f2f2f2;
  fill-opacity: .5; }

.svg-graph-percentile text,
.svg-graph-simple-trigger text {
  filter: drop-shadow(1px 1px 0 rgba(128, 128, 128, 0.1)); }

.svg-graph-legend {
  position: relative;
  top: 4px;
  box-sizing: border-box;
  display: grid;
  grid-template-columns: repeat(var(--columns, 4), auto);
  grid-gap: 0 10px;
  justify-content: start;
  max-height: calc(var(--lines) * 18px);
  margin: 0 10px;
  overflow: hidden;
  line-height: 18px; }
  .svg-graph-legend .svg-graph-legend-header {
    color: #737373; }
    .svg-graph-legend .svg-graph-legend-header:first-of-type {
      grid-column-start: 2; }
  .svg-graph-legend .svg-graph-legend-item {
    min-width: 100%;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap; }
    .svg-graph-legend .svg-graph-legend-item::before {
      position: relative;
      top: -3px;
      content: ' ';
      display: inline-block;
      width: 10px;
      height: 4px;
      margin-right: 4px;
      background: var(--color); }
  .svg-graph-legend .svg-graph-legend-no-data {
    grid-column: 2 / -1;
    color: #737373; }
  .svg-graph-legend .svg-graph-legend-header,
  .svg-graph-legend .svg-graph-legend-no-data,
  .svg-graph-legend .svg-graph-legend-value {
    text-align: right; }
  .svg-graph-legend.svg-graph-legend-statistic {
    grid-template-columns: auto repeat(3, max-content);
    justify-content: stretch; }
    .svg-graph-legend.svg-graph-legend-statistic .svg-graph-legend-item {
      grid-column-start: 1; }

.svg-pie-chart .svg-pie-chart-arc-placeholder {
  fill: transparent; }
.svg-pie-chart .svg-pie-chart-space {
  stroke: #2b2b2b;
  stroke-linecap: round; }
.svg-pie-chart .svg-pie-chart-arc-no-data-outer {
  fill: #383838; }
.svg-pie-chart .svg-pie-chart-arc-no-data-inner {
  fill: #2b2b2b; }
.svg-pie-chart .svg-pie-chart-total-value div {
  text-align: center;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  fill: #f2f2f2; }
.svg-pie-chart .svg-pie-chart-total-value-no-data {
  text-anchor: middle;
  fill: #737373; }

.svg-pie-chart-legend {
  position: relative;
  top: 4px;
  box-sizing: border-box;
  display: grid;
  grid-template-columns: repeat(var(--columns, 4), auto);
  grid-gap: 0 10px;
  justify-content: start;
  max-height: calc(var(--lines) * 18px);
  margin: 0 10px;
  overflow: hidden;
  line-height: 18px; }
  .svg-pie-chart-legend .svg-pie-chart-legend-item {
    min-width: 100%;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap; }
    .svg-pie-chart-legend .svg-pie-chart-legend-item::before {
      position: relative;
      top: -3px;
      content: ' ';
      display: inline-block;
      width: 10px;
      height: 4px;
      margin-right: 4px;
      background: var(--color); }
  .svg-pie-chart-legend .svg-pie-chart-legend-header {
    grid-column-start: 2;
    color: #737373; }
  .svg-pie-chart-legend .svg-pie-chart-legend-no-data {
    color: #737373; }
  .svg-pie-chart-legend .svg-pie-chart-legend-header,
  .svg-pie-chart-legend .svg-pie-chart-legend-value,
  .svg-pie-chart-legend .svg-pie-chart-legend-no-data {
    text-align: right;
    white-space: nowrap; }
  .svg-pie-chart-legend.svg-pie-chart-legend-show-value {
    grid-template-columns: auto max-content;
    justify-content: stretch; }
    .svg-pie-chart-legend.svg-pie-chart-legend-show-value .svg-pie-chart-legend-item {
      grid-column-start: 1; }

.svg-pie-chart-hintbox {
  display: flex;
  align-items: center;
  white-space: pre; }
  .svg-pie-chart-hintbox .svg-pie-chart-hintbox-color {
    width: 10px;
    height: 10px;
    margin-right: 10px; }

.svg-honeycomb .svg-honeycomb-container {
  --color: #eeeeee;
  --fill: #3d5059;
  --stroke: transparent;
  --no-data-color: #eeeeee;
  --no-data-fill: #383838; }
.svg-honeycomb feDropShadow {
  flood-opacity: var(--shadow-opacity, 0);
  transition: flood-opacity 0.3s; }
.svg-honeycomb .svg-honeycomb-cell {
  --px: calc(var(--x) + var(--dx, 0px));
  --py: calc(var(--y) + var(--dy, 0px));
  --cell-stroke: var(--stroke);
  --cell-stroke-width: var(--stroke-width);
  cursor: pointer; }
  .svg-honeycomb .svg-honeycomb-cell path {
    fill: var(--fill);
    stroke: var(--cell-stroke);
    stroke-width: var(--cell-stroke-width);
    transform: translate(var(--px), var(--py)) scale(var(--scale, 1));
    transition: transform 0.3s, fill 0.6s, stroke 0.6s, stroke-width 0.3s; }
    .svg-honeycomb .svg-honeycomb-cell path.svg-honeycomb-backdrop {
      --fill: transparent;
      --cell-stroke: transparent;
      transform: translate(var(--x), var(--y)); }
  .svg-honeycomb .svg-honeycomb-cell foreignObject {
    transition: x 0.3s, y 0.3s, width 0.3s, height 0.3s; }
  .svg-honeycomb .svg-honeycomb-cell .svg-honeycomb-content {
    display: grid;
    grid-auto-rows: minmax(max-content, auto);
    align-content: stretch;
    align-items: center;
    height: 100%;
    padding: 0 2em;
    text-align: center;
    line-height: var(--line-height);
    color: var(--color); }
  .svg-honeycomb .svg-honeycomb-cell .svg-honeycomb-label {
    overflow: hidden;
    transition: max-height 0.3s, font-size 0.3s; }
    .svg-honeycomb .svg-honeycomb-cell .svg-honeycomb-label > div {
      overflow: hidden;
      text-overflow: ellipsis;
      white-space: nowrap; }
  .svg-honeycomb .svg-honeycomb-cell:hover {
    --stroke: var(--stroke-selected); }
  .svg-honeycomb .svg-honeycomb-cell.svg-honeycomb-cell-selected {
    --cell-stroke: var(--stroke-selected);
    --cell-stroke-width: calc(var(--stroke-width) * 2); }
.svg-honeycomb .svg-honeycomb-cell-no-data,
.svg-honeycomb .svg-honeycomb-cell-other {
  --color: var(--no-data-color);
  --fill: var(--no-data-fill);
  --stroke-selected: transparent;
  cursor: default; }
.svg-honeycomb .svg-honeycomb-cell-other-ellipsis {
  transform: translate(var(--px), var(--py)); }
  .svg-honeycomb .svg-honeycomb-cell-other-ellipsis circle {
    fill: var(--color); }

.dashboard-is-edit-mode .svg-honeycomb-container:not(:hover) .svg-honeycomb-cell path,
.dashboard-is-edit-mode .svg-honeycomb-container:not(:hover) .svg-honeycomb-cell foreignObject,
.dashboard-is-edit-mode .svg-honeycomb-container:not(:hover) .svg-honeycomb-cell .svg-honeycomb-label {
  transition: none; }

.filter-container.tabfilter-container {
  padding: 0;
  border: 0 none;
  background-color: transparent;
  margin: 0; }
  .filter-container.tabfilter-container .form-buttons [name="filter_new"],
  .filter-container.tabfilter-container .form-buttons [name="filter_update"] {
    margin-right: 20px; }
  .filter-container.tabfilter-container .tabfilter-content-container {
    padding: 10px 0;
    margin-bottom: 8px;
    background-color: #2b2b2b;
    border: 1px solid #303030; }
    .filter-container.tabfilter-container .tabfilter-content-container.tabfilter-collapsed {
      padding: 0 0 10px 0; }
      .filter-container.tabfilter-container .tabfilter-content-container.tabfilter-collapsed .tabfilter-tabs-container,
      .filter-container.tabfilter-container .tabfilter-content-container.tabfilter-collapsed .form-buttons {
        display: none; }
  .filter-container.tabfilter-container nav > ul {
    display: flex;
    flex: 1;
    justify-content: space-between;
    margin-top: -3px; }
  .filter-container.tabfilter-container nav > ul > li {
    padding-top: 3px; }
    .filter-container.tabfilter-container nav > ul > li:nth-child(2) {
      flex-grow: 1;
      overflow: hidden;
      padding-bottom: 2px;
      margin: 0 3px -2px -3px;
      height: 30px; }
      .filter-container.tabfilter-container nav > ul > li:nth-child(2) ul {
        white-space: nowrap;
        margin-left: 3px;
        margin-right: 3px; }
    .filter-container.tabfilter-container nav > ul > li:nth-child(3) > ul > li {
      margin-right: 0; }
      .filter-container.tabfilter-container nav > ul > li:nth-child(3) > ul > li:not(:first-child) {
        margin-left: 5px; }
      .filter-container.tabfilter-container nav > ul > li:nth-child(3) > ul > li.disabled {
        opacity: 1; }
    .filter-container.tabfilter-container nav > ul > li:nth-child(3) .selected {
      background-color: transparent; }
    .filter-container.tabfilter-container nav > ul > li:nth-child(1), .filter-container.tabfilter-container nav > ul > li:nth-child(3) {
      white-space: nowrap; }
  .filter-container.tabfilter-container nav {
    margin-bottom: -1px; }
    .filter-container.tabfilter-container nav li {
      display: inline-block;
      height: 22px;
      line-height: 22px;
      vertical-align: top; }
      .filter-container.tabfilter-container nav li button {
        vertical-align: middle;
        margin-top: 0; }
  .filter-container.tabfilter-container .tabfilter-item-label {
    display: inline-block;
    border-radius: 2px;
    border: 1px solid transparent;
    color: #f2f2f2;
    margin-right: 2px;
    margin-left: 2px; }
    .filter-container.tabfilter-container .tabfilter-item-label.unsaved .tabfilter-item-link {
      font-style: italic; }
      .filter-container.tabfilter-container .tabfilter-item-label.unsaved .tabfilter-item-link::before {
        content: '*'; }
      .filter-container.tabfilter-container .tabfilter-item-label.unsaved .tabfilter-item-link[data-counter]::after {
        display: none; }
    .filter-container.tabfilter-container .tabfilter-item-label.selected.expanded {
      position: relative;
      background-color: #2b2b2b;
      border: 1px solid #303030;
      border-bottom: 0;
      border-bottom-right-radius: 0;
      border-bottom-left-radius: 0;
      padding-bottom: 7px;
      vertical-align: baseline; }
      .filter-container.tabfilter-container .tabfilter-item-label.selected.expanded .tabfilter-item-link {
        color: #f2f2f2; }
      .filter-container.tabfilter-container .tabfilter-item-label.selected.expanded:hover {
        background: #414141; }
    .filter-container.tabfilter-container .tabfilter-item-label.selected {
      background-color: #2b2b2b;
      border-color: #303030; }
      .filter-container.tabfilter-container .tabfilter-item-label.selected .tabfilter-item-link[data-counter]::after {
        display: none; }
    .filter-container.tabfilter-container .tabfilter-item-label.focused {
      box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }
      .filter-container.tabfilter-container .tabfilter-item-label.focused .btn-icon:active, .filter-container.tabfilter-container .tabfilter-item-label.focused .btn-icon:focus {
        box-shadow: none; }
    .filter-container.tabfilter-container .tabfilter-item-label.disabled {
      box-shadow: none;
      pointer-events: none; }
    .filter-container.tabfilter-container .tabfilter-item-label.sortable-item-dragging {
      background-color: #2b2b2b; }
    .filter-container.tabfilter-container .tabfilter-item-label:first-child .tabfilter-item-link {
      padding: 0 11px 7px 11px; }
      .filter-container.tabfilter-container .tabfilter-item-label:first-child .tabfilter-item-link::before {
        vertical-align: middle; }
  .filter-container.tabfilter-container .tabfilter-item-link {
    position: relative;
    display: inline-block;
    padding: 0 8px 7px 8px;
    margin-bottom: -7px;
    line-height: 22px;
    color: #f2f2f2;
    border: 1px transparent; }
  .filter-container.tabfilter-container .tabfilter-edit {
    margin: -1px 2px -1px -6px; }
  .filter-container.tabfilter-container .tabfilter-tabs .tabfilter-item-label:not(:first-child) .tabfilter-item-link::before {
    content: ' ';
    position: absolute;
    top: 1px;
    left: -3px;
    display: inline-block;
    height: 20px;
    border-left: 1px solid transparent; }
  .filter-container.tabfilter-container .tabfilter-tabs .tabfilter-item-label:not(:first-child) .tabfilter-item-link.separated::before {
    border-left: 1px solid #768d99; }
  .filter-container.tabfilter-container .tabfilter-tabs.sortable-dragging .tabfilter-item-label:not(:first-child) .tabfilter-item-link::before {
    transition: border-left-color 0.3s; }
  .filter-container.tabfilter-container .tabfilter-tabs.sortable-dragging .tabfilter-item-label.focused {
    box-shadow: none; }
  .filter-container.tabfilter-container .tabfilter-tabs-container .filter-container.time-selection-container {
    border: 0 none; }
  .filter-container.tabfilter-container .btn-time {
    padding: 0 11px;
    margin: -1px;
    color: #f2f2f2;
    border: 1px solid #69808d;
    border-radius: 2px;
    height: 24px;
    box-sizing: border-box; }
    .filter-container.tabfilter-container .btn-time:hover {
      background-color: #5e737e;
      border-color: #5e737e; }
    .filter-container.tabfilter-container .btn-time:active, .filter-container.tabfilter-container .btn-time:focus {
      background-color: #5e737e;
      border-color: #5e737e;
      box-shadow: none; }
  .filter-container.tabfilter-container .expanded .btn-time {
    background-color: transparent;
    border-color: transparent; }
  .filter-container.tabfilter-container .disabled .btn-time, .filter-container.tabfilter-container .disabled .btn-time:hover, .filter-container.tabfilter-container .disabled .btn-time:focus, .filter-container.tabfilter-container .disabled .btn-time:active {
    color: #525252;
    background-color: transparent;
    border-color: #3d3d3d;
    box-shadow: none; }

[data-counter]::after {
  margin-left: .4em;
  padding: 0.1em 0.4em;
  font-size: 0.917em;
  background-color: #4f4f4f;
  border-radius: 2px;
  color: #ffffff;
  content: attr(data-counter); }

[data-counter=""]::after {
  display: none; }

.menu-popup .menu-popup-item[data-counter]::after {
  display: block;
  position: absolute;
  top: 2px;
  right: 15px;
  line-height: 18px; }
.menu-popup .menu-popup-item[data-counter] {
  padding-right: 35px; }

.ui-tabs-nav {
  height: 30px;
  line-height: 30px;
  border-bottom: 1px solid #383838; }
  .ui-tabs-nav li {
    display: inline-block;
    outline: none; }
    .ui-tabs-nav li .btn,
    .ui-tabs-nav li button {
      margin-left: 5px; }
  .ui-tabs-nav .ui-tabs-anchor:not(.btn) {
    border: none;
    padding: 8px 10px;
    transition: background-color 0.2s ease-out; }
    .ui-tabs-nav .ui-tabs-anchor:not(.btn):hover, .ui-tabs-nav .ui-tabs-anchor:not(.btn):focus, .ui-tabs-nav .ui-tabs-anchor:not(.btn):active {
      background-color: #383838; }
  .ui-tabs-nav .ui-tabs-active .ui-tabs-anchor:not(.btn) {
    padding-bottom: 6px;
    background-color: transparent;
    border-bottom: 3px solid #69808d;
    color: #f2f2f2;
    cursor: default; }
  .ui-tabs-nav .ui-tabs-active .btn {
    height: auto;
    padding-bottom: 7px;
    margin-bottom: 0;
    color: #f2f2f2;
    background-color: #2b2b2b;
    border-color: #303030;
    border-bottom: 0;
    border-bottom-right-radius: 0;
    border-bottom-left-radius: 0; }
    .ui-tabs-nav .ui-tabs-active .btn:hover, .ui-tabs-nav .ui-tabs-active .btn:focus, .ui-tabs-nav .ui-tabs-active .btn:active {
      background-color: #383838; }
  .ui-tabs-nav .ui-state-focus .ui-tabs-anchor:not(:focus) {
    box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }
  .ui-tabs-nav .ui-state-disabled .ui-tabs-anchor {
    cursor: default;
    color: #525252;
    background-color: transparent;
    border: none; }
    .ui-tabs-nav .ui-state-disabled .ui-tabs-anchor:hover {
      background-color: transparent;
      box-shadow: none; }

.toc .toc-row {
  padding: 6px 10px;
  overflow: hidden;
  line-height: 18px;
  text-overflow: ellipsis;
  white-space: nowrap;
  background: #4f4f4f;
  border-bottom: 1px solid #383838; }
.toc .btn-link {
  color: #f2f2f2;
  border-style: none;
  border-color: #f2f2f2; }
  .toc .btn-link:focus {
    border-style: solid; }
.toc .toc-arrow {
  position: relative;
  margin-left: 12px; }
  .toc .toc-arrow .arrow-down,
  .toc .toc-arrow .arrow-right {
    position: absolute;
    left: -12px;
    top: 50%;
    transform: translateY(-50%); }
  .toc .toc-arrow .arrow-down {
    border-top-color: #f2f2f2; }
  .toc .toc-arrow .arrow-right {
    border-left-color: #f2f2f2; }
.toc .toc-sublist .toc-row {
  padding-left: 20px;
  background-color: transparent; }
.toc .toc-sublist .toc-sublist .toc-row {
  padding-left: 50px; }
.toc .toc-sublist .btn-link {
  color: #f2f2f2;
  border-color: #f2f2f2; }
.toc .toc-sublist .arrow-down {
  border-top-color: #4f4f4f; }
.toc .toc-sublist .arrow-right {
  border-left-color: #4f4f4f; }

z-bar-gauge {
  position: relative;
  display: block;
  height: 24px; }
  z-bar-gauge::part(bar) {
    position: absolute;
    top: 0;
    left: 0;
    border: 1px solid #4f4f4f;
    border-radius: 3px; }

.z-select,
z-select {
  display: inline-grid; }
  .z-select:focus,
  z-select:focus {
    outline: none; }
  .z-select ul > li.default-option,
  z-select ul > li.default-option {
    font-style: italic; }
  .z-select .focusable,
  z-select .focusable {
    position: relative;
    width: 100%;
    height: 24px;
    padding: 3px 16px 3px 4px;
    overflow: hidden;
    font-size: 1em;
    line-height: initial;
    vertical-align: baseline;
    text-align: left;
    text-overflow: ellipsis;
    word-break: break-all;
    white-space: pre;
    cursor: default;
    border-radius: 0; }
    .z-select .focusable::after,
    z-select .focusable::after {
      position: absolute;
      top: 50%;
      right: 4px;
      font-size: 10px;
      content: "";
      transform: translateY(-50%); }
    .z-select .focusable:active, .z-select .focusable:hover, .z-select .focusable:focus,
    z-select .focusable:active,
    z-select .focusable:hover,
    z-select .focusable:focus {
      color: #f2f2f2;
      background-color: #383838;
      border-color: #4f4f4f;
      box-shadow: none; }
    .z-select .focusable.red, .z-select .focusable.red.link-action:not(:hover):not(:focus), .z-select .focusable.red.link-alt:not(:hover):not(:focus),
    z-select .focusable.red,
    z-select .focusable.red.btn-icon:not([disabled]),
    z-select .focusable.red.btn-link:not([disabled]),
    z-select .focusable.red.link-action:not(:hover):not(:focus),
    z-select .focusable.red.link-alt:not(:hover):not(:focus) {
      color: #e45959; }
      .z-select .focusable.red::after, .z-select .focusable.red.link-action:not(:focus):not(:hover)::after, .z-select .focusable.red.link-alt:not(:focus):not(:hover)::after,
      z-select .focusable.red::after,
      z-select .focusable.red.btn-icon:not([disabled])::after,
      z-select .focusable.red.btn-link:not([disabled])::after,
      z-select .focusable.red.link-action:not(:focus):not(:hover)::after,
      z-select .focusable.red.link-alt:not(:focus):not(:hover)::after {
        color: #f2f2f2; }
  .z-select .list,
  z-select .list {
    display: none;
    z-index: 20000;
    max-height: 362px;
    max-width: 453px;
    box-sizing: border-box;
    overflow-x: hidden;
    overflow-y: auto;
    border: 1px solid;
    box-shadow: 0 6px 10px 0 rgba(0, 0, 0, 0.5); }
    .z-select .list li,
    z-select .list li {
      display: block;
      height: inherit;
      box-sizing: border-box;
      min-height: 24px;
      max-width: 453px;
      padding: 5px;
      line-height: normal;
      color: #f2f2f2;
      text-overflow: ellipsis;
      word-break: break-word;
      user-select: none;
      white-space: nowrap;
      overflow: hidden; }
      .z-select .list li.hover,
      z-select .list li.hover {
        background-color: #454545; }
      .z-select .list li.red, .z-select .list li.red.link-action:not(:hover):not(:focus), .z-select .list li.red.link-alt:not(:hover):not(:focus),
      z-select .list li.red,
      z-select .list li.red.btn-icon:not([disabled]),
      z-select .list li.red.btn-link:not([disabled]),
      z-select .list li.red.link-action:not(:hover):not(:focus),
      z-select .list li.red.link-alt:not(:hover):not(:focus) {
        color: #e45959; }
      .z-select .list li[disabled],
      z-select .list li[disabled] {
        color: #525252; }
      .z-select .list li[optgroup]::before,
      z-select .list li[optgroup]::before {
        content: attr(optgroup);
        display: block;
        font-weight: bold; }
    .z-select .list > li > ul,
    z-select .list > li > ul {
      display: grid;
      grid-template-columns: minmax(max-content, 1fr);
      margin: 5px -5px -5px; }
      .z-select .list > li > ul li,
      z-select .list > li > ul li {
        padding-right: 15px;
        padding-left: 15px; }
    .z-select .list.fall-upwards,
    z-select .list.fall-upwards {
      box-shadow: 0 -6px 10px 0 rgba(0, 0, 0, 0.5); }
  .z-select.is-expanded > ul,
  z-select.is-expanded > ul {
    display: block;
    position: fixed; }
  .z-select[disabled] .focusable,
  z-select[disabled] .focusable {
    pointer-events: none; }

form.dashboard-widget-clock .fields-group.fields-group-date, form.dashboard-widget-clock .fields-group.fields-group-time, form.dashboard-widget-clock .fields-group.fields-group-tzone {
  display: grid;
  grid-template-columns: repeat(2, minmax(80px, max-content) 1fr);
  align-items: center;
  column-gap: 10px;
  row-gap: 5px; }
form.dashboard-widget-clock .fields-group.fields-group-time .field-format {
  grid-column: 4 / -1; }
form.dashboard-widget-clock .fields-group.fields-group-tzone .form-field.field-tzone-timezone, form.dashboard-widget-clock .fields-group.fields-group-tzone .form-field.field-tzone-format {
  grid-column: 2 / -1; }

div.dashboard-widget-clock .clock-digital {
  box-sizing: border-box;
  height: 100%;
  padding: 10px;
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: center; }
  div.dashboard-widget-clock .clock-digital .clock-date,
  div.dashboard-widget-clock .clock-digital .clock-time,
  div.dashboard-widget-clock .clock-digital .clock-time-zone,
  div.dashboard-widget-clock .clock-digital .clock-disabled {
    max-width: 100%;
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    line-height: 1.14; }
  div.dashboard-widget-clock .clock-digital .clock-time-zone span:first-of-type {
    margin-right: 0.2em; }
  div.dashboard-widget-clock .clock-digital .clock-time-zone.separated {
    text-align: center; }
    div.dashboard-widget-clock .clock-digital .clock-time-zone.separated span {
      display: block;
      overflow: hidden;
      text-overflow: ellipsis;
      margin-right: 0; }
  div.dashboard-widget-clock .clock-digital .bold {
    font-weight: bold; }
  div.dashboard-widget-clock .clock-digital .clock-disabled {
    color: #737373;
    font-weight: bold; }

form.dashboard-widget-gauge .fields-group.fields-group-colors, form.dashboard-widget-gauge .fields-group.fields-group-description, form.dashboard-widget-gauge .fields-group.fields-group-value, form.dashboard-widget-gauge .fields-group.fields-group-value-arc, form.dashboard-widget-gauge .fields-group.fields-group-needle, form.dashboard-widget-gauge .fields-group.fields-group-scale, form.dashboard-widget-gauge .fields-group.fields-group-thresholds {
  display: grid;
  grid-template-columns: minmax(100px, max-content) 1fr max-content minmax(95px, max-content);
  align-items: center;
  column-gap: 10px;
  row-gap: 5px; }
  form.dashboard-widget-gauge .fields-group.fields-group-colors label, form.dashboard-widget-gauge .fields-group.fields-group-description label, form.dashboard-widget-gauge .fields-group.fields-group-value label, form.dashboard-widget-gauge .fields-group.fields-group-value-arc label, form.dashboard-widget-gauge .fields-group.fields-group-needle label, form.dashboard-widget-gauge .fields-group.fields-group-scale label, form.dashboard-widget-gauge .fields-group.fields-group-thresholds label {
    text-align: right; }
  form.dashboard-widget-gauge .fields-group.fields-group-colors hr, form.dashboard-widget-gauge .fields-group.fields-group-description hr, form.dashboard-widget-gauge .fields-group.fields-group-value hr, form.dashboard-widget-gauge .fields-group.fields-group-value-arc hr, form.dashboard-widget-gauge .fields-group.fields-group-needle hr, form.dashboard-widget-gauge .fields-group.fields-group-scale hr, form.dashboard-widget-gauge .fields-group.fields-group-thresholds hr {
    grid-column: 1 / -1;
    margin: 0;
    width: 100%;
    border: solid #383838;
    border-width: 1px 0 0 0; }
  form.dashboard-widget-gauge .fields-group.fields-group-colors .field-fluid, form.dashboard-widget-gauge .fields-group.fields-group-description .field-fluid, form.dashboard-widget-gauge .fields-group.fields-group-value .field-fluid, form.dashboard-widget-gauge .fields-group.fields-group-value-arc .field-fluid, form.dashboard-widget-gauge .fields-group.fields-group-needle .field-fluid, form.dashboard-widget-gauge .fields-group.fields-group-scale .field-fluid, form.dashboard-widget-gauge .fields-group.fields-group-thresholds .field-fluid {
    grid-column: 2 / -1; }
  form.dashboard-widget-gauge .fields-group.fields-group-colors .offset-3, form.dashboard-widget-gauge .fields-group.fields-group-description .offset-3, form.dashboard-widget-gauge .fields-group.fields-group-value .offset-3, form.dashboard-widget-gauge .fields-group.fields-group-value-arc .offset-3, form.dashboard-widget-gauge .fields-group.fields-group-needle .offset-3, form.dashboard-widget-gauge .fields-group.fields-group-scale .offset-3, form.dashboard-widget-gauge .fields-group.fields-group-thresholds .offset-3 {
    grid-column-start: 3; }
  form.dashboard-widget-gauge .fields-group.fields-group-colors .field-size input, form.dashboard-widget-gauge .fields-group.fields-group-description .field-size input, form.dashboard-widget-gauge .fields-group.fields-group-value .field-size input, form.dashboard-widget-gauge .fields-group.fields-group-value-arc .field-size input, form.dashboard-widget-gauge .fields-group.fields-group-needle .field-size input, form.dashboard-widget-gauge .fields-group.fields-group-scale .field-size input, form.dashboard-widget-gauge .fields-group.fields-group-thresholds .field-size input {
    margin-right: 5px; }
  form.dashboard-widget-gauge .fields-group.fields-group-colors .form-field, form.dashboard-widget-gauge .fields-group.fields-group-description .form-field, form.dashboard-widget-gauge .fields-group.fields-group-value .form-field, form.dashboard-widget-gauge .fields-group.fields-group-value-arc .form-field, form.dashboard-widget-gauge .fields-group.fields-group-needle .form-field, form.dashboard-widget-gauge .fields-group.fields-group-scale .form-field, form.dashboard-widget-gauge .fields-group.fields-group-thresholds .form-field {
    line-height: 24px; }
form.dashboard-widget-gauge .fields-group.fields-group-colors {
  grid-template-columns: repeat(3, minmax(100px, max-content) 1fr); }
form.dashboard-widget-gauge .fields-group.fields-group-description .form-field:nth-child(1) {
  grid-column: 1 / -1; }
form.dashboard-widget-gauge .fields-group.fields-group-value .units-show {
  display: flex; }
  form.dashboard-widget-gauge .fields-group.fields-group-value .units-show label[for='units'] {
    width: 100%; }
form.dashboard-widget-gauge .fields-group.fields-group-scale .scale-show {
  display: flex; }
  form.dashboard-widget-gauge .fields-group.fields-group-scale .scale-show label[for='scale_show_units'] {
    width: 100%; }
form.dashboard-widget-gauge .fields-group.fields-group-thresholds .form-field:first-child {
  grid-column: 1 / -1; }

div.dashboard-widget-gauge {
  overflow-x: hidden; }
  div.dashboard-widget-gauge .dashboard-grid-widget-body a {
    display: block; }
    div.dashboard-widget-gauge .dashboard-grid-widget-body a:hover, div.dashboard-widget-gauge .dashboard-grid-widget-body a:focus, div.dashboard-widget-gauge .dashboard-grid-widget-body a:active {
      border: 0; }
    div.dashboard-widget-gauge .dashboard-grid-widget-body a svg {
      display: block; }

div.dashboard-widget-geomap {
  --hover-fill: #3a3a3a;
  --selected-fill: #4a4a4a;
  --selected-stroke: #383838; }
  div.dashboard-widget-geomap .geomap-filter {
    position: absolute;
    right: 0;
    display: none;
    padding: 5px 20px 5px 10px;
    margin-top: 5px;
    margin-right: -2px;
    background-color: #2b2b2b;
    border: 1px solid #383838; }
    div.dashboard-widget-geomap .geomap-filter.collapsed {
      display: block; }
    div.dashboard-widget-geomap .geomap-filter li {
      width: calc(100% + 20px); }
    div.dashboard-widget-geomap .geomap-filter label {
      white-space: nowrap; }
  div.dashboard-widget-geomap .marker-cluster {
    background: none; }
    div.dashboard-widget-geomap .marker-cluster > .cluster-outer-shape {
      border-radius: 50%;
      height: 30px;
      width: 30px;
      padding: 10px; }
    div.dashboard-widget-geomap .marker-cluster:hover > .cluster-outer-shape {
      background: var(--hover-fill); }
    div.dashboard-widget-geomap .marker-cluster.selected > .cluster-outer-shape {
      background: var(--selected-fill);
      outline: 2px solid var(--selected-stroke) !important; }
    div.dashboard-widget-geomap .marker-cluster div {
      text-align: center;
      border-radius: 50%;
      height: 30px;
      width: 30px; }
    div.dashboard-widget-geomap .marker-cluster span {
      line-height: 30px; }
  div.dashboard-widget-geomap .leaflet-marker-icon:focus {
    box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35);
    border-radius: 2px;
    outline: none; }
  div.dashboard-widget-geomap .leaflet-control .navigate-home-button,
  div.dashboard-widget-geomap .leaflet-control .geomap-filter-button {
    cursor: pointer; }
    div.dashboard-widget-geomap .leaflet-control .navigate-home-button::before,
    div.dashboard-widget-geomap .leaflet-control .geomap-filter-button::before {
      vertical-align: middle; }
  div.dashboard-widget-geomap .leaflet-control.disabled {
    background-color: #ffffff;
    opacity: 1; }
    div.dashboard-widget-geomap .leaflet-control.disabled .geomap-filter-button {
      cursor: default;
      opacity: .4; }
  div.dashboard-widget-geomap .leaflet-container {
    height: 100%;
    z-index: 1; }

.geomap-hintbox {
  overflow: auto;
  white-space: normal; }
  .geomap-hintbox tr[data-hostid] {
    cursor: pointer; }

form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label, form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label {
  display: grid;
  grid-template-columns: minmax(85px, max-content) 1fr max-content minmax(110px, max-content);
  align-items: center;
  column-gap: 10px;
  row-gap: 5px; }
  form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .form-field, form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .form-field {
    line-height: 24px; }
  form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .label-size, form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .label-size {
    margin-right: 5px; }
  form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label hr, form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label hr {
    grid-column: 1 / -1;
    margin: 0;
    width: 100%;
    border: solid #383838;
    border-width: 1px 0 0 0; }
form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .form-field:nth-child(1), form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .form-field:nth-child(1) {
  grid-column: 1 / -1; }
form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .field-fluid, form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .field-fluid {
  grid-column: 2 / -1; }
form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .label-units-show, form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .label-units-show {
  display: flex; }
  form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .label-units-show label[for='primary_label_units'],
  form.dashboard-widget-honeycomb .fields-group.fields-group-primary-label .label-units-show label[for='secondary_label_units'], form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .label-units-show label[for='primary_label_units'],
  form.dashboard-widget-honeycomb .fields-group.fields-group-secondary-label .label-units-show label[for='secondary_label_units'] {
    width: 100%; }
form.dashboard-widget-honeycomb .fields-group .form-row-interpolation {
  margin-bottom: 10px; }

div.dashboard-widget-honeycomb {
  overflow-x: hidden; }
  div.dashboard-widget-honeycomb .dashboard-grid-widget-body svg {
    display: block; }

div.dashboard-widget-hostavail .host-avail-table thead .host-avail-total {
  padding-top: 5px;
  border-top: 1px solid #383838; }
div.dashboard-widget-hostavail .host-avail-table th {
  width: 15%;
  min-width: 100px;
  color: #737373; }
div.dashboard-widget-hostavail .host-avail-table td {
  min-width: 95px;
  border-left: 1px dotted #383838; }
div.dashboard-widget-hostavail .host-avail-table th:last-child,
div.dashboard-widget-hostavail .host-avail-table td:last-child {
  padding-right: 5px; }
div.dashboard-widget-hostavail .host-avail-table .host-avail-true {
  color: #f2f2f2;
  background-color: #34af67; }
div.dashboard-widget-hostavail .host-avail-table .host-avail-false {
  color: #f2f2f2;
  background-color: #d64e4e; }
div.dashboard-widget-hostavail .host-avail-table .host-avail-mixed {
  color: #f2f2f2;
  background-color: #f1a50b; }
div.dashboard-widget-hostavail .host-avail-table .host-avail-unknown {
  color: #f2f2f2;
  background-color: #737373; }
div.dashboard-widget-hostavail .host-avail-table .host-avail-total {
  color: #f2f2f2;
  background-color: transparent; }

form.dashboard-widget-hostnavigator .host-grouping {
  line-height: 1.4em; }
  form.dashboard-widget-hostnavigator .host-grouping tr:not(.form_row):first-child {
    line-height: 24px; }

div.dashboard-widget-hostnavigator .host-navigator .host-navigator-limit {
  display: flex;
  justify-content: flex-end;
  height: 18px;
  padding: 9px 8px;
  color: #737373; }
div.dashboard-widget-hostnavigator .host-navigator.no-data {
  height: 100%; }

.dashboard-widget-inaccessible {
  display: grid;
  align-items: center;
  padding-right: 10px;
  padding-left: 10px;
  text-align: center;
  color: #737373; }

form.dashboard-widget-item .fields-group.fields-group-description, form.dashboard-widget-item .fields-group.fields-group-value, form.dashboard-widget-item .fields-group.fields-group-time, form.dashboard-widget-item .fields-group.fields-group-change-indicator {
  display: grid;
  grid-template-columns: minmax(100px, max-content) 1fr max-content minmax(95px, max-content);
  align-items: center;
  column-gap: 10px;
  row-gap: 5px; }
  form.dashboard-widget-item .fields-group.fields-group-description hr, form.dashboard-widget-item .fields-group.fields-group-value hr, form.dashboard-widget-item .fields-group.fields-group-time hr, form.dashboard-widget-item .fields-group.fields-group-change-indicator hr {
    grid-column: 1 / -1;
    margin: 0;
    width: 100%;
    border: solid #383838;
    border-width: 1px 0 0 0; }
  form.dashboard-widget-item .fields-group.fields-group-description .field-fluid, form.dashboard-widget-item .fields-group.fields-group-value .field-fluid, form.dashboard-widget-item .fields-group.fields-group-time .field-fluid, form.dashboard-widget-item .fields-group.fields-group-change-indicator .field-fluid {
    grid-column: 2 / -1; }
  form.dashboard-widget-item .fields-group.fields-group-description .offset-3, form.dashboard-widget-item .fields-group.fields-group-value .offset-3, form.dashboard-widget-item .fields-group.fields-group-time .offset-3, form.dashboard-widget-item .fields-group.fields-group-change-indicator .offset-3 {
    grid-column-start: 3; }
  form.dashboard-widget-item .fields-group.fields-group-description .field-size input, form.dashboard-widget-item .fields-group.fields-group-value .field-size input, form.dashboard-widget-item .fields-group.fields-group-time .field-size input, form.dashboard-widget-item .fields-group.fields-group-change-indicator .field-size input {
    margin-right: 5px; }
  form.dashboard-widget-item .fields-group.fields-group-description .form-field, form.dashboard-widget-item .fields-group.fields-group-value .form-field, form.dashboard-widget-item .fields-group.fields-group-time .form-field, form.dashboard-widget-item .fields-group.fields-group-change-indicator .form-field {
    line-height: 24px; }
form.dashboard-widget-item .fields-group.fields-group-description .form-field:nth-child(1) {
  grid-column: 1 / -1; }
form.dashboard-widget-item .fields-group.fields-group-value .units-show {
  display: flex; }
  form.dashboard-widget-item .fields-group.fields-group-value .units-show label[for='units'] {
    width: 100%; }
form.dashboard-widget-item .fields-group.fields-group-change-indicator {
  grid-template-columns: repeat(3, max-content 96px); }
  form.dashboard-widget-item .fields-group.fields-group-change-indicator .input-color-picker {
    display: block; }

div.dashboard-widget-item .dashboard-grid-widget-body > div {
  box-sizing: border-box;
  height: 100%;
  padding: 10px;
  overflow-x: hidden; }
div.dashboard-widget-item .dashboard-grid-widget-body a {
  box-sizing: border-box;
  display: flex;
  flex-direction: column;
  height: 100%;
  color: inherit; }
  div.dashboard-widget-item .dashboard-grid-widget-body a:focus, div.dashboard-widget-item .dashboard-grid-widget-body a:hover, div.dashboard-widget-item .dashboard-grid-widget-body a:visited {
    border: none; }
  div.dashboard-widget-item .dashboard-grid-widget-body a > div {
    display: flex;
    flex: 1 1 calc(100% / 3); }
div.dashboard-widget-item .dashboard-grid-widget-body .item-description,
div.dashboard-widget-item .dashboard-grid-widget-body .item-value,
div.dashboard-widget-item .dashboard-grid-widget-body .item-time {
  flex: 1 1 auto;
  max-width: 100%; }
div.dashboard-widget-item .dashboard-grid-widget-body .item-value {
  display: flex;
  flex-wrap: wrap;
  margin: 0 5px; }
  div.dashboard-widget-item .dashboard-grid-widget-body .item-value > .units:first-child, div.dashboard-widget-item .dashboard-grid-widget-body .item-value > .units:last-child {
    flex: 0 0 100%; }
  div.dashboard-widget-item .dashboard-grid-widget-body .item-value > .units:first-child {
    margin-bottom: -0.07em; }
  div.dashboard-widget-item .dashboard-grid-widget-body .item-value > .units:last-child {
    margin-top: -0.07em; }
  div.dashboard-widget-item .dashboard-grid-widget-body .item-value.type-text {
    min-width: 0; }
    div.dashboard-widget-item .dashboard-grid-widget-body .item-value.type-text .item-value-content {
      min-width: 0; }
div.dashboard-widget-item .dashboard-grid-widget-body .item-value-content {
  display: flex;
  align-items: baseline;
  overflow: hidden;
  white-space: nowrap; }
div.dashboard-widget-item .dashboard-grid-widget-body .item-description,
div.dashboard-widget-item .dashboard-grid-widget-body .item-time,
div.dashboard-widget-item .dashboard-grid-widget-body .type-text .value {
  display: block;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis; }
div.dashboard-widget-item .dashboard-grid-widget-body .item-description,
div.dashboard-widget-item .dashboard-grid-widget-body .value,
div.dashboard-widget-item .dashboard-grid-widget-body .decimals,
div.dashboard-widget-item .dashboard-grid-widget-body .units,
div.dashboard-widget-item .dashboard-grid-widget-body .item-time {
  font-size: calc(var(--content-height) * var(--widget-item-font) / 1.14);
  line-height: 1.14; }
div.dashboard-widget-item .dashboard-grid-widget-body .units:not(:last-child),
div.dashboard-widget-item .dashboard-grid-widget-body .change-indicator:not(:last-child) {
  margin-right: 5px; }
div.dashboard-widget-item .dashboard-grid-widget-body .units:not(:first-child),
div.dashboard-widget-item .dashboard-grid-widget-body .change-indicator:not(:first-child) {
  margin-left: 5px; }
div.dashboard-widget-item .dashboard-grid-widget-body .svg-arrow {
  height: calc(var(--content-height) * var(--widget-item-font) * 0.72 / 1.14); }
div.dashboard-widget-item .dashboard-grid-widget-body .item-value-no-data {
  color: #737373; }
div.dashboard-widget-item .dashboard-grid-widget-body .left {
  justify-content: flex-start;
  max-width: max-content;
  margin-right: auto; }
div.dashboard-widget-item .dashboard-grid-widget-body .center {
  justify-content: center; }
div.dashboard-widget-item .dashboard-grid-widget-body .right {
  justify-content: flex-end;
  max-width: max-content;
  margin-left: auto; }
div.dashboard-widget-item .dashboard-grid-widget-body .top {
  align-self: flex-start; }
div.dashboard-widget-item .dashboard-grid-widget-body .middle {
  align-self: center; }
div.dashboard-widget-item .dashboard-grid-widget-body .bottom {
  align-self: flex-end; }
div.dashboard-widget-item .dashboard-grid-widget-body .bold {
  font-weight: bold; }

.dashboard-widget-item .svg-arrow-up {
  fill: #3DC51D; }
.dashboard-widget-item .svg-arrow-down {
  fill: #FC5A5A; }
.dashboard-widget-item .svg-arrow-up-down {
  fill: #3DC51D; }

form.dashboard-widget-itemhistory #list_columns .overflow-ellipsis {
  max-width: 250px; }
form.dashboard-widget-itemhistory #list_columns .inaccessible {
  color: #525252; }

div.dashboard-widget-itemhistory .list-table iframe {
  vertical-align: top; }
div.dashboard-widget-itemhistory .list-table pre {
  font-family: inherit; }
div.dashboard-widget-itemhistory .list-table.show-thumbnail tbody td {
  height: 56px;
  vertical-align: middle; }
div.dashboard-widget-itemhistory .list-table .has-broadcast-data {
  cursor: pointer; }
div.dashboard-widget-itemhistory .btn-thumbnail {
  overflow: hidden; }
  div.dashboard-widget-itemhistory .btn-thumbnail:not(.btn-link) {
    width: 100px;
    height: 56px;
    background: transparent var(--thumbnail) no-repeat center;
    background-size: cover;
    border-color: transparent;
    text-indent: -9999px; }
    div.dashboard-widget-itemhistory .btn-thumbnail:not(.btn-link):not(.is-loading) {
      border: 1px solid #303030;
      border-radius: 3px; }
div.dashboard-widget-itemhistory z-bar-gauge {
  min-width: 64px;
  height: 20px; }

.dashboard-widget-itemhistory-hintbox-image .dashboard-widget-itemhistory-hintbox {
  overflow: hidden; }
.dashboard-widget-itemhistory-hintbox-image.is-loading {
  min-width: 100px;
  min-height: 56px; }
  .dashboard-widget-itemhistory-hintbox-image.is-loading::before {
    content: ' '; }
  .dashboard-widget-itemhistory-hintbox-image.is-loading img {
    opacity: 0; }
.dashboard-widget-itemhistory-hintbox-image img {
  display: block;
  max-width: 50vw;
  max-height: 50vh;
  opacity: 1;
  transition: opacity 0.3s ease-out; }

form.dashboard-widget-itemnavigator .item-grouping {
  line-height: 1.4em; }
  form.dashboard-widget-itemnavigator .item-grouping tr:not(.form_row):first-child {
    line-height: 24px; }

div.dashboard-widget-itemnavigator .item-navigator .item-navigator-limit {
  display: flex;
  justify-content: flex-end;
  height: 18px;
  padding: 9px 8px;
  color: #737373; }
div.dashboard-widget-itemnavigator .item-navigator.no-data {
  height: 100%; }

div.dashboard-widget-map .sysmap-widget-container {
  overflow: hidden;
  height: 100%;
  display: flex; }
  div.dashboard-widget-map .sysmap-widget-container svg {
    flex-grow: 1; }
    div.dashboard-widget-map .sysmap-widget-container svg .map-selections ellipse {
      fill: #3a3a3a; }
      div.dashboard-widget-map .sysmap-widget-container svg .map-selections ellipse.selected {
        fill: #4a4a4a;
        stroke: #383838;
        stroke-width: 1px; }
div.dashboard-widget-map .btn-back-map-container {
  text-align: left;
  position: absolute;
  left: -1px;
  max-width: 100%; }
  div.dashboard-widget-map .btn-back-map-container a:focus, div.dashboard-widget-map .btn-back-map-container a:hover {
    outline: none; }
    div.dashboard-widget-map .btn-back-map-container a:focus .btn-back-map, div.dashboard-widget-map .btn-back-map-container a:hover .btn-back-map {
      background-color: #768d99;
      border: #69808d solid 1px; }
      div.dashboard-widget-map .btn-back-map-container a:focus .btn-back-map .btn-back-map-content, div.dashboard-widget-map .btn-back-map-container a:hover .btn-back-map .btn-back-map-content {
        display: inline-block;
        line-height: 24px;
        padding: 0 6px 0 0;
        flex: auto; }
  div.dashboard-widget-map .btn-back-map-container .btn-back-map {
    border: #303030 solid 1px;
    border-radius: 0 2px 2px 0;
    background-color: rgba(0, 0, 0, 0.8);
    color: #ffffff;
    margin-top: 10px;
    padding: 0;
    display: flex;
    height: 24px;
    line-height: 24px;
    vertical-align: middle;
    text-decoration: none; }
    div.dashboard-widget-map .btn-back-map-container .btn-back-map .btn-back-map-content {
      display: none; }
    div.dashboard-widget-map .btn-back-map-container .btn-back-map .btn-back-map-icon {
      margin-left: 5px;
      display: inline-block;
      flex: 1 0 auto;
      width: 28px;
      height: 24px;
      vertical-align: middle; }

@keyframes fadein {
  from {
    opacity: 0; }
  to {
    opacity: 1; } }
div.dashboard-widget-navtree .navtree .tree .tree-list .margin-lvl {
  flex-shrink: 0; }
div.dashboard-widget-navtree .navtree .tree .tree-list.root > .tree-item > .tree-row > .content > .margin-lvl {
  width: 15px; }
div.dashboard-widget-navtree .navtree .tree .tree-list > .tree-item.ui-sortable-helper .content {
  padding-left: 5px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="0"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 5px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="0"] > .tree-item.ui-sortable-helper {
  margin-left: 10px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="0"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="1"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 20px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="1"] > .tree-item.ui-sortable-helper {
  margin-left: 25px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="1"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="2"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 35px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="2"] > .tree-item.ui-sortable-helper {
  margin-left: 40px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="2"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="3"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 50px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="3"] > .tree-item.ui-sortable-helper {
  margin-left: 55px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="3"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="4"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 65px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="4"] > .tree-item.ui-sortable-helper {
  margin-left: 70px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="4"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="5"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 80px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="5"] > .tree-item.ui-sortable-helper {
  margin-left: 85px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="5"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="6"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 95px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="6"] > .tree-item.ui-sortable-helper {
  margin-left: 100px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="6"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="7"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 110px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="7"] > .tree-item.ui-sortable-helper {
  margin-left: 115px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="7"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="8"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 125px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="8"] > .tree-item.ui-sortable-helper {
  margin-left: 130px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="8"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="9"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 140px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="9"] > .tree-item.ui-sortable-helper {
  margin-left: 145px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="9"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="10"] > .tree-item > .tree-row > .content > .margin-lvl {
  width: 155px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="10"] > .tree-item.ui-sortable-helper {
  margin-left: 160px; }
  div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth="10"] > .tree-item.ui-sortable-helper > .tree-row > .content > .margin-lvl {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper > .tree-row > .content > .margin-lvl {
  width: 15px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 30px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 45px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 60px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 75px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 90px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 105px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 120px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 135px; }
div.dashboard-widget-navtree .navtree .tree .tree-list[data-depth] .ui-sortable-helper .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list .tree-list > li > .tree-row > .content > .margin-lvl {
  width: 150px; }
div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row {
  display: flex;
  align-items: center;
  justify-content: space-between;
  min-width: 288px;
  height: 18px;
  padding: 9px 0;
  border-bottom: 1px solid #383838; }
  div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row:hover {
    background-color: #383838; }
  div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .content {
    position: relative;
    display: flex;
    min-width: 0; }
    div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .content .arrow {
      flex-shrink: 0;
      width: 16px;
      margin-right: 2px; }
      div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .content .arrow > .treeview {
        display: none; }
    div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .content > .item-name {
      height: 14px;
      white-space: nowrap;
      overflow: hidden;
      text-overflow: ellipsis; }
  div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .problem-icon-list {
    padding-left: 10px;
    margin-right: 10px; }
    div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .problem-icon-list .problem-icon-list-item {
      border-radius: 2px; }
  div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .tools {
    padding-left: 10px;
    margin-right: 5px;
    width: 87px;
    white-space: nowrap; }
    div.dashboard-widget-navtree .navtree .tree .tree-item .tree-row .tools .btn-icon:not(:first-child) {
      margin-left: 5px; }
div.dashboard-widget-navtree .navtree .tree .tree-item.is-parent > .tree-row > .content > .arrow > .treeview {
  display: block; }
div.dashboard-widget-navtree .navtree .tree .tree-item.no-map > .tree-row > .content > .item-name, div.dashboard-widget-navtree .navtree .tree .tree-item.inaccessible > .tree-row > .content > .item-name {
  color: #4f4f4f; }
div.dashboard-widget-navtree .navtree .tree .tree-item.ui-sortable-helper {
  background: #2b2b2b;
  border-color: #303030;
  border-width: 1px;
  border-style: solid; }
  div.dashboard-widget-navtree .navtree .tree .tree-item.ui-sortable-helper .tools {
    display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-item.opened > ul {
  display: block; }
div.dashboard-widget-navtree .navtree .tree .tree-item.closed > ul {
  display: none; }
div.dashboard-widget-navtree .navtree .tree .tree-item .sortable-error {
  border-color: transparent;
  background: rgba(228, 89, 89, 0.2); }
div.dashboard-widget-navtree .navtree .tree .highlighted-parent > .tree-row {
  background: #1f1f1f; }
div.dashboard-widget-navtree .navtree .tree .placeholder {
  background-color: #121212;
  animation: fadein .5s; }
div.dashboard-widget-navtree .treeview {
  display: inline-block;
  padding: 0;
  margin-right: 2px;
  cursor: pointer;
  line-height: 16px;
  text-align: center;
  border: 0;
  background-color: transparent; }
  div.dashboard-widget-navtree .treeview .arrow-right {
    border-left-color: #737373; }
  div.dashboard-widget-navtree .treeview .arrow-down {
    position: relative;
    top: -1px;
    border-top-color: #737373; }
  div.dashboard-widget-navtree .treeview:hover .arrow-right, div.dashboard-widget-navtree .treeview:focus .arrow-right {
    border-left-color: #4796c4; }
  div.dashboard-widget-navtree .treeview:hover .arrow-down, div.dashboard-widget-navtree .treeview:focus .arrow-down {
    border-top-color: #4796c4; }

form.dashboard-widget-piechart {
  max-width: 1330px;
  min-width: 1120px;
  width: calc(100vw - 42px); }
  form.dashboard-widget-piechart .pie-chart-widget-config-tabs {
    grid-column: 1 / -1; }
  form.dashboard-widget-piechart .pie-chart-widget-config-tabs > .tabs-nav {
    border-top: 1px solid #303030; }
  form.dashboard-widget-piechart .pie-chart-widget-config-tabs .ui-tabs-nav {
    position: sticky;
    top: 0;
    background: #2b2b2b;
    z-index: 3; }
  form.dashboard-widget-piechart .table-forms-container, form.dashboard-widget-piechart .browser-warning-container {
    margin: 0;
    border: 1px solid #303030;
    border-top: none; }
  form.dashboard-widget-piechart .table-forms-separator {
    padding: 0; }
  form.dashboard-widget-piechart .dataset-head,
  form.dashboard-widget-piechart .dataset-body.list-accordion-item-body {
    display: contents; }
  form.dashboard-widget-piechart .dataset-head .multiselect-control {
    overflow: hidden;
    margin: -2px;
    padding: 2px; }
    form.dashboard-widget-piechart .dataset-head .multiselect-control .multiselect {
      overflow: hidden;
      flex-grow: 1; }
  form.dashboard-widget-piechart .dataset-body .form-grid {
    padding-top: 0; }
    form.dashboard-widget-piechart .dataset-body .form-grid:first-child {
      grid-column-start: 5; }
  form.dashboard-widget-piechart .list-item-actions {
    grid-column: span 1 / -1;
    text-align: right; }
  form.dashboard-widget-piechart .items-list {
    grid-column: 4 / span 3; }
  form.dashboard-widget-piechart .list-vertical-accordion {
    position: relative;
    overflow: visible;
    margin-top: -5px;
    margin-bottom: 5px; }
    form.dashboard-widget-piechart .list-vertical-accordion .sortable:not(.sortable-disabled) .list-accordion-item > .drag-icon + label {
      cursor: grab; }
  form.dashboard-widget-piechart .list-accordion-item {
    position: relative;
    display: grid;
    grid-template-columns: 5px 100px 24px 24px 1fr 1fr 24px;
    grid-gap: 10px;
    align-items: start;
    padding: 5px 0; }
    form.dashboard-widget-piechart .list-accordion-item > .drag-icon {
      height: 24px;
      margin-left: -2px; }
    form.dashboard-widget-piechart .list-accordion-item > label {
      pointer-events: initial;
      overflow: hidden;
      line-height: 24px;
      text-align: left;
      text-overflow: ellipsis; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-opened::before {
      content: ' ';
      position: absolute;
      top: 24px;
      bottom: 0;
      left: 136px;
      width: 2px;
      background: #303030; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-opened .single-item-table .table-col-name {
      width: 1%; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .multiselect {
      height: 24px;
      overflow: hidden; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-head .table-forms-separator {
      border: none; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table thead,
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table tfoot,
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table .table-col-handle,
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table .table-col-type,
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table .table-col-action {
      display: none; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .dataset-body {
      display: none; }
    form.dashboard-widget-piechart .list-accordion-item.list-accordion-item-closed .items-list {
      padding-left: 0; }
  form.dashboard-widget-piechart .single-item-table .table-col-handle {
    width: 12px; }
  form.dashboard-widget-piechart .single-item-table .table-col-no {
    width: 20px; }
  form.dashboard-widget-piechart .single-item-table .table-col-color {
    width: 24px; }
  form.dashboard-widget-piechart .single-item-table .table-col-action {
    width: 55px; }
  form.dashboard-widget-piechart .single-item-table th {
    padding: 5px 5px 5px 0; }
  form.dashboard-widget-piechart .single-item-table .single-item-table-row td {
    padding: 0 5px 5px 0; }
    form.dashboard-widget-piechart .single-item-table .single-item-table-row td.table-col-name {
      padding-right: 10px; }
  form.dashboard-widget-piechart .single-item-table th:first-child,
  form.dashboard-widget-piechart .single-item-table td:first-child {
    padding-left: 5px; }
  form.dashboard-widget-piechart .single-item-table tfoot td {
    padding: 5px 5px 5px 10px; }
  form.dashboard-widget-piechart .single-item-table tbody:not(:empty) + tfoot td {
    padding-top: 0; }
  form.dashboard-widget-piechart .single-item-table .reference-hint {
    margin-right: 4px; }
    form.dashboard-widget-piechart .single-item-table .reference-hint::before {
      font-size: 16px;
      color: #4796c4; }
  form.dashboard-widget-piechart .single-item-table .unavailable-widget .reference-hint::before {
    color: #525252; }
  form.dashboard-widget-piechart .single-item-table .unavailable-widget a {
    color: #525252; }
  form.dashboard-widget-piechart .no-items-message {
    display: none;
    line-height: 24px;
    color: #737373; }
  form.dashboard-widget-piechart #merge_percent {
    margin-left: 5px; }

div.dashboard-widget-piechart {
  overflow-x: hidden; }
  div.dashboard-widget-piechart .dashboard-grid-widget-body svg {
    display: block; }

div.dashboard-widget-problemhosts .list-table tr[data-hostgroupid] {
  cursor: pointer; }

div.dashboard-widget-problems .list-table tr[data-eventid] {
  cursor: pointer; }

div.dashboard-widget-problemsbysv .list-table tr[data-hostgroupid] {
  cursor: pointer; }

form.dashboard-widget-svggraph {
  max-width: 1330px;
  min-width: 1120px;
  width: calc(100vw - 42px); }
  form.dashboard-widget-svggraph .svg-graph-preview,
  form.dashboard-widget-svggraph .graph-widget-config-tabs {
    grid-column: 1 / -1; }
  form.dashboard-widget-svggraph .svg-graph-preview {
    position: relative;
    min-width: 1110px;
    height: 300px; }
    form.dashboard-widget-svggraph .svg-graph-preview > div {
      position: absolute;
      top: 0;
      right: 0;
      left: 0;
      margin: 0 -10px;
      height: 300px;
      background: #2b2b2b;
      z-index: 3; }
  form.dashboard-widget-svggraph .graph-widget-config-tabs > .tabs-nav {
    border-top: 1px solid #303030; }
  form.dashboard-widget-svggraph .graph-widget-config-tabs .ui-tabs-nav {
    position: sticky;
    top: 0;
    background: #2b2b2b;
    z-index: 3; }
  form.dashboard-widget-svggraph .table-forms-container, form.dashboard-widget-svggraph .browser-warning-container {
    margin: -10px 0 0 0;
    border: 1px solid #303030;
    border-top: none; }
  form.dashboard-widget-svggraph .table-forms-separator {
    padding: 0; }
  form.dashboard-widget-svggraph .dataset-head,
  form.dashboard-widget-svggraph .dataset-body.list-accordion-item-body {
    display: contents; }
  form.dashboard-widget-svggraph .dataset-head .multiselect-control {
    overflow: hidden;
    margin: -2px;
    padding: 2px; }
    form.dashboard-widget-svggraph .dataset-head .multiselect-control .multiselect {
      overflow: hidden;
      flex-grow: 1; }
  form.dashboard-widget-svggraph .dataset-body .form-grid {
    padding-top: 0; }
    form.dashboard-widget-svggraph .dataset-body .form-grid:first-child {
      grid-column-start: 5; }
  form.dashboard-widget-svggraph .list-item-actions {
    grid-column: span 1 / -1;
    text-align: right; }
  form.dashboard-widget-svggraph .items-list {
    grid-column: 4 / span 3; }
  form.dashboard-widget-svggraph .list-vertical-accordion {
    position: relative;
    overflow: visible;
    margin-top: -5px;
    margin-bottom: 5px; }
    form.dashboard-widget-svggraph .list-vertical-accordion .sortable:not(.sortable-disabled) .list-accordion-item > .drag-icon + label {
      cursor: grab; }
  form.dashboard-widget-svggraph .list-accordion-item {
    position: relative;
    display: grid;
    grid-template-columns: 5px 100px 24px 24px 1fr 1fr 24px;
    grid-gap: 10px;
    align-items: start;
    padding: 5px 0; }
    form.dashboard-widget-svggraph .list-accordion-item > .drag-icon {
      height: 24px;
      margin-left: -2px; }
    form.dashboard-widget-svggraph .list-accordion-item > label {
      pointer-events: initial;
      overflow: hidden;
      line-height: 24px;
      text-align: left;
      text-overflow: ellipsis; }
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-opened::before {
      content: ' ';
      position: absolute;
      top: 24px;
      bottom: 0;
      left: 136px;
      width: 2px;
      background: #303030; }
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .multiselect {
      height: 24px;
      overflow: hidden; }
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .dataset-head .table-forms-separator {
      border: none; }
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table thead,
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table tfoot,
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table .table-col-handle,
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .dataset-head .single-item-table .table-col-action {
      display: none; }
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .dataset-body {
      display: none; }
    form.dashboard-widget-svggraph .list-accordion-item.list-accordion-item-closed .items-list {
      padding-left: 0; }
  form.dashboard-widget-svggraph .single-item-table .table-col-handle {
    width: 12px; }
  form.dashboard-widget-svggraph .single-item-table .table-col-no {
    width: 20px; }
  form.dashboard-widget-svggraph .single-item-table .table-col-color {
    width: 24px; }
  form.dashboard-widget-svggraph .single-item-table .table-col-action {
    width: 55px; }
  form.dashboard-widget-svggraph .single-item-table th {
    padding: 5px 5px 5px 0; }
  form.dashboard-widget-svggraph .single-item-table .single-item-table-row td {
    padding: 0 5px 5px 0; }
  form.dashboard-widget-svggraph .single-item-table th:first-child,
  form.dashboard-widget-svggraph .single-item-table td:first-child {
    padding-left: 5px; }
  form.dashboard-widget-svggraph .single-item-table tfoot td {
    padding: 5px 5px 5px 10px; }
  form.dashboard-widget-svggraph .single-item-table tbody:not(:empty) + tfoot td {
    padding-top: 0; }
  form.dashboard-widget-svggraph .single-item-table .reference-hint {
    margin-right: 4px; }
    form.dashboard-widget-svggraph .single-item-table .reference-hint::before {
      font-size: 16px;
      color: #4796c4; }
  form.dashboard-widget-svggraph .single-item-table .unavailable-widget .reference-hint::before {
    color: #525252; }
  form.dashboard-widget-svggraph .single-item-table .unavailable-widget a {
    color: #525252; }
  form.dashboard-widget-svggraph .override-host-field-view {
    overflow: hidden;
    margin: -2px;
    padding: 2px; }
    form.dashboard-widget-svggraph .override-host-field-view .multiselect {
      overflow: hidden;
      flex-grow: 1; }
  form.dashboard-widget-svggraph .overrides-list {
    position: relative;
    margin: -5px 0 -5px 15px; }
  form.dashboard-widget-svggraph .overrides-list-item {
    position: relative;
    display: grid;
    grid-template-columns: 1fr 1fr 24px;
    grid-gap: 5px 10px;
    align-items: start;
    padding: 5px 0;
    overflow: visible;
    margin-top: -5px;
    margin-bottom: -5px; }
    form.dashboard-widget-svggraph .overrides-list-item .drag-icon {
      position: absolute;
      top: 5px;
      left: -17px;
      height: 24px; }
    form.dashboard-widget-svggraph .overrides-list-item .multiselect-control {
      overflow: hidden;
      margin: -2px;
      padding: 2px; }
      form.dashboard-widget-svggraph .overrides-list-item .multiselect-control .multiselect {
        overflow: hidden;
        flex-grow: 1; }
  form.dashboard-widget-svggraph .overrides-foot {
    padding: 5px 0; }
  form.dashboard-widget-svggraph .overrides-options-list {
    grid-column: 1 / -1;
    padding: 0 24px 8px 0;
    border-bottom: 1px solid #383838;
    white-space: normal; }
    form.dashboard-widget-svggraph .overrides-options-list li {
      display: inline-block;
      margin-bottom: 2px;
      vertical-align: middle;
      white-space: nowrap; }
      form.dashboard-widget-svggraph .overrides-options-list li > div {
        padding: 1px 0 1px 1px;
        background-color: #4f4f4f;
        border-radius: 2px;
        line-height: 22px; }
        form.dashboard-widget-svggraph .overrides-options-list li > div span {
          padding-left: 8px;
          color: #fff; }
        form.dashboard-widget-svggraph .overrides-options-list li > div input[type=text] {
          width: 85px;
          min-height: 22px;
          border-style: none; }
      form.dashboard-widget-svggraph .overrides-options-list li:not(:last-child) {
        margin-right: 5px; }
    form.dashboard-widget-svggraph .overrides-options-list .btn-icon {
      min-width: 18px;
      margin: -1px 0;
      color: #fff; }
      form.dashboard-widget-svggraph .overrides-options-list .btn-icon:hover, form.dashboard-widget-svggraph .overrides-options-list .btn-icon:focus {
        color: #dfe4e7; }
    form.dashboard-widget-svggraph .overrides-options-list .color-picker .color-picker-preview {
      width: 20px;
      min-width: 20px;
      height: 20px;
      margin: 1px 0 1px 1px; }
  form.dashboard-widget-svggraph .no-items-message {
    display: none;
    line-height: 24px;
    color: #737373; }

div.dashboard-widget-svggraph {
  overflow-x: hidden; }

form.dashboard-widget-tophosts #list_columns .text {
  max-width: 250px; }
form.dashboard-widget-tophosts #column {
  max-width: 250px; }

div.dashboard-widget-tophosts .list-table th.center {
  text-align: center; }
div.dashboard-widget-tophosts .list-table tr[data-hostid] {
  cursor: pointer; }
div.dashboard-widget-tophosts td {
  line-height: 20px;
  height: 20px; }
div.dashboard-widget-tophosts .link-action {
  white-space: nowrap; }
div.dashboard-widget-tophosts z-bar-gauge {
  min-width: 64px;
  height: 20px; }

div.dashboard-widget-url .widget-url {
  display: block; }

div.dashboard-widget-web .list-table tr[data-hostgroupid] {
  cursor: pointer; }

.link-action {
  color: #f2f2f2;
  cursor: pointer; }
  .link-action:visited {
    color: #f2f2f2; }
  .link-action:hover {
    margin-bottom: -1px;
    color: #4796c4;
    border-bottom: 1px solid rgba(71, 150, 196, 0.35); }
  .link-action:focus {
    margin-bottom: -2px;
    outline: none;
    border-bottom: 2px solid rgba(71, 150, 196, 0.35); }

.link-action, .link-action:visited, .link-action:active {
  margin-bottom: -1px;
  border-bottom: 1px dotted;
  text-decoration: none; }

.link-alt {
  text-decoration: none;
  cursor: pointer; }
  .link-alt:link, .link-alt:visited {
    border-bottom: 1px solid rgba(118, 141, 153, 0.35); }
  .link-alt:hover {
    color: #4796c4;
    border-bottom: 1px solid rgba(71, 150, 196, 0.5); }
  .link-alt:focus {
    color: #4796c4;
    outline: none;
    border-bottom: 2px solid rgba(71, 150, 196, 0.35); }
  .link-alt:active {
    color: #4796c4;
    border-bottom: 1px solid rgba(71, 150, 196, 0.5); }

.link-external {
  position: relative; }
  .link-external::after {
    content: "";
    position: absolute;
    bottom: 1px;
    right: calc(-12px - 2px);
    font-size: 12px; }

.disabled {
  cursor: default !important;
  opacity: .35;
  background-color: transparent; }
  .disabled .subfilter-enabled {
    color: black; }
  .disabled .disabled,
  .disabled [disabled] {
    opacity: 1 !important; }

pre, pre a, pre td, pre span {
  white-space: pre-wrap;
  word-break: break-word;
  overflow-wrap: break-word; }

.arrow-up {
  border: 0;
  border-left: 4px solid transparent;
  border-right: 4px solid transparent;
  border-bottom: 6px solid #f2f2f2; }

.arrow-down {
  border: 0;
  border-left: 4px solid transparent;
  border-right: 4px solid transparent;
  border-top: 6px solid #f2f2f2; }

.arrow-left {
  border-bottom: 4px solid transparent;
  border-top: 4px solid transparent;
  border-right: 6px solid #f2f2f2; }

.arrow-right {
  border-bottom: 4px solid transparent;
  border-top: 4px solid transparent;
  border-left: 6px solid #f2f2f2; }

.arrow-up, .arrow-down, .arrow-left, .arrow-right {
  display: inline-block;
  height: 0;
  width: 0;
  font-size: 0;
  line-height: 0; }

.hor-list > li {
  display: inline-block;
  margin: 0 6px 0 0; }
  .hor-list > li:last-child {
    margin: 0; }

.host-dashboard-header-navigation,
.header-navigation {
  min-height: 29px; }
  .host-dashboard-header-navigation > ul,
  .header-navigation > ul {
    display: flex; }
    .host-dashboard-header-navigation > ul > li,
    .header-navigation > ul > li {
      margin-right: 5px;
      line-height: 24px; }
      .host-dashboard-header-navigation > ul > li > ul,
      .header-navigation > ul > li > ul {
        display: flex; }
        .host-dashboard-header-navigation > ul > li > ul li,
        .header-navigation > ul > li > ul li {
          margin-right: 5px; }
        .host-dashboard-header-navigation > ul > li > ul.breadcrumbs > li,
        .header-navigation > ul > li > ul.breadcrumbs > li {
          margin-right: 0; }
          .host-dashboard-header-navigation > ul > li > ul.breadcrumbs > li:not(:last-child)::after,
          .header-navigation > ul > li > ul.breadcrumbs > li:not(:last-child)::after {
            content: "/"; }
    .host-dashboard-header-navigation > ul li > span,
    .header-navigation > ul li > span {
      display: inline-block;
      padding: 2px 7px;
      margin: 1px 0;
      line-height: 18px;
      white-space: nowrap;
      vertical-align: top;
      text-overflow: ellipsis;
      overflow: hidden;
      max-width: 250px; }
      .host-dashboard-header-navigation > ul li > span a,
      .header-navigation > ul li > span a {
        margin-top: 2px;
        vertical-align: top;
        text-overflow: ellipsis;
        overflow: hidden;
        max-width: 200px;
        display: inline-block;
        line-height: 14px; }

.host-dashboard-header-navigation {
  display: grid;
  grid-template-columns: max-content 1fr max-content;
  column-gap: 10px; }
  .host-dashboard-header-navigation .host-dashboard-navigation {
    display: grid;
    grid-template-columns: max-content 1fr max-content;
    column-gap: 5px; }
    .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs {
      margin: -2px 0; }
    .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-controls .btn-icon:not(:first-child) {
      margin-left: 5px; }
  .host-dashboard-header-navigation .filter-space {
    display: contents; }
    .host-dashboard-header-navigation .filter-space .filter-container {
      grid-column: 1 / -1; }
    .host-dashboard-header-navigation .filter-space li .ui-tabs-anchor {
      vertical-align: top; }
    .host-dashboard-header-navigation .filter-space ul.ui-tabs-nav li:focus {
      outline: none; }
    .host-dashboard-header-navigation .filter-space .ui-tabs-nav a {
      display: inline-block;
      box-sizing: border-box;
      line-height: 22px;
      text-align: center; }

.header-navigation {
  display: inline-block;
  position: relative;
  z-index: 3;
  max-width: calc(100% - 200px); }
  .header-navigation > ul {
    flex-wrap: wrap; }
    .header-navigation > ul > li .status-container {
      margin-right: 5px; }
    .header-navigation > ul > li .status-container,
    .header-navigation > ul > li .rel-container {
      line-height: 17px;
      vertical-align: baseline; }
    .header-navigation > ul li > span.selected {
      background-color: #2b2b2b;
      border-radius: 2px;
      box-shadow: inset 0 0 0 1px #303030; }
  .header-navigation + .filter-space {
    margin-top: -29px; }

form.is-loading {
  position: relative; }

.form-submit-hidden {
  position: absolute;
  visibility: hidden;
  min-height: 0;
  max-height: 0; }

.table-paging {
  text-align: center;
  background-color: #2b2b2b;
  margin-top: -1px;
  padding: 4px 5px;
  overflow: hidden;
  position: relative;
  border: 1px solid #303030;
  border-top-color: #383838; }
  .table-paging a:focus .arrow-right {
    border-left-color: #f2f2f2; }
  .table-paging a:focus .arrow-left {
    border-right-color: #f2f2f2; }

.paging-btn-container {
  min-height: 24px;
  position: relative; }

.table-stats {
  color: #737373;
  display: inline-block;
  padding: 4px 0;
  position: absolute;
  right: 5px; }

.radio-switch {
  cursor: default;
  background-color: #383838; }

.table-paging a, .radio-switch {
  display: inline-block;
  margin-left: -1px;
  padding: 3px 11px;
  background-color: #383838;
  border: 1px solid #4f4f4f !important;
  text-decoration: none;
  color: #f2f2f2;
  outline: 0;
  transition: background-color 0.2s ease-out; }
  .table-paging a:hover, .radio-switch:hover {
    color: #f2f2f2;
    background-color: #414141; }
  .table-paging a:focus, .radio-switch:focus {
    color: #f2f2f2;
    background-color: #5e737e;
    border-color: #5e737e;
    z-index: 10;
    position: relative; }
  .table-paging a:active, .radio-switch:active {
    color: #f2f2f2;
    background-color: #414141;
    z-index: 10;
    position: relative; }
  .table-paging a:first-of-type, .radio-switch:first-of-type {
    border-radius: 2px 0 0 2px; }
  .table-paging a:last-of-type, .radio-switch:last-of-type {
    border-radius: 0 2px 2px 0; }
  .table-paging a.paging-selected, .paging-selected.radio-switch {
    color: #f2f2f2;
    background-color: #4f4f4f;
    position: relative;
    z-index: 1; }
    .table-paging a.paging-selected:hover, .paging-selected.radio-switch:hover {
      background-color: #454545; }
    .table-paging a.paging-selected:focus, .paging-selected.radio-switch:focus {
      color: #f2f2f2;
      background-color: #5e737e;
      border-color: #5e737e;
      position: relative; }
    .table-paging a.paging-selected:active, .paging-selected.radio-switch:active {
      background-color: #454545;
      position: relative; }

.table {
  display: table; }

.row {
  display: table-row; }

.cell {
  display: table-cell;
  vertical-align: top; }
  .cell.expression-cell {
    line-height: 24px; }
    .cell.expression-cell span {
      white-space: normal; }

.adm-img {
  width: 75%;
  text-align: center;
  margin: 0 auto;
  table-layout: fixed; }
  .adm-img .cell {
    vertical-align: bottom;
    padding: 20px 10px;
    width: 20%;
    text-align: center; }
    .adm-img .cell img {
      max-width: 100%; }

.cell-width {
  width: 8px; }

.nowrap {
  white-space: nowrap; }

.wordwrap {
  white-space: normal;
  word-break: break-all; }

.wordbreak {
  white-space: pre-wrap;
  word-break: break-word; }

.line-clamp {
  display: -webkit-box;
  overflow: hidden;
  -webkit-box-orient: vertical;
  -webkit-line-clamp: var(--line-clamp); }

.filter-btn-container {
  position: relative;
  text-align: right; }

.filter-container {
  position: relative;
  padding: 10px 0;
  margin-bottom: 8px;
  background-color: #2b2b2b;
  border: 1px solid #303030; }
  .filter-container .table {
    border-bottom: 1px solid #383838; }
  .filter-container .filter-forms {
    padding: 0 10px; }
  .filter-container .list-table {
    margin-bottom: -10px;
    border: 0; }
    .filter-container .list-table thead th {
      color: initial;
      padding: 8px 5px 5px;
      border-bottom-width: 1px; }

.filter-forms {
  margin: 0 auto;
  text-align: center;
  white-space: nowrap;
  background-color: #2b2b2b; }
  .filter-forms .cell {
    text-align: left;
    padding: 0 20px 5px; }
    .filter-forms .cell:first-child {
      padding-left: 0; }
    .filter-forms .cell:last-child {
      padding-right: 0;
      border-right: 0; }
    .filter-forms .cell button {
      margin: 0; }
    .filter-forms .cell .btn-icon {
      margin-left: 5px; }
    .filter-forms .cell table {
      width: auto; }
  .filter-forms button {
    margin: 10px 5px 0; }
  .filter-forms select {
    vertical-align: top; }
  .filter-forms .table-forms .table-forms-td-right td {
    padding: 0 5px 5px 0;
    vertical-align: middle; }

.btn-time-zoomout {
  color: #f2f2f2;
  background-color: transparent;
  border-color: transparent;
  vertical-align: top; }
  .btn-time-zoomout[disabled] {
    color: #525252;
    background-color: transparent;
    border-color: transparent; }

.table-forms-container, .browser-warning-container {
  margin: 0 0 10px;
  background-color: #2b2b2b;
  border: 1px solid #303030;
  padding: 10px;
  text-align: left; }
  .table-forms-container > .ui-tabs-nav, .browser-warning-container > .ui-tabs-nav {
    margin: -10px -10px 10px; }

.form-btns button {
  margin: 10px 6px 5px; }

.table-forms {
  display: table;
  width: 100%;
  color: #f2f2f2; }
  .table-forms > li {
    display: table-row; }
  .table-forms .multiselect-suggest li, .table-forms .multiselect-matches li {
    display: block; }
  .table-forms th {
    color: #737373;
    padding: 0 5px 0 0;
    text-align: left; }
    .table-forms th:last-child {
      padding: 0; }
  .table-forms tfoot .table-forms-td-right,
  .table-forms .tfoot-buttons .table-forms-td-right {
    padding-top: 5px; }
  .table-forms tfoot button,
  .table-forms .tfoot-buttons button {
    margin: 0 10px 0 0; }
  .table-forms td {
    position: relative; }
  .table-forms .table-forms-td-left {
    position: relative;
    display: table-cell;
    padding: 5px 0;
    text-align: right;
    vertical-align: top;
    width: 15%;
    white-space: nowrap; }
    .table-forms .table-forms-td-left label {
      display: block;
      height: 24px;
      line-height: 24px; }
      .table-forms .table-forms-td-left label .btn-icon,
      .table-forms .table-forms-td-left label .icon {
        margin-left: 5px; }
    .table-forms .table-forms-td-left .checkbox-radio {
      right: 0;
      margin-right: 0;
      margin-left: 0; }
  .table-forms .table-forms-td-right {
    display: table-cell;
    padding: 5px 0 5px 10px;
    vertical-align: middle;
    width: 85%;
    position: relative; }
    .table-forms .table-forms-td-right td {
      padding: 5px 5px 5px 0;
      position: relative; }
      .table-forms .table-forms-td-right td.center {
        text-align: center;
        vertical-align: middle; }
        .table-forms .table-forms-td-right td.center .btn-grey {
          margin: 3px 0; }
      .table-forms .table-forms-td-right td.overflow-break {
        max-width: 150px;
        overflow-wrap: break-word;
        white-space: initial;
        word-wrap: break-word; }
      .table-forms .table-forms-td-right td:last-child {
        padding-right: 0; }
      .table-forms .table-forms-td-right td.na-bg, .table-forms .table-forms-td-right td.normal-bg, .table-forms .table-forms-td-right td.info-bg, .table-forms .table-forms-td-right td.average-bg, .table-forms .table-forms-td-right td.warning-bg, .table-forms .table-forms-td-right td.high-bg, .table-forms .table-forms-td-right td.disaster-bg {
        padding-left: 5px; }
    .table-forms .table-forms-td-right .wrap-multiple-controls {
      display: flex; }
  .table-forms .table-subforms {
    margin-top: -5px;
    margin-bottom: -5px; }
  .table-forms h4 {
    margin-bottom: -5px; }
  .table-forms .hor-list li {
    display: inline-block; }
  .table-forms .dynamic-row > td {
    padding-top: 0; }
  .table-forms .dynamic-row-control > td {
    padding-top: 0;
    padding-bottom: 0; }
  .table-forms .btn-icon:not(:first-child),
  .table-forms .icon:not(:first-child) {
    margin-left: 5px; }
  .table-forms .btn-icon.btn-small {
    top: 1px;
    vertical-align: text-bottom; }

.table-forms-separator {
  border: 1px solid #383838;
  box-sizing: border-box;
  display: inline-block;
  padding: 5px;
  position: relative; }
  .table-forms-separator .wordbreak {
    white-space: normal; }
  @media screen and (min-width: 1200px) {
    .table-forms-separator .tags-wrapper {
      max-width: 266px; } }
  @media screen and (min-width: 1400px) {
    .table-forms-separator .tags-wrapper {
      max-width: 359px; } }
  @media screen and (min-width: 1600px) {
    .table-forms-separator .tags-wrapper {
      max-width: 452px; } }

.border-left {
  border-left: 1px solid #383838; }

.form-new-group {
  border: 5px solid #32453a;
  border-radius: 2px;
  margin-left: -5px;
  padding: 4px 0; }

.list-check-radio li {
  display: block;
  line-height: 24px; }
.list-check-radio.hor-list li {
  margin-right: 15px; }
  .list-check-radio.hor-list li:last-child {
    margin-right: 0; }
.list-check-radio label {
  display: inline-block;
  max-width: 600px;
  padding-left: 20px;
  line-height: 1.4em;
  text-indent: -20px; }
  .list-check-radio label input[type="checkbox"] {
    left: -3px;
    margin: 0; }
  .list-check-radio label input[type="radio"] {
    left: -3px;
    margin: 0; }

.list-numbered {
  counter-reset: line; }
  .list-numbered .list-numbered-item::before {
    content: counter(line);
    counter-increment: line;
    user-select: none;
    text-align: right; }

label.form-label-asterisk::before {
  color: #e45959;
  display: inline-block;
  content: '*';
  margin-right: .3em; }
label input[type="checkbox"] {
  margin-right: 3px; }
label input[type="radio"] {
  margin-right: 3px; }

input {
  font-family: Arial, Tahoma, Verdana, sans-serif;
  font-size: 1em; }
  input[type="file"] {
    padding: 1px; }

textarea {
  margin: 0;
  padding: 4px 5px;
  overflow: auto;
  font-family: Arial, Tahoma, Verdana, sans-serif;
  font-size: 1em;
  line-height: normal;
  vertical-align: top; }
  textarea[readonly]:focus {
    border-color: #768d99; }

select {
  font-family: Arial, Tahoma, Verdana, sans-serif;
  font-size: 1em;
  height: 24px;
  padding: 3px 3px 3px 0;
  background-color: #383838; }
  select option {
    color: #f2f2f2; }
  select[multiple] {
    padding: 4px 5px;
    width: 300px;
    height: 150px;
    vertical-align: top; }
    select[multiple] option {
      padding: 0; }
  select[readonly] {
    color: #f2f2f2 !important;
    pointer-events: none; }

.form-fields-inline label {
  margin: 0 8px 0 3px; }

.form-input-margin {
  display: inline-block;
  margin: 0 3px 0 0; }

.checkbox-radio {
  position: absolute !important;
  top: auto !important;
  z-index: 1;
  width: 16px !important;
  height: 16px !important;
  opacity: 0; }
  .checkbox-radio + label span {
    position: relative;
    box-sizing: border-box;
    display: inline-block;
    width: 16px;
    height: 16px;
    margin-right: 4px;
    vertical-align: text-bottom;
    background-color: #383838;
    border: 1px solid #4f4f4f; }
    .checkbox-radio + label span:active {
      border-color: #4f4f4f; }
  .checkbox-radio + label.label-pos-left span {
    margin-left: 4px;
    margin-right: 0; }
  .checkbox-radio:checked[readonly] + label span {
    transition: none; }
    .checkbox-radio:checked[readonly] + label span::after {
      border-color: #4f4f4f !important; }
  .checkbox-radio[readonly], .checkbox-radio[readonly] + label {
    pointer-events: none; }
    .checkbox-radio[readonly] span, .checkbox-radio[readonly] + label span {
      transition: none; }
  .checkbox-radio[type="checkbox"] + label span {
    border-radius: 2px; }
  .checkbox-radio[type="checkbox"]:checked + label span::after {
    content: '';
    position: absolute;
    display: block;
    width: 7px;
    height: 4px;
    top: 6px;
    left: 7px;
    border-left: 2px solid #69808d;
    border-bottom: 2px solid #69808d;
    transform: translate(-50%, -50%) rotate(-45deg); }
  .checkbox-radio[type="checkbox"]:checked[disabled] + label span::after {
    border-color: #4f4f4f !important; }
  .checkbox-radio[type="checkbox"]:checked[readonly] + label span::after {
    border-color: #f2f2f2 !important; }
  .checkbox-radio[type="radio"] + label span {
    border-radius: 50%; }
  .checkbox-radio[type="radio"]:checked + label span::after {
    content: '';
    position: absolute;
    display: block;
    width: 8px;
    height: 8px;
    top: 3px;
    left: 3px;
    background-color: #69808d;
    border-radius: 50%; }
  .checkbox-radio[type="radio"]:checked[disabled] + label span::after {
    background-color: #4f4f4f !important; }
  .checkbox-radio[type="radio"]:checked[readonly] + label span::after {
    background-color: #f2f2f2 !important; }
  .checkbox-radio:focus + label span {
    border-color: #768d99; }
  .checkbox-radio:checked[disabled] + label span {
    transition: none; }
    .checkbox-radio:checked[disabled] + label span::after {
      border-color: #4f4f4f !important; }
  .checkbox-radio[disabled] + label span {
    transition: none; }

input[type="radio"] {
  margin: 0;
  padding: 0;
  border: 0;
  vertical-align: middle;
  position: relative;
  top: -1px;
  overflow: hidden;
  width: 13px;
  height: 13px;
  background: none; }

input[type="text"], input[type="password"], input[type="search"], input[type="number"], input[type="email"], input[type="time"] {
  margin: 0;
  padding: 0 5px;
  min-height: 24px; }

.multiselect, .z-select .focusable,
z-select .focusable, .z-select .list,
z-select .list, input[type="text"], input[type="password"], input[type="search"], input[type="number"], input[type="email"], input[type="time"], input[type="file"], textarea, select {
  background-color: #383838;
  border: 1px solid #4f4f4f;
  box-sizing: border-box;
  color: #f2f2f2;
  outline: 0;
  transition: border-color 0.2s ease-out, box-shadow 0.2s ease-out; }
  .multiselect:focus, .z-select .focusable:focus,
  z-select .focusable:focus, .z-select .list:focus,
  z-select .list:focus, input[type="text"]:focus, input[type="password"]:focus, input[type="search"]:focus, input[type="number"]:focus, input[type="email"]:focus, input[type="time"]:focus, input[type="file"]:focus, textarea:focus, select:focus {
    border-color: #768d99; }
  .multiselect::placeholder, .z-select .focusable::placeholder,
  z-select .focusable::placeholder, .z-select .list::placeholder,
  z-select .list::placeholder, input[type="text"]::placeholder, input[type="password"]::placeholder, input[type="search"]::placeholder, input[type="number"]::placeholder, input[type="email"]::placeholder, input[type="time"]::placeholder, input[type="file"]::placeholder, textarea::placeholder, select::placeholder {
    color: #9d9d9d;
    -webkit-text-fill-color: #9d9d9d;
    opacity: 1; }

.radio-list-control.severity {
  vertical-align: top; }
  .radio-list-control.severity label {
    max-width: 110px;
    text-overflow: ellipsis;
    overflow: hidden; }

.table-forms .radio-list-control.severity {
  vertical-align: middle; }

.filter-space {
  box-sizing: border-box; }
  .filter-space li .ui-tabs-anchor {
    vertical-align: top; }
  .filter-space ul.ui-tabs-nav li:focus {
    outline: none; }
  .filter-space .filter-btn-container {
    height: 29px;
    z-index: 2; }
  .filter-space .ui-tabs-nav {
    height: 29px;
    border-bottom: 0; }
    .filter-space .ui-tabs-nav a {
      display: inline-block;
      box-sizing: border-box;
      line-height: 22px;
      margin-bottom: 5px;
      text-align: center; }

.time-quick {
  font-size: 12px;
  line-height: 20px; }
  .time-quick li:last-of-type a {
    margin-bottom: 0; }
  .time-quick li a {
    display: inline-block;
    padding: 2px 10px;
    margin-bottom: 2px;
    margin-left: -10px;
    border-bottom: 0; }
    .time-quick li a:hover, .time-quick li a:focus, .time-quick li a.selected {
      border-radius: 2px;
      border-bottom: 0; }
    .time-quick li a:hover {
      background: #454545;
      transition: background-color 0.2s ease-out; }
    .time-quick li a:focus {
      box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }
    .time-quick li a.selected {
      background: #4f4f4f;
      color: #ffffff;
      transition: background-color 0.2s ease-out; }

.time-selection-container {
  display: flex;
  justify-content: flex-end; }
  .time-selection-container .time-input {
    border-right: 1px solid #303030;
    padding: 0 30px 0 10px;
    text-align: right;
    white-space: nowrap; }
    .time-selection-container .time-input .time-input-error {
      margin: -8px 29px 0 0; }
    .time-selection-container .time-input ul {
      padding: 0 0 10px; }
    .time-selection-container .time-input li {
      display: inline-block;
      vertical-align: baseline;
      padding: 0 0 0 10px; }
  .time-selection-container .time-quick-range {
    text-align: right;
    white-space: nowrap;
    margin: 0 0 0 20px; }
    .time-selection-container .time-quick-range .cell {
      display: inline-flex;
      vertical-align: top;
      text-align: left;
      margin: 0 10px; }
    .time-selection-container .time-quick-range .time-quick-selected {
      display: inline-block;
      padding: 2px 10px;
      margin-left: -10px;
      color: #f2f2f2;
      background-color: #4f4f4f;
      border-bottom: 0; }

.calendar-control .btn-icon:not(:first-child) {
  margin-left: 2px; }
.calendar-control .btn-icon::before {
  font-size: 22px; }

.header-kioskmode-controls {
  position: fixed;
  top: 5px;
  right: 45px;
  z-index: 1010;
  display: flex;
  transition: opacity 1s ease-out; }
  .header-kioskmode-controls.hidden {
    opacity: 0; }
  .header-kioskmode-controls > li > ul {
    margin-right: 10px; }
  .header-kioskmode-controls ul {
    display: flex; }
    .header-kioskmode-controls ul li {
      margin-right: 10px; }

.btn-dashboard-normal {
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.5); }

.multiselect[aria-readonly], .z-select[readonly] .focusable,
z-select[readonly] .focusable, input[readonly], textarea[readonly] {
  color: #f2f2f2;
  -webkit-text-fill-color: #f2f2f2;
  background-color: #2b2b2b;
  border-color: #3d3d3d; }

.multiselect[aria-disabled], .radio-list-control input[type="radio"][readonly] + label, .radio-list-control input[type="radio"][disabled] + label, .z-select[disabled] .focusable,
z-select[disabled] .focusable, input[disabled], textarea[disabled], select option[disabled], select[disabled], select[readonly], .checkbox-radio[readonly] span, .checkbox-radio[readonly] + label span, .checkbox-radio[disabled] + label span {
  color: #525252;
  -webkit-text-fill-color: #525252;
  background-color: #2b2b2b;
  border-color: #3d3d3d; }
  .multiselect[aria-disabled]::placeholder, .radio-list-control input[type="radio"][readonly] + label::placeholder, .radio-list-control input[type="radio"][disabled] + label::placeholder, .z-select[disabled] .focusable::placeholder,
  z-select[disabled] .focusable::placeholder, input[disabled]::placeholder, textarea[disabled]::placeholder, select option[disabled]::placeholder, select[disabled]::placeholder, select[readonly]::placeholder, .checkbox-radio[readonly] span::placeholder, .checkbox-radio[readonly] + label span::placeholder, .checkbox-radio[disabled] + label span::placeholder {
    color: #404040;
    -webkit-text-fill-color: #404040;
    opacity: 1; }

.inaccessible .subfilter-enabled {
  color: #b2b2b2; }

.action-buttons {
  margin-top: 10px;
  color: #737373; }
  .action-buttons button {
    vertical-align: top; }
  .action-buttons > button:not(:first-child),
  .action-buttons .btn-split:not(:first-child) {
    margin-right: 0;
    margin-left: 10px; }
  .action-buttons .selected-item-count {
    display: inline-block;
    margin-right: 4px;
    line-height: 24px;
    vertical-align: top; }

.ui-draggable .overlay-dialogue-header {
  cursor: grab; }
.ui-draggable.ui-draggable-dragging .overlay-dialogue-header {
  cursor: grabbing; }

.setup-container {
  background-color: #2b2b2b;
  width: 766px;
  height: 420px;
  margin: 0 auto;
  margin-top: 5%;
  padding: 42px;
  vertical-align: top;
  position: relative;
  border: 1px solid #303030; }
  .setup-container h1 {
    border: 0;
    margin: 3px 0 11px;
    padding: 0; }
  .setup-container .zabbix-logo {
    margin-bottom: 40px; }
  .setup-container #tls_encryption_hint {
    font-style: italic;
    color: #737373; }

.setup-left {
  float: left;
  width: 200px;
  padding-right: 10px; }
  .setup-left ul {
    margin: -4px 0 0 -42px; }
    .setup-left ul li {
      padding: 4px 0 4px 42px; }
      .setup-left ul li.setup-left-current {
        border-left: 3px solid #69808d;
        color: #737373;
        padding-left: 39px; }

.setup-right {
  width: auto;
  margin-left: 200px; }

.setup-right-body {
  padding-right: 5px;
  max-height: 345px;
  overflow-y: auto; }
  .setup-right-body .list-table {
    border: 0; }
  .setup-right-body h1 {
    margin: 25% 0 .4em; }

.setup-title {
  margin: 109px 0 16px 0;
  color: #737373;
  font-size: 7em;
  line-height: 0.75em; }
  .setup-title span {
    display: block;
    font-size: 0.25em; }

.setup-footer {
  position: absolute;
  bottom: 32px;
  width: 766px;
  text-align: right; }
  .setup-footer div {
    float: right; }
  .setup-footer button {
    margin: 0 0 0 10px; }
    .setup-footer button.float-left {
      margin: 0;
      float: left; }

.signin-container {
  background-color: #2b2b2b;
  width: 280px;
  margin: 0 auto;
  margin-top: 5%;
  padding: 42px 42px 39px;
  border: 1px solid #303030; }
  .signin-container h1 {
    border: 0;
    margin: 0 0 17px;
    padding: 0; }
  .signin-container ul li {
    padding: 16px 0 0;
    font-size: 1.167em; }
    .signin-container ul li.sign-in-txt {
      text-align: center; }
  .signin-container label {
    display: inline-block;
    margin: 0 0 2px; }
  .signin-container button {
    font-size: 1em;
    margin: 7px 0;
    min-height: 35px;
    line-height: 35px;
    width: 100%; }

.signin-logo {
  display: flex;
  margin-bottom: 21px;
  justify-content: center; }

.signin-links {
  width: 100%;
  text-align: center;
  color: #737373;
  margin: 11px 0 0; }

.signin-container input[type="text"], .signin-container input[type="password"] {
  padding: 9px 5px;
  width: 100%; }

.qr-code {
  height: 190px;
  width: 190px;
  margin: 20px auto; }

.qr-code img {
  image-rendering: pixelated; }

.menu-popup-overlay {
  position: fixed;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 1001; }

.menu-popup {
  position: fixed;
  z-index: 1001;
  padding: 5px 15px 5px 25px;
  box-sizing: border-box;
  min-width: 166px;
  max-width: 250px;
  max-height: calc(100vh - 20px);
  overflow-y: auto;
  background-color: #2b2b2b;
  border: 1px solid #383838;
  border-bottom-color: #353535;
  box-shadow: 0 4px 20px 0 rgba(0, 0, 0, 0.5); }
  .menu-popup:focus {
    outline: none; }
  .menu-popup li {
    display: block; }
    .menu-popup li div {
      border-top: 1px solid #383838;
      margin: 5px -15px 5px -25px; }
  .menu-popup .menu-popup-item {
    display: block;
    height: 24px;
    line-height: 24px;
    cursor: pointer;
    color: #f2f2f2;
    text-decoration: none;
    border: 0;
    outline: 0;
    padding: 0 25px 0 25px;
    margin: 0 -15px 0 -25px;
    position: relative;
    transition: background-color 0.2s ease-out; }
    .menu-popup .menu-popup-item:hover, .menu-popup .menu-popup-item:focus, .menu-popup .menu-popup-item:active, .menu-popup .menu-popup-item.highlighted {
      background-color: #454545; }
    .menu-popup .menu-popup-item[class^="zi-"], .menu-popup .menu-popup-item[class*=" zi-"] {
      padding-left: 3px; }
      .menu-popup .menu-popup-item[class^="zi-"]::before, .menu-popup .menu-popup-item[class*=" zi-"]::before {
        margin-right: 4px;
        color: #69808d; }
    .menu-popup .menu-popup-item .arrow-right {
      position: absolute;
      right: 10px;
      top: 8px;
      height: auto;
      line-height: auto;
      padding: 0;
      margin: 0;
      border-left-color: #737373; }
    .menu-popup .menu-popup-item.disabled {
      background-color: transparent;
      color: #737373;
      opacity: 1; }
  .menu-popup h3 {
    height: 24px;
    line-height: 24px; }
  .menu-popup.page-title-submenu::before {
    content: '';
    position: absolute;
    top: -7px;
    left: 10px;
    width: 12px;
    height: 12px;
    background-color: #2b2b2b;
    border-top: 1px solid #383838;
    border-left: 1px solid #383838;
    transform: rotate(45deg); }

.overlay-dialogue {
  position: absolute;
  z-index: 1000;
  padding: 17px 10px 10px;
  background-color: #2b2b2b;
  border: 1px solid #383838;
  border-bottom-color: #353535;
  box-shadow: 0 4px 20px 0 rgba(0, 0, 0, 0.5); }
  .overlay-dialogue .overlay-dialogue-header {
    margin: -8px -10px 6px -10px;
    padding: 8px 10px;
    display: flex; }
    .overlay-dialogue .overlay-dialogue-header .btn-icon,
    .overlay-dialogue .overlay-dialogue-header .btn-overlay-close {
      position: initial;
      margin-top: -9px;
      margin-left: 4px; }
      .overlay-dialogue .overlay-dialogue-header .btn-icon:last-child,
      .overlay-dialogue .overlay-dialogue-header .btn-overlay-close:last-child {
        margin-right: -6px; }
    .overlay-dialogue .overlay-dialogue-header h4 {
      color: #dedede;
      font-weight: bold;
      margin-right: auto; }
    .overlay-dialogue .overlay-dialogue-header > ul {
      margin: -8px -10px -10px 10px; }
  .overlay-dialogue.modal {
    position: fixed;
    overflow: hidden;
    margin: 0 10px;
    padding: 13px 10px 10px; }
    .overlay-dialogue.modal .overlay-dialogue-header {
      margin-bottom: 12px; }
    .overlay-dialogue.modal .overlay-dialogue-body {
      overflow-y: auto;
      overflow-x: hidden;
      width: 100%;
      max-height: calc(100vh - 220px);
      max-width: inherit;
      margin: 0 -10px 8px;
      padding: 0 10px;
      position: relative; }
      .overlay-dialogue.modal .overlay-dialogue-body form {
        padding: 2px 0; }
      .overlay-dialogue.modal .overlay-dialogue-body .table-forms .table-forms-td-right {
        padding-right: 8px; }
      .overlay-dialogue.modal .overlay-dialogue-body .table-forms .table-forms-row-with-second-field {
        position: relative; }
        .overlay-dialogue.modal .overlay-dialogue-body .table-forms .table-forms-row-with-second-field .table-forms-second-column {
          position: absolute;
          top: 5px;
          right: 8px; }
      .overlay-dialogue.modal .overlay-dialogue-body .columns-wrapper .column-50:last-of-type {
        margin-left: 10px; }
    .overlay-dialogue.modal .overlay-dialogue-controls {
      text-align: right;
      padding: 0 0 10px; }
      .overlay-dialogue.modal .overlay-dialogue-controls z-select {
        text-align: left;
        padding: inherit; }
        .overlay-dialogue.modal .overlay-dialogue-controls z-select li {
          display: block;
          padding-left: 5px; }
      .overlay-dialogue.modal .overlay-dialogue-controls .inline-filter li {
        padding-left: 0; }
  .overlay-dialogue.modal-popup {
    top: 50px;
    max-width: 1024px; }
    .overlay-dialogue.modal-popup.modal-popup-small {
      max-width: 450px;
      width: 100%; }
    .overlay-dialogue.modal-popup.modal-popup-medium {
      min-width: 450px; }
    .overlay-dialogue.modal-popup.modal-popup-large {
      width: 100%; }
    .overlay-dialogue.modal-popup.modal-popup-generic {
      min-width: 650px; }
    .overlay-dialogue.modal-popup.modal-popup-static {
      max-width: 700px;
      width: 100%; }
    .overlay-dialogue.modal-popup.modal-popup-preprocessing {
      max-width: 900px;
      width: 100%; }
    .overlay-dialogue.modal-popup.modal-popup-fullscreen {
      width: calc(100vw - 40px);
      max-width: calc(100vw - 40px); }
  .overlay-dialogue .hintbox-raw-data {
    white-space: pre-wrap; }
  .overlay-dialogue .hintbox-wrap,
  .overlay-dialogue .hintbox-wrap-horizontal {
    overflow: auto;
    max-width: 800px;
    overflow-wrap: break-word; }
  .overlay-dialogue .hintbox-wrap {
    max-height: 240px; }
  .overlay-dialogue .msg-bad,
  .overlay-dialogue .msg-good,
  .overlay-dialogue .msg-warning {
    position: -webkit-sticky;
    position: sticky;
    top: 0;
    z-index: 4;
    margin: 0 0 10px; }
  .overlay-dialogue .form-grid {
    grid-template-columns: minmax(var(--label-width, 100px), max-content) 1fr; }
    .overlay-dialogue .form-grid.form-grid-1-1 {
      grid-template-columns: repeat(2, minmax(100px, max-content) 1fr); }
    .overlay-dialogue .form-grid.form-grid-3-1 {
      grid-template-columns: minmax(100px, max-content) 3fr max-content auto; }
  .overlay-dialogue .list-table {
    white-space: normal;
    border: 0; }
  .overlay-dialogue .table-paging {
    border: 0;
    border-top: 1px solid #383838; }
  .overlay-dialogue .table-forms-td-right td label {
    display: block; }
  .overlay-dialogue .maps-container {
    max-height: 128px;
    overflow-y: auto;
    margin-left: -10px; }
  .overlay-dialogue .table-forms-second-column {
    float: right;
    line-height: 24px; }

.multilineinput-control {
  display: flex;
  width: 100%; }
  .multilineinput-control input[type=text] {
    overflow: hidden;
    cursor: pointer;
    white-space: nowrap;
    text-overflow: ellipsis;
    border-right: 0;
    width: 100%; }
    .multilineinput-control input[type=text][readonly] {
      background: #383838 !important;
      border-color: #4f4f4f;
      color: #e1e3ed; }
    .multilineinput-control input[type=text].monospace-font {
      padding-top: 1px; }
  .multilineinput-control.editable {
    background-color: #383838 !important;
    border-color: #525252; }
  .multilineinput-control button {
    position: relative;
    min-width: 24px;
    margin-right: 1px;
    border-top-left-radius: 0;
    border-bottom-left-radius: 0; }
    .multilineinput-control button:not([disabled]):not([readonly]) {
      background-color: #383838;
      border-color: #4f4f4f; }
      .multilineinput-control button:not([disabled]):not([readonly]):hover {
        background-color: #414141; }
  .multilineinput-control.multilineinput-readonly input[type=text][readonly] {
    background-color: #2b2b2b !important; }
  .multilineinput-control.multilineinput-disabled input[type=text] {
    cursor: default; }
  .multilineinput-control.multilineinput-disabled button::after {
    opacity: .5; }
  .multilineinput-control button.zi-pencil::before {
    color: #69808d; }

.multilineinput-modal {
  min-width: 960px; }
  .multilineinput-modal .multilineinput-container {
    padding: 1px;
    position: relative;
    display: flex;
    overflow: hidden; }
  .multilineinput-modal .multilineinput-line-numbers {
    counter-reset: line;
    overflow: hidden;
    padding: 1px 5px 1px 0;
    position: absolute;
    left: 0;
    top: 0;
    bottom: 0; }
    .multilineinput-modal .multilineinput-line-numbers li {
      color: #737373;
      line-height: 18px;
      text-align: right;
      min-width: 24px; }
      .multilineinput-modal .multilineinput-line-numbers li::before {
        counter-increment: line;
        content: counter(line);
        user-select: none; }
  .multilineinput-modal .multilineinput-label {
    color: #737373;
    line-height: 24px; }
  .multilineinput-modal .multilineinput-textarea {
    flex: 1;
    line-height: 18px;
    tab-size: 4;
    resize: none;
    white-space: pre;
    padding: 0 5px;
    z-index: 20; }
    .multilineinput-modal .multilineinput-textarea::-webkit-scrollbar {
      height: 9px; }
  .multilineinput-modal .multilineinput-char-count {
    color: #737373;
    float: left;
    line-height: 24px; }

.textarea-flexible-container td {
  line-height: 22px; }
  .textarea-flexible-container td.textarea-flexible-parent {
    vertical-align: top; }
  .textarea-flexible-container td.macro-textarea-parent textarea {
    float: left; }
  .textarea-flexible-container td.macro-textarea-parent span {
    margin-left: 10px; }
  .textarea-flexible-container td button {
    line-height: normal; }

.textarea-flexible {
  min-height: 24px;
  overflow: hidden;
  resize: none;
  word-break: break-word;
  height: 24px;
  line-height: 14px;
  white-space: break-spaces; }

.overlay-dialogue-body {
  margin: 0 0 25px;
  white-space: nowrap; }
  .overlay-dialogue-body .debug-output {
    margin: 10px 0 0; }

.overlay-dialogue-footer {
  text-align: right; }
  .overlay-dialogue-footer button {
    margin: 0 0 0 10px; }

.overlay-bg {
  background-color: #000000;
  width: 100%;
  height: 100%;
  position: fixed;
  z-index: 1000;
  top: 0;
  left: 0;
  opacity: .35; }

.calendar {
  width: 200px; }
  .calendar .calendar-year.highlighted, .calendar .highlighted.calendar-month,
  .calendar .calendar-month.highlighted {
    background-color: #454545; }
  .calendar .calendar-year:focus, .calendar .calendar-month:focus,
  .calendar .calendar-month:focus {
    outline: none; }
  .calendar table {
    width: 100%;
    margin: 5px 0; }
    .calendar table thead {
      text-transform: uppercase;
      color: #737373; }
    .calendar table tbody {
      text-align: center; }
      .calendar table tbody td {
        transition: background-color 0.2s ease-out; }
        .calendar table tbody td.selected {
          background-color: #4f4f4f;
          color: #ffffff; }
          .calendar table tbody td.selected:hover {
            background-color: #454545; }
        .calendar table tbody td:hover {
          background-color: #454545;
          cursor: pointer; }
        .calendar table tbody td.highlighted {
          color: #f2f2f2;
          background-color: #5e737e;
          border-color: #5e737e;
          cursor: pointer; }
        .calendar table tbody td:focus {
          outline: none; }
        .calendar table tbody td span {
          z-index: -1;
          padding: 4px;
          display: block; }

.calendar-header {
  text-align: center; }
  .calendar-header .btn-grey {
    border: 0;
    background-color: transparent; }

.calendar-year, .calendar-month {
  height: 24px;
  line-height: 26px;
  display: table;
  width: 100%; }
  .calendar-year button:first-child, .calendar-month button:first-child {
    float: left; }
  .calendar-year button:last-child, .calendar-month button:last-child {
    float: right; }
  .calendar-year button:hover, .calendar-month button:hover {
    background-color: #454545;
    cursor: pointer; }

.calendar-time {
  text-align: center; }
  .calendar-time input[type="text"] {
    width: 32px; }

.calendar-footer {
  margin: 26px 0 0;
  text-align: right; }
  .calendar-footer .btn-grey {
    float: left;
    margin-top: 0; }

.notif.overlay-dialogue {
  width: 250px; }
  .notif.overlay-dialogue .overlay-dialogue-header ul {
    margin: 0 auto 0 0; }
    .notif.overlay-dialogue .overlay-dialogue-header ul li {
      display: inline-block; }
      .notif.overlay-dialogue .overlay-dialogue-header ul li:not(:first-child) {
        margin-left: 5px; }
      .notif.overlay-dialogue .overlay-dialogue-header ul li .btn-icon {
        margin-left: 0;
        margin-right: 0; }

.notif-body {
  max-height: 600px;
  padding: 0 5px 0 0;
  overflow-y: auto; }
  .notif-body h4 {
    display: block;
    max-width: 250px;
    padding-bottom: .083em; }
  .notif-body p {
    margin: .25em 0 0; }
  .notif-body li {
    padding: 0 0 10px 19px; }
    .notif-body li:last-child {
      padding-bottom: 0; }

.notif-indic {
  width: 14px;
  height: 14px;
  float: left;
  margin: 2px 0 0 -19px;
  border-radius: 2px; }
  .notif-indic .notif-indic-snooze {
    width: 14px;
    height: 14px;
    float: left;
    margin-top: 20px; }
  .notif-indic .zi-bell-off {
    color: #69808d; }

.dashed-border td {
  border: 1px dashed #383838; }

.top {
  vertical-align: top !important; }

.right {
  text-align: right !important; }

.bottom {
  vertical-align: bottom; }

.left {
  text-align: left; }

.center {
  text-align: center; }

.middle {
  vertical-align: middle; }

.no-padding {
  padding: 0 !important; }

.graph-selection {
  position: absolute;
  z-index: 98;
  overflow: hidden;
  background-color: rgba(167, 136, 32, 0.35);
  border: 1px solid rgba(233, 144, 3, 0.6);
  border-top: 0;
  border-bottom: 0; }

.svg-graph-selection {
  fill: rgba(167, 136, 32, 0.35);
  stroke: rgba(233, 144, 3, 0.6);
  stroke-width: 1px; }

.svg-graph-selection-text {
  fill: #f2f2f2; }

.svg-helper {
  stroke-opacity: .35;
  stroke: #e45959;
  stroke-width: 2px; }

.svg-graph-hintbox {
  font-size: 12px;
  line-height: 18px;
  white-space: nowrap;
  min-width: 145px; }
  .svg-graph-hintbox .table-paging {
    min-height: 18px;
    padding: 0 0 2px;
    border: 0;
    top: 2px; }
    .svg-graph-hintbox .table-paging .paging-btn-container {
      min-height: inherit; }
  .svg-graph-hintbox .list-table tbody tr:last-child td {
    border-bottom-style: none;
    box-shadow: none; }
  .svg-graph-hintbox li {
    padding-left: 23px; }
    .svg-graph-hintbox li .svg-graph-hintbox-item-color,
    .svg-graph-hintbox li .svg-graph-hintbox-trigger-color {
      margin: 3px 10px 3px -20px;
      width: 10px;
      height: 10px;
      float: left;
      display: block; }
    .svg-graph-hintbox li .svg-graph-hintbox-trigger-color {
      border-radius: 50%; }
  .svg-graph-hintbox .header {
    margin: 0 0 10px 3px; }

.ui-selectable-helper {
  position: absolute;
  z-index: 100;
  background-color: rgba(167, 136, 32, 0.35);
  border: 1px solid rgba(233, 144, 3, 0.6); }

#map-area .map-element-area-bg {
  background-color: rgba(69, 69, 69, 0.35); }

.map-element-selected {
  border: 3px dashed #e99003;
  margin: -3px; }

.debug-output {
  display: none;
  max-height: 600px;
  overflow-y: auto;
  padding: 10px;
  margin: 10px;
  background-color: #2f280a;
  border: 1px solid #303030; }

.overlay-descr {
  max-height: 150px;
  overflow-y: auto;
  padding: 0;
  margin: 5px 0;
  background-color: #2b2b2b; }

.overlay-descr-url {
  padding: 3px 0 7px; }

.overlay-descr {
  background: linear-gradient(#2b2b2b 30%, rgba(43, 43, 43, 0)), linear-gradient(rgba(43, 43, 43, 0), #2b2b2b 70%) 0 100%, radial-gradient(50% 0, farthest-side, rgba(0, 0, 0, 0.4), transparent), radial-gradient(50% 100%, farthest-side, rgba(0, 0, 0, 0.4), transparent) 0 100%;
  background: linear-gradient(#2b2b2b 30%, rgba(43, 43, 43, 0)), linear-gradient(rgba(43, 43, 43, 0), #2b2b2b 70%) 0 100%, radial-gradient(farthest-side at 50% 0, rgba(0, 0, 0, 0.4), transparent), radial-gradient(farthest-side at 50% 100%, rgba(0, 0, 0, 0.4), transparent) 0 100%;
  background-repeat: no-repeat;
  background-color: #2b2b2b;
  background-size: 100% 40px, 100% 40px, 100% 4px, 100% 4px;
  background-attachment: local, local, scroll, scroll; }

.tags-wrapper {
  display: flex;
  flex-wrap: wrap;
  gap: 2px 3px; }
  .tags-wrapper .tag {
    margin-top: 1px;
    margin-bottom: 1px; }
  .tags-wrapper.form-field .tag {
    margin-top: 4px;
    margin-bottom: 4px; }

.tag {
  display: inline-block;
  color: #f2f2f2;
  background-color: #4f4f4f;
  padding: 2px 3px;
  line-height: 1em;
  vertical-align: middle;
  border-radius: 2px;
  box-sizing: border-box; }
  .tag a.link-action {
    color: inherit !important;
    border-bottom: none; }
    .tag a.link-action:hover {
      text-decoration: underline;
      border-bottom: none; }
  .tag:last-of-type {
    margin-right: 0; }
  .tag.green-bg {
    color: #0e4123; }
  .tag.yellow-bg {
    color: #734d00; }

.green-bg {
  background-color: #59db8f; }

.red-bg {
  background-color: #e45959; }

.yellow-bg {
  background-color: #ffc859; }

.na-bg {
  color: #2a353a;
  position: relative; }
  .na-bg a.link-action,
  .na-bg input[type="radio"]:enabled:checked + label {
    color: #2a353a; }

.log-na-bg {
  color: #2a353a;
  background-color: #97aab3; }

.normal-bg {
  color: #0e4123;
  position: relative;
  background-color: #59db8f; }
  .normal-bg a.link-action {
    color: #0e4123; }

.log-normal-bg {
  color: #0e4123;
  background-color: #59db8f; }

.info-bg {
  color: #00268e;
  position: relative; }
  .info-bg a.link-action,
  .info-bg input[type="radio"]:enabled:checked + label {
    color: #00268e; }

.log-info-bg {
  color: #00268e;
  background-color: #7499ff; }

.average-bg {
  color: #733100;
  position: relative; }
  .average-bg a.link-action,
  .average-bg input[type="radio"]:enabled:checked + label {
    color: #733100; }

.log-average-bg {
  color: #733100;
  background-color: #ffa059; }

.warning-bg {
  color: #734d00;
  position: relative; }
  .warning-bg a.link-action,
  .warning-bg input[type="radio"]:enabled:checked + label {
    color: #734d00; }

.log-warning-bg {
  color: #734d00;
  background-color: #ffc859; }

.high-bg {
  color: #52190b;
  position: relative; }
  .high-bg a.link-action,
  .high-bg input[type="radio"]:enabled:checked + label {
    color: #52190b; }

.log-high-bg {
  color: #52190b;
  background-color: #e97659; }

.disaster-bg {
  color: #4b0c0c;
  position: relative; }
  .disaster-bg a.link-action,
  .disaster-bg input[type="radio"]:enabled:checked + label {
    color: #4b0c0c; }

.log-disaster-bg {
  color: #4b0c0c;
  background-color: #e45959; }

.normal-bg input[type="radio"]:checked + label {
  background-color: #209450; }

.na-bg a,
.normal-bg a,
.info-bg a,
.average-bg a,
.warning-bg a,
.high-bg a,
.disaster-bg a {
  transition: none; }
.na-bg.blink-hidden,
.normal-bg.blink-hidden,
.info-bg.blink-hidden,
.average-bg.blink-hidden,
.warning-bg.blink-hidden,
.high-bg.blink-hidden,
.disaster-bg.blink-hidden {
  background-color: transparent; }
  .na-bg.blink-hidden a,
  .normal-bg.blink-hidden a,
  .info-bg.blink-hidden a,
  .average-bg.blink-hidden a,
  .warning-bg.blink-hidden a,
  .high-bg.blink-hidden a,
  .disaster-bg.blink-hidden a {
    color: #f2f2f2; }

.inactive-bg, td.inactive-bg {
  color: #4b0c0c;
  background-color: #e45959; }

.table-forms-second-column {
  display: inline-block;
  width: 50%;
  min-width: 200px;
  text-align: right; }
  .table-forms-second-column .second-column-label {
    padding: 0 10px; }

.problem-unack-fg {
  color: #ff3333; }

.problem-ack-fg {
  color: #ff3333; }

.ok-unack-fg {
  color: #00ff00; }

.ok-ack-fg {
  color: #00ff00; }

.navigation-tree .navigation-tree-node-is-selected > .navigation-tree-node-info, .navigation-tree .navigation-tree-node-is-selected > .navigation-tree-node-info:hover, div.dashboard-widget-itemhistory .list-table tr.selected td,
div.dashboard-widget-itemhistory .list-table td.selected, div.dashboard-widget-itemhistory .list-table tr.selected td:hover,
div.dashboard-widget-itemhistory .list-table td.selected:hover, div.dashboard-widget-navtree .navtree .tree .tree-item.selected > .tree-row, div.dashboard-widget-navtree .navtree .tree .tree-item.selected > .tree-row:hover, .drag-drop-area {
  background-color: #2f280a; }

@keyframes is-loading-kf {
  to {
    transform: rotate(360deg); } }
.is-loading {
  pointer-events: none; }
  .is-loading::before {
    background-color: rgba(43, 43, 43, 0.8);
    content: 'loading';
    text-indent: -9999px;
    overflow: hidden;
    position: absolute;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    z-index: 5; }
  .is-loading::after {
    background-color: rgba(43, 43, 43, 0.8);
    border: 2px solid #ccd5d9;
    border-bottom-color: #0275b8;
    border-radius: 50%;
    box-shadow: 0 0 0 7px rgba(43, 43, 43, 0.8);
    width: 20px;
    height: 20px;
    position: absolute;
    margin: auto;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    animation: is-loading-kf .6s infinite linear;
    z-index: 5;
    content: ''; }
  .is-loading.is-loading-fadein::before {
    opacity: 0;
    animation: fadein 2s ease-in .5s normal forwards; }
  .is-loading.is-loading-fadein::after {
    opacity: 0;
    animation: fadein 2s ease-in .5s normal forwards, is-loading-kf .6s infinite linear; }
  .is-loading.is-loading-fadein.delayed-15s::before, .is-loading.is-loading-fadein.delayed-15s::after {
    animation-delay: 15s; }

button.is-loading {
  position: relative; }
  button.is-loading::before {
    background: none; }
  button.is-loading::after {
    box-shadow: none;
    background: none;
    border: 1px solid #fdfdfd;
    border-bottom-color: #0275b8;
    width: 12px;
    height: 12px; }

.link-action.is-loading {
  position: relative;
  color: transparent !important;
  border-bottom-color: transparent; }
  .link-action.is-loading::before {
    background-color: transparent; }
  .link-action.is-loading::after {
    box-shadow: none;
    background: none;
    border: 1px solid #fdfdfd;
    border-bottom-color: #0275b8;
    width: 12px;
    height: 12px; }

span.is-loading {
  position: relative;
  display: inline-block;
  min-width: 40px;
  min-height: 40px; }

.overlay-dialogue.modal .overlay-dialogue-body.is-loading {
  overflow: hidden; }

.menu-popup-preloader {
  background: #2b2b2b;
  border: 1px solid #383838;
  box-shadow: 0 4px 20px 0 rgba(0, 0, 0, 0.5);
  height: 50px;
  width: 50px;
  position: fixed;
  right: auto;
  bottom: auto;
  z-index: 1010; }

.hintbox-preloader {
  height: 50px;
  width: 50px; }

.browser-logo-chrome, .browser-logo-ff, .browser-logo-ed, .browser-logo-opera, .browser-logo-safari {
  background: url("../img/browser-sprite.png?20220722") no-repeat;
  width: 66px;
  height: 66px;
  margin: 0 auto;
  margin-bottom: 5px; }

.browser-logo-chrome {
  background-position: 0 0; }

.browser-logo-ff {
  background-position: -66px 0; }

.browser-logo-ed {
  background-position: 0 -66px; }

.browser-logo-opera {
  background-position: -66px -66px; }

.browser-logo-safari {
  background-position: 0 -132px; }

.browser-warning-container {
  margin-top: 5%;
  margin-left: auto;
  margin-right: auto;
  width: 766px;
  text-align: center;
  padding: 28px 28px 10px; }
  .browser-warning-container h2 {
    text-align: left; }
  .browser-warning-container p {
    margin: .7em 0;
    text-align: left; }
  .browser-warning-container li {
    display: inline-block;
    margin: 25px 20px; }

.browser-warning-footer {
  border-top: 1px solid #383838;
  margin: 25px 0 0;
  padding: 10px 0 0;
  text-align: center; }

.dashboard-grid-widget-contents, output .msg-details ul, .multiselect-available, .z-select .focusable,
z-select .focusable, .z-select .list,
z-select .list, div.dashboard-widget-item .dashboard-grid-widget-body > div, textarea, select, .setup-right-body, .menu-popup, .overlay-dialogue.modal .overlay-dialogue-body, .overlay-dialogue .hintbox-wrap,
.overlay-dialogue .hintbox-wrap-horizontal, .overlay-dialogue .maps-container, .notif-body, .debug-output, .overlay-descr, .import-compare .toc,
.import-compare .diff {
  scrollbar-width: thin; }
  .dashboard-grid-widget-contents::-webkit-scrollbar, output .msg-details ul::-webkit-scrollbar, .multiselect-available::-webkit-scrollbar, .z-select .focusable::-webkit-scrollbar,
  z-select .focusable::-webkit-scrollbar, .z-select .list::-webkit-scrollbar,
  z-select .list::-webkit-scrollbar, div.dashboard-widget-item .dashboard-grid-widget-body > div::-webkit-scrollbar, textarea::-webkit-scrollbar, select::-webkit-scrollbar, .setup-right-body::-webkit-scrollbar, .menu-popup::-webkit-scrollbar, .overlay-dialogue.modal .overlay-dialogue-body::-webkit-scrollbar, .overlay-dialogue .hintbox-wrap::-webkit-scrollbar,
  .overlay-dialogue .hintbox-wrap-horizontal::-webkit-scrollbar, .overlay-dialogue .maps-container::-webkit-scrollbar, .notif-body::-webkit-scrollbar, .debug-output::-webkit-scrollbar, .overlay-descr::-webkit-scrollbar, .import-compare .toc::-webkit-scrollbar,
  .import-compare .diff::-webkit-scrollbar {
    width: 9px;
    height: 9px; }
  .dashboard-grid-widget-contents::-webkit-scrollbar-track, output .msg-details ul::-webkit-scrollbar-track, .multiselect-available::-webkit-scrollbar-track, .z-select .focusable::-webkit-scrollbar-track,
  z-select .focusable::-webkit-scrollbar-track, .z-select .list::-webkit-scrollbar-track,
  z-select .list::-webkit-scrollbar-track, div.dashboard-widget-item .dashboard-grid-widget-body > div::-webkit-scrollbar-track, textarea::-webkit-scrollbar-track, select::-webkit-scrollbar-track, .setup-right-body::-webkit-scrollbar-track, .menu-popup::-webkit-scrollbar-track, .overlay-dialogue.modal .overlay-dialogue-body::-webkit-scrollbar-track, .overlay-dialogue .hintbox-wrap::-webkit-scrollbar-track,
  .overlay-dialogue .hintbox-wrap-horizontal::-webkit-scrollbar-track, .overlay-dialogue .maps-container::-webkit-scrollbar-track, .notif-body::-webkit-scrollbar-track, .debug-output::-webkit-scrollbar-track, .overlay-descr::-webkit-scrollbar-track, .import-compare .toc::-webkit-scrollbar-track,
  .import-compare .diff::-webkit-scrollbar-track {
    background-color: #1f1f1f; }
  .dashboard-grid-widget-contents::-webkit-scrollbar-thumb, output .msg-details ul::-webkit-scrollbar-thumb, .multiselect-available::-webkit-scrollbar-thumb, .z-select .focusable::-webkit-scrollbar-thumb,
  z-select .focusable::-webkit-scrollbar-thumb, .z-select .list::-webkit-scrollbar-thumb,
  z-select .list::-webkit-scrollbar-thumb, div.dashboard-widget-item .dashboard-grid-widget-body > div::-webkit-scrollbar-thumb, textarea::-webkit-scrollbar-thumb, select::-webkit-scrollbar-thumb, .setup-right-body::-webkit-scrollbar-thumb, .menu-popup::-webkit-scrollbar-thumb, .overlay-dialogue.modal .overlay-dialogue-body::-webkit-scrollbar-thumb, .overlay-dialogue .hintbox-wrap::-webkit-scrollbar-thumb,
  .overlay-dialogue .hintbox-wrap-horizontal::-webkit-scrollbar-thumb, .overlay-dialogue .maps-container::-webkit-scrollbar-thumb, .notif-body::-webkit-scrollbar-thumb, .debug-output::-webkit-scrollbar-thumb, .overlay-descr::-webkit-scrollbar-thumb, .import-compare .toc::-webkit-scrollbar-thumb,
  .import-compare .diff::-webkit-scrollbar-thumb {
    background-color: #383838;
    border: 1px solid #2b2b2b; }

.cursor-dragging {
  cursor: grabbing; }

.cursor-move {
  cursor: move; }

tr.cursor-move td * {
  cursor: move; }

.cursor-pointer {
  cursor: pointer; }

.overflow-ellipsis {
  table-layout: fixed; }

.rel-container {
  position: relative;
  display: inline-block;
  min-width: 16px;
  white-space: nowrap; }

main .server-name {
  float: right;
  margin: 10px;
  color: #737373; }

.uppercase {
  text-transform: uppercase; }

.flickerfreescreen {
  position: relative;
  overflow: hidden; }
  .flickerfreescreen.is-loading {
    min-height: 64px; }

.graph-wrapper {
  display: inline; }

.clock {
  display: flex;
  justify-content: center;
  padding: 0 10px;
  height: 100%; }

.clock-svg {
  flex-grow: 1; }

.time-zone {
  margin: 0 0 .5em;
  white-space: nowrap; }

.local-clock {
  margin: .5em 0 0;
  white-space: nowrap; }

.clock-face {
  fill: #383838; }

.clock-hand {
  fill: #f2f2f2; }

.clock-hand-sec {
  fill: #69808d; }

.clock-lines {
  fill: #4f4f4f; }

svg {
  overflow: hidden; }

.sysmap {
  height: 100%;
  width: auto;
  padding: 0 10px;
  text-align: center; }
  .sysmap .sysmap-caption {
    font-size: 0; }

.sysmap-scroll-container {
  overflow-x: auto;
  overflow-y: hidden;
  position: relative;
  width: calc(100% - 20px);
  border: 10px solid #2b2b2b;
  background: #2b2b2b;
  display: block;
  margin-top: 4px; }
  .sysmap-scroll-container .map-container {
    display: table; }
  .sysmap-scroll-container .flickerfreescreen {
    display: inline-block; }
  .sysmap-scroll-container .table-forms-container, .sysmap-scroll-container .browser-warning-container {
    display: table;
    margin: 0;
    padding: 0;
    border: 0; }

.lld-overrides-operations-table,
.confirmation-msg {
  white-space: normal;
  word-break: break-word; }

.list-accordion-foot > div {
  display: table-cell;
  padding-top: 10px; }

.color-preview-box {
  height: 24px;
  width: 24px;
  float: left;
  margin-right: 10px;
  cursor: pointer;
  border: 0;
  border-radius: 0; }

.list-vertical-accordion .list-accordion-item-head,
.list-vertical-accordion .list-accordion-item-body {
  padding-bottom: 2px; }
.list-vertical-accordion .list-accordion-item-opened .list-accordion-item-body {
  display: block; }
.list-vertical-accordion .list-accordion-item-closed .list-accordion-item-body {
  display: none; }
.list-vertical-accordion .list-accordion-item-closed .list-accordion-item-toggle {
  margin: 0;
  transform: rotate(180deg); }

.display-none, .table-forms > li.display-none {
  display: none; }

.visibility-hidden {
  visibility: hidden; }

.visibility-box-caption {
  line-height: 24px; }

.drag-icon {
  width: 12px;
  height: 1.4em; }
  .drag-icon::before {
    font-size: 22px; }

.sortable.sortable-disabled .sortable-item .sortable:not(.sortable-disabled) .sortable-item .drag-icon, .drag-icon {
  cursor: grab;
  color: #69808d; }

.sortable.sortable-disabled .sortable-item .drag-icon {
  cursor: unset;
  color: #3d4b53; }

.preprocessing-list {
  display: block;
  max-width: 930px;
  min-width: 800px;
  position: relative; }
  .preprocessing-list > li {
    display: block;
    position: relative; }
  .preprocessing-list .list-numbered-item::before {
    content: counter(line) ":";
    flex: 0 0 15px;
    max-width: 15px;
    line-height: 24px;
    padding-right: 5px; }
  .preprocessing-list input[type=text],
  .preprocessing-list select {
    width: 100%;
    min-width: 0; }
  .preprocessing-list .btn-link:not(:first-child) {
    margin-left: 10px; }
  .preprocessing-list .preprocessing-list-head,
  .preprocessing-list .preprocessing-list-foot,
  .preprocessing-list .preprocessing-step,
  .preprocessing-list .on-fail-options,
  .preprocessing-list .step-name,
  .preprocessing-list .step-parameters,
  .preprocessing-list .step-on-fail,
  .preprocessing-list .step-action {
    display: flex;
    align-items: center;
    box-sizing: border-box; }
  .preprocessing-list .step-name,
  .preprocessing-list .step-parameters,
  .preprocessing-list .step-on-fail,
  .preprocessing-list .step-action,
  .preprocessing-list .on-fail-options > label,
  .preprocessing-list .on-fail-options > .radio-segmented {
    padding: 5px 5px 5px 0; }
  .preprocessing-list .step-name {
    flex: 0 0 285px;
    max-width: 285px; }
  .preprocessing-list .preprocessing-step {
    align-items: start; }
    .preprocessing-list .preprocessing-step .drag-icon {
      top: 5px;
      left: -8px;
      width: 10px;
      height: 24px; }
    .preprocessing-list .preprocessing-step .step-on-fail,
    .preprocessing-list .preprocessing-step .step-action {
      height: 34px; }
  .preprocessing-list .step-parameters {
    flex: 1; }
    .preprocessing-list .step-parameters z-select {
      flex: 0 0 auto; }
    .preprocessing-list .step-parameters > input[type=text]:not(:last-child),
    .preprocessing-list .step-parameters z-select:not(:last-child) {
      margin-right: 5px; }
  .preprocessing-list .step-on-fail {
    flex: 0 0 100px;
    max-width: 100px;
    justify-content: center;
    text-align: center; }
  .preprocessing-list .step-action {
    flex: 0 0 120px;
    max-width: 120px;
    padding-right: 0; }
  .preprocessing-list .on-fail-options {
    padding-right: 225px;
    margin-bottom: 5px; }
    .preprocessing-list .on-fail-options > label {
      padding-left: 30px; }
    .preprocessing-list .on-fail-options input[type=text] {
      flex: 1; }
      .preprocessing-list .on-fail-options input[type=text]:not(:first-child) {
        margin-left: 5px; }
  .preprocessing-list .preprocessing-list-head {
    color: #737373;
    line-height: 14px; }
    .preprocessing-list .preprocessing-list-head .step-name {
      flex: 0 0 295px;
      max-width: 295px;
      padding-left: 30px; }
    .preprocessing-list .preprocessing-list-head .step-on-fail {
      display: inline-block; }
  .preprocessing-list .preprocessing-list-foot {
    justify-content: space-between; }
    .preprocessing-list .preprocessing-list-foot .step-action {
      height: 24px; }
      .preprocessing-list .preprocessing-list-foot .step-action + .step-action:last-child {
        margin-left: auto;
        justify-self: flex-end; }

.problem-icon-link,
.problem-icon-list {
  display: inline-block;
  height: 16px;
  white-space: nowrap; }
  .problem-icon-link .problem-icon-list-item,
  .problem-icon-list .problem-icon-list-item {
    font-size: 12px;
    color: #f2f2f2;
    border-color: transparent; }

.problem-icon-list .problem-icon-list-item:not(:last-of-type) {
  margin-right: 5px; }

.problem-icon-link {
  overflow: hidden;
  border-radius: 2px; }
  .problem-icon-link .problem-icon-list-item {
    border-radius: 0; }
  .problem-icon-link:link, .problem-icon-link:hover, .problem-icon-link:focus {
    border-bottom: none; }
  .problem-icon-link:focus {
    box-shadow: 0 0 0 2px rgba(30, 135, 227, 0.35); }

.ui-sortable tbody {
  position: relative; }
.ui-sortable .td-drag-icon {
  width: 12px; }
.ui-sortable.ui-sortable-disabled .drag-icon {
  cursor: default;
  color: #3d4b53; }

#expressions_list .ui-sortable-helper {
  display: table; }

.graph-legend {
  text-align: left;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden; }

.preprocessing-test-results td {
  vertical-align: top !important; }
.preprocessing-test-results .rel-container {
  top: 4px;
  margin-left: 3px; }

#preprocessing-test-form .table-forms-separator {
  margin-top: -2px; }
#preprocessing-test-form .runtime-error-fields {
  display: flex;
  justify-content: space-between; }
  #preprocessing-test-form .runtime-error-fields label {
    width: auto;
    padding-inline-end: 8px; }
  #preprocessing-test-form .runtime-error-fields > div {
    display: inline-flex; }
  #preprocessing-test-form .runtime-error-fields .multilineinput-control {
    width: 250px; }
#preprocessing-test-form .item-test-result {
  display: inline-flex; }

.totals-list:not(.list-table) {
  display: flex;
  height: 100%; }
.totals-list > div {
  flex: 1;
  align-items: center;
  display: flex;
  line-height: 20px;
  padding: 0 10px;
  position: relative; }
.totals-list .count {
  font-size: 16px; }
.totals-list.totals-list-horizontal > div {
  flex-direction: column;
  justify-content: center;
  min-width: 55px;
  text-align: center; }
.totals-list.totals-list-horizontal .count,
.totals-list.totals-list-horizontal .name {
  width: 100%; }
.totals-list.totals-list-vertical {
  flex-direction: column;
  min-width: min-content; }
  .totals-list.totals-list-vertical > div {
    column-gap: 5px;
    min-height: 20px;
    padding-top: 2px;
    padding-bottom: 2px;
    white-space: nowrap; }

.macro-input-group {
  position: relative;
  display: flex;
  align-items: stretch;
  width: 100%; }
  .macro-input-group * {
    box-sizing: border-box; }
  .macro-input-group input,
  .macro-input-group textarea {
    flex: 1;
    border-right: 0; }
    .macro-input-group input:focus + .btn-undo,
    .macro-input-group textarea:focus + .btn-undo {
      border-color: #768d99; }
  .macro-input-group .btn-change,
  .macro-input-group .btn-undo {
    border-radius: 0; }
  .macro-input-group .btn-change[disabled] {
    display: none; }
  .macro-input-group .btn-undo {
    display: none;
    color: #69808d;
    background-color: #383838;
    border-width: 1px 0;
    border-color: #4f4f4f; }
    .macro-input-group .btn-undo::before {
      font-size: 16px; }
    .macro-input-group .btn-undo:focus, .macro-input-group .btn-undo:hover {
      color: #f2f2f2;
      background-color: #5e737e;
      border-color: #768d99; }
    .macro-input-group .btn-undo.is-focused {
      border-color: #768d99; }

.input-secret {
  position: relative;
  display: flex;
  flex: 1; }
  .input-secret input {
    flex: 1; }
  .input-secret button {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    opacity: 0; }
    .input-secret button:enabled:hover, .input-secret button:enabled:focus, .input-secret button:enabled:active {
      opacity: 1;
      animation: fadein 0.2s; }

.btn-dropdown-container {
  display: flex; }
  .btn-dropdown-container .btn-dropdown-toggle {
    width: auto;
    height: auto;
    padding-right: 3px;
    align-items: start;
    line-height: 22px;
    border-radius: 0 2px 2px 0; }
    .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"], .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"] {
      padding-left: 3px;
      color: #69808d; }
      .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"]::before, .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"]::before {
        margin-top: 2px;
        margin-right: 2px; }
      .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"]:hover, .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"]:active, .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"]:focus, .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"]:hover, .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"]:active, .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"]:focus {
        color: #f2f2f2; }
      .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"][disabled], .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"][disabled] {
        color: #525252; }
      .btn-dropdown-container .btn-dropdown-toggle[class^="zi-"][aria-expanded="true"], .btn-dropdown-container .btn-dropdown-toggle[class*=" zi-"][aria-expanded="true"] {
        color: #f2f2f2;
        background-color: #5e737e;
        border-color: #5e737e; }
    .btn-dropdown-container .btn-dropdown-toggle::after {
      content: "";
      margin-top: 7px;
      font-size: 10px; }
    .btn-dropdown-container .btn-dropdown-toggle:not(:empty)::before {
      padding-left: 4px; }
    .btn-dropdown-container .btn-dropdown-toggle:not(:empty)::after {
      margin-left: 4px; }

.btn-split {
  position: relative;
  display: inline-flex;
  margin-right: 10px; }
  .btn-split li {
    display: inline-flex; }
    .btn-split li:first-child button {
      border-top-right-radius: 0;
      border-bottom-right-radius: 0; }
    .btn-split li:last-child {
      margin-left: -1px; }
      .btn-split li:last-child button {
        border-top-left-radius: 0;
        border-bottom-left-radius: 0; }
    .btn-split li:only-child button {
      border-radius: 2px; }
  .btn-split button:focus {
    z-index: 1; }
  .btn-split button[aria-expanded="true"] {
    color: #f2f2f2;
    background-color: #5e737e;
    border-color: #5e737e; }

.checkbox-block {
  margin: 10px 0; }

#tbl_macros.massupdate-remove th,
#tbl_macros.massupdate-remove td {
  display: none; }
#tbl_macros.massupdate-remove th:nth-child(1),
#tbl_macros.massupdate-remove td:nth-child(1),
#tbl_macros.massupdate-remove td.nowrap {
  display: table-cell; }

.table-forms-td-right .interfaces {
  margin-top: 5px; }

.interfaces {
  display: grid;
  grid-template-columns: 26px repeat(7, max-content);
  margin-bottom: 5px;
  line-height: 24px; }
  .interfaces .interface-container {
    display: contents; }
    .interfaces .interface-container.interface-container-header .interface-cell {
      padding: 0 5px 0 0;
      color: #737373; }
    .interfaces .interface-container.interface-container-header .interface-cell-action {
      grid-column: span 2; }
    .interfaces .interface-container:not(.interface-container-header):not(:empty) ~ .interface-container:not(:empty) :first-child .interface-cell:not(.interface-cell-details) {
      padding-top: 10px;
      margin-top: 5px;
      border-top: 1px solid #4f4f4f; }
  .interfaces .interface-row {
    display: contents; }
    .interfaces .interface-row .interface-btn-toggle {
      display: none;
      vertical-align: top; }
      .interfaces .interface-row .interface-btn-toggle[aria-expanded="true"] {
        transform: rotate(-180deg); }
    .interfaces .interface-row[data-type="2"] .list-accordion-item-body {
      display: none;
      padding-top: 0; }
    .interfaces .interface-row[data-type="2"] .interface-btn-toggle {
      display: inline-block; }
    .interfaces .interface-row[data-type="2"].list-accordion-item-opened .list-accordion-item-body {
      display: block; }
  .interfaces .interface-cell {
    padding: 5px 5px 5px 0; }
    .interfaces .interface-cell.interface-cell-details {
      display: none;
      grid-column: 1 / -1; }
  .interfaces .no-interface {
    grid-column: 1 / -1;
    color: #737373; }

.debug-modal.overlay-dialogue.modal .overlay-dialogue-body {
  overflow-x: auto; }
  .debug-modal.overlay-dialogue.modal .overlay-dialogue-body::-webkit-scrollbar {
    height: 9px; }
.debug-modal .logitems pre {
  white-space: nowrap; }
.debug-modal .logtotalms {
  color: #737373;
  float: left;
  line-height: 24px; }

#itemsTable .table-col-handle {
  width: 12px; }
#itemsTable .table-col-no {
  width: 20px; }
#itemsTable .table-col-name-normal {
  width: 285px; }
#itemsTable .table-col-name {
  width: 365px; }
#itemsTable .table-col-type {
  width: 85px; }
#itemsTable .table-col-function {
  width: 85px; }
#itemsTable .table-col-draw-style {
  width: 85px; }
#itemsTable .table-col-y-axis-side {
  width: 85px; }
#itemsTable .table-col-color {
  width: 55px; }
#itemsTable .table-col-action {
  width: 55px; }

.z-select.z-select-host-interface .description:not(:empty),
z-select.z-select-host-interface .description:not(:empty) {
  display: block;
  margin-top: 5px;
  color: #9d9d9d; }
.z-select.z-select-host-interface[disabled] .description:not(:empty),
.z-select.z-select-host-interface li[disabled] .description:not(:empty),
z-select.z-select-host-interface[disabled] .description:not(:empty),
z-select.z-select-host-interface li[disabled] .description:not(:empty) {
  color: #525252; }

.multiselect-description-container {
  display: flex;
  align-items: center; }
  .multiselect-description-container > .multiselect-control {
    margin-right: 5px; }

.rules-status-container .status-green,
.rules-status-container .status-grey {
  margin: 3px 3px 3px 0; }

.input-section-header {
  padding-top: 10px; }

.modal-popup-preprocessing .preprocessing-list-head .step-name {
  flex: 0 0 190px; }
.modal-popup-preprocessing .preprocessing-list-item .step-name {
  flex: 0 0 180px; }
.modal-popup-preprocessing .preprocessing-list {
  min-width: unset;
  max-width: unset; }

.modal-popup-medium .table-forms-container, .modal-popup-medium .browser-warning-container,
.modal-popup-large .table-forms-container,
.modal-popup-large .browser-warning-container,
.modal-popup-static .table-forms-container,
.modal-popup-static .browser-warning-container,
.modal-popup-preprocessing .table-forms-container,
.modal-popup-preprocessing .browser-warning-container {
  border: none; }
.modal-popup-medium .ui-tabs-nav,
.modal-popup-large .ui-tabs-nav,
.modal-popup-static .ui-tabs-nav,
.modal-popup-preprocessing .ui-tabs-nav {
  position: sticky;
  top: 0;
  background: #2b2b2b;
  z-index: 3; }
.modal-popup-medium .textarea-flexible,
.modal-popup-large .textarea-flexible,
.modal-popup-static .textarea-flexible,
.modal-popup-preprocessing .textarea-flexible {
  max-height: 200px;
  overflow-y: auto; }

.global-macro-table .table-col-value,
.global-macro-table .table-col-template-value,
.host-macros-table .table-col-value,
.host-macros-table .table-col-template-value {
  width: 300px; }

.inherited-macros-table .table-col-value,
.inherited-macros-table .table-col-parent-value,
.inherited-macros-table .table-col-template-value,
.inherited-macros-table .table-col-global-value {
  width: 300px; }

.active-readonly[readonly] {
  color: #f2f2f2;
  background-color: #383838 !important;
  border-color: #4f4f4f; }

.valuemap-list-table tbody td {
  border-bottom: 1px solid #383838; }
.valuemap-list-table tbody tr:first-child td {
  border-top: 1px solid #383838; }

.mappings-table {
  display: grid;
  grid-template-columns: auto auto minmax(auto, 100%); }
  .mappings-table > div {
    text-align: left; }
  .mappings-table > div:nth-child(3n + 2) {
    text-align: center;
    padding: 0 10px; }

.valuemap-checkbox {
  margin-top: 10px; }

.overlay-dialogue .valuemap-list-table td {
  vertical-align: top; }

.form-grid .subscriptions-table th {
  color: #737373; }
.form-grid .subscriptions-table .icon {
  margin-right: 3px;
  vertical-align: text-bottom; }
.form-grid .tags-table {
  width: auto; }
  .form-grid .tags-table tr:not(.form_row) td {
    padding-top: 0; }

.import-compare {
  display: flex;
  max-height: calc(100vh - 220px); }
  .import-compare .toc {
    flex: 20%;
    overflow-y: auto;
    overflow-x: hidden; }
  .import-compare .diff {
    flex: 80%;
    margin-left: 5px;
    overflow: auto;
    border: 1px dashed #4f4f4f; }

.list-dashed li {
  margin-left: 1em; }
  .list-dashed li::before {
    content: '\2013';
    float: left;
    margin-left: -1em; }

.overlay-dialogue #host-form #macrosFormList .form-field,
.overlay-dialogue #templates-form #macrosFormList .form-field {
  grid-column-start: 1; }
.overlay-dialogue #host-form #macrosFormList .table-forms-td-left,
.overlay-dialogue #templates-form #macrosFormList .table-forms-td-left {
  display: none; }
.overlay-dialogue #host-form #macrosFormList .table-forms-td-right,
.overlay-dialogue #templates-form #macrosFormList .table-forms-td-right {
  width: 100%; }

.linked-templates > .table-forms {
  margin-bottom: 5px;
  line-height: 24px; }
  .linked-templates > .table-forms td, .linked-templates > .table-forms th {
    padding: 0 5px 0 0; }
    .linked-templates > .table-forms td:last-child, .linked-templates > .table-forms th:last-child {
      width: 0;
      padding-right: 0; }

.audit-details-popup-wrapper {
  margin: 5px 10px; }
  .audit-details-popup-wrapper .audit-details-popup-textarea {
    width: 100%;
    height: 135px; }

.audit-details-wrapper {
  display: flex;
  justify-content: space-between; }
  .audit-details-wrapper .audit-show-details-btn-wrapper {
    margin: 0 5px 0 5px; }

.color-accent, .color-accent.btn-icon:not([disabled]), .color-accent.btn-link:not([disabled]), .color-accent.link-action:not(:hover):not(:focus), .color-accent.link-alt:not(:hover):not(:focus) {
  color: #f24f1d; }
  .color-accent.btn-link:not([disabled]), .btn-link.color-accent.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.color-accent.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #f24f1d; }

.color-icon, .color-icon.btn-icon:not([disabled]), .color-icon.btn-link:not([disabled]), .color-icon.link-action:not(:hover):not(:focus), .color-icon.link-alt:not(:hover):not(:focus) {
  color: #69808d; }
  .color-icon.btn-link:not([disabled]), .btn-link.color-icon.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.color-icon.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #69808d; }

.color-negative, .color-negative.btn-icon:not([disabled]), .color-negative.btn-link:not([disabled]), .color-negative.link-action:not(:hover):not(:focus), .color-negative.link-alt:not(:hover):not(:focus) {
  color: #e45959; }
  .color-negative.btn-link:not([disabled]), .btn-link.color-negative.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.color-negative.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #e45959; }

.color-warning, .color-warning.btn-icon:not([disabled]), .color-warning.btn-link:not([disabled]), .color-warning.link-action:not(:hover):not(:focus), .color-warning.link-alt:not(:hover):not(:focus) {
  color: #e99003; }
  .color-warning.btn-link:not([disabled]), .btn-link.color-warning.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.color-warning.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #e99003; }

.color-positive, .color-positive.btn-icon:not([disabled]), .color-positive.btn-link:not([disabled]), .color-positive.link-action:not(:hover):not(:focus), .color-positive.link-alt:not(:hover):not(:focus) {
  color: #59db8f; }
  .color-positive.btn-link:not([disabled]), .btn-link.color-positive.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.color-positive.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #59db8f; }

.blue, .blue.btn-icon:not([disabled]), .blue.btn-link:not([disabled]), .blue.link-action:not(:hover):not(:focus), .blue.link-alt:not(:hover):not(:focus) {
  color: #429ae3; }
  .blue.btn-link:not([disabled]), .btn-link.blue.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.blue.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #429ae3; }

.green, .green.btn-icon:not([disabled]), .green.btn-link:not([disabled]), .green.link-action:not(:hover):not(:focus), .green.link-alt:not(:hover):not(:focus) {
  color: #59db8f; }
  .green.btn-link:not([disabled]), .btn-link.green.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.green.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #59db8f; }

.grey, .grey.btn-icon:not([disabled]), .grey.btn-link:not([disabled]), .grey.link-action:not(:hover):not(:focus), .grey.link-alt:not(:hover):not(:focus) {
  color: #737373; }
  .grey.btn-link:not([disabled]), .btn-link.grey.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.grey.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #737373; }

.orange, .orange.btn-icon:not([disabled]), .orange.btn-link:not([disabled]), .orange.link-action:not(:hover):not(:focus), .orange.link-alt:not(:hover):not(:focus) {
  color: #f24f1d; }
  .orange.btn-link:not([disabled]), .btn-link.orange.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.orange.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #f24f1d; }

.red, .red.btn-icon:not([disabled]), .red.btn-link:not([disabled]), .red.link-action:not(:hover):not(:focus), .red.link-alt:not(:hover):not(:focus) {
  color: #e45959; }
  .red.btn-link:not([disabled]), .btn-link.red.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.red.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #e45959; }

.teal, .teal.btn-icon:not([disabled]), .teal.btn-link:not([disabled]), .teal.link-action:not(:hover):not(:focus), .teal.link-alt:not(:hover):not(:focus) {
  color: #0f998b; }
  .teal.btn-link:not([disabled]), .btn-link.teal.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.teal.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #0f998b; }

.yellow, .yellow.btn-icon:not([disabled]), .yellow.btn-link:not([disabled]), .yellow.link-action:not(:hover):not(:focus), .yellow.link-alt:not(:hover):not(:focus) {
  color: #e99003; }
  .yellow.btn-link:not([disabled]), .btn-link.yellow.link-action:not([disabled]):not(:hover):not(:focus), .btn-link.yellow.link-alt:not([disabled]):not(:hover):not(:focus) {
    border-color: #e99003; }

.sidebar.is-compact:not(.is-opened) .form-search .search-icon {
  opacity: .75; }

.list-table thead th {
  border-color: #404040; }
.list-table.compact-view tr:hover td.na-bg, .list-table.compact-view tr:hover td.info-bg, .list-table.compact-view tr:hover td.warning-bg, .list-table.compact-view tr:hover td.average-bg, .list-table.compact-view tr:hover td.high-bg, .list-table.compact-view tr:hover td.disaster-bg, .list-table.has-highlighted-rows tr:hover td.na-bg, .list-table.has-highlighted-rows tr:hover td.info-bg, .list-table.has-highlighted-rows tr:hover td.warning-bg, .list-table.has-highlighted-rows tr:hover td.average-bg, .list-table.has-highlighted-rows tr:hover td.high-bg, .list-table.has-highlighted-rows tr:hover td.disaster-bg {
  color: #000000; }
.list-table.compact-view tr[class*='flh-'] td, .list-table.has-highlighted-rows tr[class*='flh-'] td {
  box-shadow: inset 0 -1px 0 0 rgba(0, 0, 0, 0.1); }
.list-table.compact-view td.na-bg, .list-table.compact-view td.info-bg, .list-table.compact-view td.warning-bg, .list-table.compact-view td.average-bg, .list-table.compact-view td.high-bg, .list-table.compact-view td.disaster-bg, .list-table.has-highlighted-rows td.na-bg, .list-table.has-highlighted-rows td.info-bg, .list-table.has-highlighted-rows td.warning-bg, .list-table.has-highlighted-rows td.average-bg, .list-table.has-highlighted-rows td.high-bg, .list-table.has-highlighted-rows td.disaster-bg {
  color: #000000;
  box-shadow: inset 0 -1px 0 0 rgba(0, 0, 0, 0.1); }
.list-table.has-highlighted-rows .problem-unack-fg,
.list-table.has-highlighted-rows .problem-ack-fg {
  color: #cc0000; }
.list-table.has-highlighted-rows .ok-unack-fg,
.list-table.has-highlighted-rows .ok-ack-fg {
  color: #009900; }
.list-table.has-highlighted-rows tr.row-selected .link-alt:not(:hover):not(:focus), .list-table.has-highlighted-rows tr:hover .link-alt:not(:hover):not(:focus), .list-table.has-highlighted-rows tr:not([class*='flh-']) .link-alt:not(:hover):not(:focus) {
  color: #f2f2f2;
  border-bottom-color: #f2f2f2; }
.list-table.sticky-footer tfoot tr:first-of-type td {
  border-color: #404040; }

.dashboard-navigation-tabs .sortable li div:hover, .host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li div:hover,
.dashboard-navigation-tabs .sortable li.sortable-item-dragging div,
.host-dashboard-header-navigation .host-dashboard-navigation .host-dashboard-navigation-tabs .sortable li.sortable-item-dragging div {
  border: 1px solid transparent; }

.dashboard-grid-widget.dashboard-grid-widget-hidden-header.dashboard-grid-widget-focus .dashboard-grid-widget-header {
  box-shadow: 0 -6px 8px -2px rgba(0, 0, 0, 0.5); }

.dashboard-grid-iterator.dashboard-grid-iterator-focus .dashboard-grid-iterator-header {
  box-shadow: 0 -6px 8px -2px rgba(0, 0, 0, 0.5); }

.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging.dashboard-grid-widget-hidden-header .dashboard-grid-widget-header {
  box-shadow: 0 -6px 8px -2px rgba(0, 0, 0, 0.5); }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing::before, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging::before {
  box-shadow: 0 4px 20px 2px rgba(0, 0, 0, 0.5); }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing .dashboard-grid-widget-header, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging .dashboard-grid-widget-header {
  border-color: #303030; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-resizable-resizing .dashboard-grid-widget-contents, .dashboard.dashboard-is-edit-mode .dashboard-grid-widget.ui-draggable-dragging .dashboard-grid-widget-contents {
  border-color: #303030; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-header {
  box-shadow: 0 -6px 8px -2px rgba(0, 0, 0, 0.5); }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing::before, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging::before {
  box-shadow: 0 4px 20px 2px rgba(0, 0, 0, 0.5); }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-header, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-header {
  border-color: #303030; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-header, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-header {
  border-color: #303030; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-contents, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-contents .dashboard-grid-widget .dashboard-grid-widget-contents {
  border-color: #303030; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-contents .dashboard-grid-iterator-placeholder > div, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-contents .dashboard-grid-iterator-placeholder > div {
  border-color: #303030; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-resizable-resizing .dashboard-grid-iterator-mask, .dashboard.dashboard-is-edit-mode .dashboard-grid-iterator.ui-draggable-dragging .dashboard-grid-iterator-mask {
  border: 0; }
.dashboard.dashboard-is-edit-mode .dashboard-grid-widget-placeholder > div {
  background-color: #232323; }

.time-quick li a:focus {
  color: #ffffff; }

.totals-list {
  color: #191919; }

.breadcrumbs > ::after {
  content: " /"; }
.breadcrumbs > :last-child::after {
  content: ""; }
